﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection, TrackableItem   } from 'framework3/editors';
import { DomicilioInfo } from 'framework3/general';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Wijmo
import { Control } from 'wijmo/wijmo';

//Features
import { ContribuyenteService} from '../services/contribuyente.service';
import { ContribuyenteInfo, ContribuyenteItem, UsoCFDIItem} from '../models/contribuyente';
import { ComprobanteCorreo } from '../models/comprobante-correo'
import { ComprobantesCorreoEditor } from '../components/comprobante-correo.editor'

@Component({
    selector: 'contabilidad-contribuyentes-editor',
    templateUrl: './contribuyentes.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ContribuyentesEditor)
        },
        {
            provide: CatalogService, useExisting: ContribuyenteService
        }
    ]
})
export class ContribuyentesEditor extends CatalogEditor {
    contribuyente: ContribuyenteInfo = new ContribuyenteInfo();        
    usosCFDI: UsoCFDIItem[] = [];
    
    comprobantesCorreosCollection: TrackableCollection = new TrackableCollection();

    @ViewChild(ComprobantesCorreoEditor) comprobanteCorreoDialog: ComprobantesCorreoEditor;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private contribuyenteService: ContribuyenteService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'rfc': ['', Validators.required],
            'razonSocial': ['', Validators.required],
            'domicilio': [''],           
            'correo': [''],
            'cuenta': [''],
            'idUsoCFDI': [0]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////
    ngAfterViewChecked() {
        this.comprobantesCorreosCollection.sourceCollection = this.contribuyente.comprobantesCorreos;
    }
	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadUsosCFDI();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.contribuyenteService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.contribuyente = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.contribuyenteService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.contribuyente = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.contribuyenteService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.contribuyente = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.contribuyenteService.update(this.contribuyente)
            .then(info => {
                this.context.app.hideSpinner();
                this.contribuyente = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    loadUsosCFDI() {
        this.contribuyenteService.loadUsosCFDI(this.context.app.defaultCompany.identity)
            .then(usos => this.usosCFDI = usos)
            .catch(error => this.handleError(error));
    }

    domicilioChanged(domicilio: DomicilioInfo)
    {
        this.contribuyente.domicilio = domicilio;
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }


    //Correos
    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.comprobanteCorreoDialog.show();
        }
    }
    
    onAgregarComprobanteCorreo(args: ComprobanteCorreo) {
        let comprobanteExists: boolean = false;
        this.contribuyente.comprobantesCorreos.forEach(comproMail => {
            if (comproMail.idComprobanteTipo == args.idComprobanteTipo && comproMail.state != 'DELETED') {
                this.showAlert("El tipo de comprobante ya fue agregado", "danger");
                comprobanteExists = true;
            }
        })
        if (!comprobanteExists) {
            this.contribuyente.comprobantesCorreos.push(args);
            this.comprobantesCorreosCollection.refresh();
        }
    }

    onRemoveItem(item) {
        this.comprobantesCorreosCollection.remove(item);
        this.comprobantesCorreosCollection.refresh();
    }


}