﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { CuentaInfo, CuentaTipoItem } from '../models/cuenta';
import { CuentaService } from '../services/cuenta.service';
import { DivisaItem } from '../models/divisa';
import { DivisaService } from '../services/divisa.service';
import { SegmentoNegocioService } from '../services/segmento-negocio.service';
import { SegmentoNegocioItem } from '../models/segmento-negocio';

@Component({    
    selector: 'accounting-cuentas-manager',
    templateUrl: './cuentas.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CuentasManager)
        },
        {
            provide: CatalogService, useExisting: CuentaService
        }
    ]
})
export class CuentasManager extends HierarchyManager {
    cuentas: CuentaInfo[] = [];    
    cuentaEdicion: CuentaInfo = new CuentaInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    //
    cuentasTipo: CuentaTipoItem[] = [];
    divisas: DivisaItem[] = [];
    segmentosNegocio: SegmentoNegocioItem[] = [];

    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder,
        private cuentaService: CuentaService, private divisaService: DivisaService, protected router: Router, protected route: ActivatedRoute,
        private segmentoNegocioService: SegmentoNegocioService) {

        super(context, router, route, changeDetector);
       
        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idCuentaTipo': [0, AztecaValidators.selected],
            'idCategoriaFlujo': [0],
            'idDivisa': [0],
            'idSegmentoNegocio':[0],
            'afectable': [false],
            'agrupadorSAT': ['']
        });
    }

	////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.cuentaService.loadHierarchy(this.context.app.defaultCompany.identity,0)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.cuentas = nodes;
            })
            .catch(error => this.handleError(error));
    }


	//////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;        
        this.loadHierarchy();       
        //Hacer llamadas de carga inicial de datos aquí
        this.loadCuentastipos();
        //this.loadDivisas();
        this.loadSegmentosNegocio();
    }


    onNodesChanged(cuentas: CuentaInfo[]) {
        this.cuentas = cuentas;        
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.cuentaEdicion = new CuentaInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.cuentaEdicion = new CuentaInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.cuentaEdicion.idBranch = this.context.app.defaultSite.identity;
        this.cuentaEdicion.idCompany = this.context.app.defaultCompany.identity;
		//
        this.currentParent = parentNode;        
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {        
        //this.cuentaEdicion = this.cuentas.find(x => x.identity = node.data.identity)
        this.cuentaEdicion = this.cuentaService.copy(node.data);
        this.showEditor();        
    }

    onSavingNode() {        
        //Si es un nuevo nodo
        if (this.cuentaEdicion.clientID == 0) {
            this.addNode(this.cuentaEdicion, this.currentParent);            
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.cuentaEdicion);
        }
		this.cuentaEdicion = new CuentaInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<CuentaInfo>) {
        this.context.app.showSpinner();
        this.cuentaService.saveHierarchy(changes)
            .then(cuentas => {
                this.cuentas = cuentas;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));            
    }

    loadCuentastipos(){
        this.cuentaService.loadTipos(this.context.app.defaultCompany.identity.toString())
            .then(tipos => this.cuentasTipo = tipos)
        .catch(error => this.handleError(error));
    }

    loadDivisas() {
        this.divisaService.loadList(this.context.app.defaultCompany.identity)
            .then(divisas => this.divisas = divisas)
            .catch(error => this.handleError(error));
    }

    loadSegmentosNegocio() {
        this.segmentoNegocioService.loadList(this.context.app.defaultCompany.identity)
            .then(items => this.segmentosNegocio = items)
            .catch(error => this.handleError(error));
    }


}