﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { FormatoTipoService, FormatoItem } from 'framework3/general';
//Features
import { EmpresaService } from '../services/empresa.service';
import { EmpresaInfo, EmpresaItem, RegimenFiscalItem } from '../models/empresa';
import { SucursalItem } from '../models/sucursal';
import { SucursalService } from '../services/sucursal.service';
import { DivisaItem } from '../models/divisa';
import { DivisaService } from '../services/divisa.service';
import { PACItem } from '../models/pac';
import { PACService } from '../services/pac.service';
import { SerieFiscalService } from '../services/serie-fiscal.service';
import { SerieFiscalItem } from '../models/serie-fiscal';
import { ContribuyenteService } from '../services/contribuyente.service';
import { UsoCFDIItem } from '../models/contribuyente';

import { DomicilioInfo } from 'framework3/general';

import { Control } from 'wijmo/wijmo';


@Component({
    selector: 'contabilidad-empresas-editor',
    templateUrl: './empresas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EmpresasEditor)
        },
        {
            provide: CatalogService, useExisting: EmpresaService
        }
    ]
})
export class EmpresasEditor extends CatalogEditor {
    empresa: EmpresaInfo = new EmpresaInfo();
    @Input() sucursales: SucursalItem[] = [];
    pacs: DivisaItem[] = [];
    series: SerieFiscalItem[] = [];
    formatosCfdi: FormatoItem[] = [];
    regimenesFiscales: RegimenFiscalItem[] = [];
    usosCFDI: UsoCFDIItem[] = [];
    tiposCambio: { id: string, nombre: string }[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private empresaService: EmpresaService, private sucursalService: SucursalService,
        private divisaService: DivisaService, private pacService: PACService, private serieFiscalService: SerieFiscalService, private contribuyenteService: ContribuyenteService,
        private formatoService: FormatoTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'logo': [''],
            'codigo': [''],
            'nombre': ['', Validators.required],

            'idDivisa': [0, AztecaValidators.selected],
            'idPAC': [''],
            'formatoContable': [''],
            'idRegimenFiscal': [0, AztecaValidators.selected],
            'idRemitenteCompborantes': [0],            
            'domicilio': [''],

            'rfc': ['', Validators.required],
            'razonSocial': ['', Validators.required],
            'domicilioFiscal': ['', Validators.required],

            'idFormatoIngreso': [0],
            'idFormatoEgreso': [0],
            'idFormatoPago': [0],
            'idFormatoReciboNomina': [0],
            'tipoCambio': ['', Validators.required],
            'parametro': ['']
        });

        this.tiposCambio =
            [
                { "id": "C", "nombre": "a la compra" },
                { "id": "V", "nombre": "a la venta" },
                { "id": "O", "nombre": "otro" }
            ];
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    loadFormatosCfdi() {
        this.formatoService.loadFormatos('ACCOUNTING_CFDI', this.empresa.identity)
            .then(formatos => {
                this.formatosCfdi = formatos;
                this.formatosCfdi.unshift({ id: 0, nombre: '[NINGUNO]', activo: true, codigo: '' });
            })
            .catch(error => this.handleError(error));
    }

    loadUsosCFDI() {
        this.contribuyenteService.loadUsosCFDI(this.context.app.defaultCompany.identity)
            .then(usos => this.usosCFDI = usos)
            .catch(error => this.handleError(error));
    }

    loadRegimenesFiscales() {
        this.empresaService.loadRegimenesFiscales(this.context.app.defaultCompany.identity)
            .then(regimenes => this.regimenesFiscales = regimenes)
            .catch(error => this.handleError(error));
    }
    ///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadPACs();
        this.loadSeries();
        this.loadRegimenesFiscales();
    }

    onCreatingItem() {

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;
                this.loadSucursales();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.empresaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;
                this.loadSucursales();
                this.loadFormatosCfdi();
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.empresaService.update(this.empresa)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.empresa.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.empresa = info;
            })
            .catch(error => this.handleError(error));
    }


    //Manejo de Estados
    selectSucursalesTab(arg: any) {
        Control.invalidateAll();
    }

    loadSucursales() {
        let request: SearchRequest = new SearchRequest();
        request.paged = false;
        //Agregar condición de empresa actual
        let condicion: SearchCondition = { field: "IdEmpresa", operator: "=", valueType: "int", value: this.empresa.identity.toString() };
        request.conditions.push(condicion);
        //
        this.context.app.showSpinner();
        this.sucursalService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.sucursales = response.rows;
            })
            .catch(error => this.handleError(error));
    }

    loadPACs() {
        this.pacService.loadList()
            .then(pacs => this.pacs = pacs)
            .catch(error => this.handleError(error));
    }

    loadSeries() {
        this.serieFiscalService.loadList(this.context.app.defaultCompany.identity.toString())
            .then(series => this.series = series)
            .catch(error => this.handleError(error));
    }

    onDomicilioFiscalChanged(domicilio: DomicilioInfo) {
        this.empresa.contribuyente.domicilio = domicilio;
    }

    onDomicilioValidChange() {
        this.changeDetector.detectChanges();
    }

}