﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { RemitentesMailManager } from 'framework3/notification';

//Features
import { SucursalService} from '../services/sucursal.service';
import { SucursalInfo, SucursalItem} from '../models/sucursal';
import { SerieFiscalService } from '../../accounting/services/serie-fiscal.service';
import { SerieFiscalItem } from '../../accounting/models/serie-fiscal'
import { SegmentoNegocioService } from '../../accounting/services/segmento-negocio.service';
import { SegmentoNegocioItem } from '../../accounting/models/segmento-negocio'

import { Control } from 'wijmo/wijmo';


@Component({
    selector: 'contabilidad-sucursales-editor',
    templateUrl: './sucursales.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SucursalesEditor)
        },
        {
            provide: CatalogService, useExisting: SucursalService
        }
    ]
})
export class SucursalesEditor extends CatalogEditor {
    sucursal: SucursalInfo = new SucursalInfo();  
    idEmpresa: number;    
    series: SerieFiscalItem[] = [];
    segmentos: SegmentoNegocioItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private sucursalService: SucursalService, private serieFiscalService: SerieFiscalService,
        private segmentoNegocioService: SegmentoNegocioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'serieIngreso': [0],
            'serieCargo': [0],
            'serieCredito': [0],
            'seriePago': [0],
            'serieNomina': [0],
            'segmentoNegocio': [0],
            'domicilio': [''],
            'telefono1': [''],
            'telefono2': [''],
            'fax': [''],
            'paginaWeb': [''],
            'correo': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.route.params.forEach((params: Params) => {
            this.idEmpresa = +params['idEmpresa'];
        }); 
        this.loadSeries();
        this.loadSegmentos();
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        //Asignar y cargar datos de la empresa que dependerá la sucursal
        this.sucursal.idCompany = this.idEmpresa;
        this.sucursal.sucursalBase.idCompany = this.idEmpresa;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.sucursalService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.sucursal = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.sucursalService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.sucursal = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.sucursalService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.sucursal = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.sucursalService.update(this.sucursal)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.sucursal.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.location.back();
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.sucursal = info;
            })
            .catch(error => this.handleError(error));        
    }
    onCancelingEdit() {
        this.location.back();
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {            
            this.router.navigate(['kernel/security/sucursales/', this.process.id, this.idEmpresa, this.sucursal.identity,'members']);
        }
    }

    onSelectTab(arg: any) {
        Control.invalidateAll();
    }

    loadSeries() {
        this.serieFiscalService.loadList(this.context.app.defaultCompany.identity.toString())
            .then(series => this.series = series)
            .catch(error => this.handleError(error));
    }

    loadSegmentos() {
        this.segmentoNegocioService.loadList(this.context.app.defaultCompany.identity)
            .then(segmentos => this.segmentos = segmentos)
            .catch(error => this.handleError(error));
    }

    onDomicilioValidChange() {
        this.changeDetector.detectChanges();
    }

}