﻿//Angular
import { Component, forwardRef, Input, Output, EventEmitter } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
//Framework 3
import { BaseComponent, Context } from 'framework3/core';
import { CatalogSelector, SearchRequest, SearchCondition } from 'framework3/search';
//Reactive
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
import { map } from 'rxjs/operators';
//Features
import { ImpuestoItem, ImpuestoInfo } from '../models/impuesto';
import { ImpuestoService } from '../services/impuesto.service';

@Component({
    selector: 'accounting-impuestos-selector',
    templateUrl: 'impuestos-selector.component.html',
    providers: [
        {
            provide: BaseComponent,
            useExisting: forwardRef(() => ImpuestosSelectorComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => ImpuestosSelectorComponent),
            multi: true
        }
    ]
})

export class ImpuestosSelectorComponent extends CatalogSelector {
    //variables
    impuestos: any[];
    selectedItem: ImpuestoInfo = new ImpuestoInfo();

    //Inputs
    @Input() traslado: boolean = true;
    @Input() retencion: boolean = true;

    //Outputs
    @Output() selectedItemChanged = new EventEmitter<ImpuestoInfo>();

    constructor(protected context: Context, private impuestoService: ImpuestoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findImpuestos(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.impuestoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + '-' + info.nombre;
                    this.selectedItemChanged.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        if (!this.traslado || !this.retencion) {
            let condicionTraslado: SearchCondition = { field: "traslado", operator: "=", valueType: "boolean", value: this.traslado }
            let condicionRetencion: SearchCondition = { field: "retencion", operator: "=", valueType: "boolean", value: this.retencion }
            request.conditions.push(condicionTraslado);
            request.conditions.push(condicionRetencion);
        }

        this.context.app.showSpinner();        
        this.impuestoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();

                this.impuestos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(Error));
    }

    findImpuestos(keyword: string): Observable<ImpuestoItem[]> {
        return this.impuestoService.searchKeywordImpuestos(this.context.app.defaultSite.identity, keyword)
            .pipe(map(impuestos => {
                return impuestos.filter(impuesto => impuesto.traslado == this.traslado || impuesto.retencion == this.retencion);
            }));
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }
}