﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { EjercicioInfo, EjercicioItem } from '../models/ejercicio';
import { EjercicioDetail } from '../models/ejercicio-detail';
import { EjercicioService } from '../services/ejercicio.service';
import { EjercicioTipoItem } from '../models/ejercicio-tipo';
import { EjercicioTipoService } from '../services/ejercicio-tipo.service';
import * as moment from 'moment';

@Component({
    selector: 'contabilidad-ejercicios-editor',
    templateUrl: './ejercicios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EjerciciosEditor)
        },
        {
            provide: DocumentService, useExisting: EjercicioService
        }
    ]
})
export class EjerciciosEditor extends DocumentEditor {
    ejercicio = new EjercicioInfo();

    ejerciciosTipos: EjercicioTipoItem[]=[];
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    
    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private ejercicioService: EjercicioService,
        private ejercicioTipoService: EjercicioTipoService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'nombre': ['', Validators.required],
            'fechaIni': ['', Validators.required],
            'fechaFin': ['', Validators.required],
            'idEjercicioTipo': [0, AztecaValidators.selected],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {
        this.applyPolicies();
        this.loadEjerciciosTipos();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.ejercicioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.ejercicio = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.ejercicioService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.ejercicio = doc;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.ejercicioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.ejercicio = doc;
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.ejercicio.idEmpresa = this.context.app.defaultCompany.identity;
        this.ejercicio.fechaIni = moment(this.ejercicio.fechaIni).startOf('day').toDate();
        this.ejercicio.fechaFin = moment(this.ejercicio.fechaFin).startOf('day').toDate();
        this.ejercicioService.update(this.ejercicio)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.ejercicio.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.ejercicio = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }
    
    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.ejercicioService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ejercicioService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.ejercicio.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    //
    loadEjerciciosTipos() {
        this.ejercicioTipoService.loadList(this.context.app.defaultSite.identity)
            .then(tipos => this.ejerciciosTipos = tipos)
            .catch(error => this.handleError(error));

    }



}