﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { FichaInfo, FichaItem } from '../models/ficha';
import { FichaDetail } from '../models/ficha-detail';
import { FichaService } from '../services/ficha.service';
import { FichaDetailEditor } from './parts/fichas-detail.editor';

@Component({
    selector: 'contabilidad-fichas-editor',
    templateUrl: './fichas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FichasEditor)
        },
        {
            provide: DocumentService, useExisting: FichaService
        }
    ]
})
export class FichasEditor extends DocumentEditor {
    ficha = new FichaInfo();
    //
    afectadores: { id: string, nombre: string }[] = [];
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: FichaDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private fichaService: FichaService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'cuenta': [0],
            'afectador': ['', Validators.required],
            'observaciones': ['', Validators.required],
            'importe': [0, Validators.required],
            'afectacion': ['', Validators.required],
        });

        this.afectadores =
            [
                { "id": "C", "nombre": "Cargo" },
                { "id": "A", "nombre": "Abono" }
            ];

    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.fichaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.ficha = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.fichaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.ficha = doc;
                this.detailsEditor.clientIndexStart = this.ficha.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.fichaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.ficha = doc;
                this.detailsEditor.clientIndexStart = this.ficha.details.length;
                if (this.ficha.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.fichaService.update(this.ficha)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.ficha.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.ficha = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.fichaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.fichaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.ficha.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    /*
    selectedAfectador(afectador) {
    } 
    */

    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new FichaDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: FichaDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: FichaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: FichaDetail[]) {
        this.ficha.details = updatedDetails;
    }

    onSavingDetail(detail: FichaDetail) {
        this.detailsList.updateItem(detail);
    }

}

