﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control, CollectionView } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { NotaInfo, NotaItem } from '../models/nota';
import { NotaDetail } from '../models/nota-detail';
import { NotaRelaciones } from '../models/nota-relaciones';
import { NotaService } from '../services/nota.service';
import { NotaDetailEditor } from './parts/notas-detail.editor';
import { DivisaService } from '../services/divisa.service';
import { DivisaInfo, DivisaItem } from '../models/divisa';

import { FormaPagoService } from '../services/forma-pago.service';
import { FormaPagoItem } from '../models/forma-pago';

import { ComprobanteFiscalInfo, ComprobanteFiscalItem, ComprobanteFiscalSelectorComponent, ContribuyenteSelectorComponent } from '../../accounting';

import { ContribuyenteService } from '../services/contribuyente.service';
import { ContribuyenteInfo, ContribuyenteItem, UsoCFDIItem } from '../models/contribuyente';

@Component({
    selector: 'contabilidad-notas-editor',
    templateUrl: './notas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => NotasEditor)
        },
        {
            provide: DocumentService, useExisting: NotaService
        }
    ]
})
export class NotasEditor extends DocumentEditor {
    nota = new NotaInfo();
    divisaInfo = new DivisaInfo();
    formasPago: FormaPagoItem[] = [];
    usosCFDI: UsoCFDIItem[] = [];

    //

    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: NotaDetailEditor;
    @ViewChild('selectorComprobante') selectorComprobante: ComprobanteFiscalSelectorComponent;
    @ViewChild('selectorContribuyente') selectorContribuyente: ContribuyenteSelectorComponent;

    comprobantes: NotaRelaciones[] = [];
    currentComprobante: NotaRelaciones = new NotaRelaciones();
    comprobantesCollection: CollectionView = new CollectionView();
    divisaDefault: number = 0;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private notaService: NotaService,
        private formaPagoService: FormaPagoService,
        private divisaService: DivisaService,
        private contribuyenteService: ContribuyenteService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'contribuyente': [0, Validators.required],
            'divisa': [0, Validators.required],
            'observaciones': ['', Validators.required],
            'comprobante': [''],
            'idFormaPago': [0, Validators.required],
            'idUsoCFDI': [0, Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.loadFormasPago();
        this.applyPolicies();
        this.getDivisaDefault();
        this.loadUsosCFDI();
    }

    onCreatingDocument() {
        this.notaService.create().then(doc => {
            this.nota = doc;
            this.nota.idSucursal = this.context.app.defaultSite.identity;
            console.log("Creandon documebti");
        }) 
        .catch(error => this.handleError(error));
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.notaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nota = doc;
                this.listComprobantes();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.notaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nota = doc;
                this.detailsEditor.clientIndexStart = this.nota.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.notaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nota = doc;
                this.listComprobantes();
                this.detailsEditor.clientIndexStart = this.nota.details.length;
                if (this.nota.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.nota.origen = 'M';
        this.notaService.updateNota(this.nota)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.nota.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.nota = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.notaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.notaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.nota.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new NotaDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: NotaDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: NotaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: NotaDetail[]) {
        this.nota.details = updatedDetails;
    }

    onSavingDetail(detail: NotaDetail) {
        this.detailsList.updateItem(detail);
    }

    loadFormasPago() {
        this.formaPagoService.loadList(this.context.app.defaultSite.identity)
            .then(formas => this.formasPago = formas)
            .catch(error => this.handleError(error));
    }


    listComprobantes() {
        this.notaService.ListarComprobantes(this.nota.identity)
            .then(recs => {
                this.comprobantes = recs
                this.comprobantesCollection.sourceCollection = this.comprobantes;
            })
            .catch(error => this.handleError(error));
    }


    asignarComprobante(comprobante: ComprobanteFiscalInfo) {

        var rel: NotaRelaciones = new NotaRelaciones;

        rel.folio = +comprobante.folio;
        rel.serie = comprobante.serie;
        rel.total = comprobante.total;
        rel.uuid = comprobante.timbreFiscal.uuid;

        var newRel =  this.nota.relaciones.slice();

        newRel.push(rel);


        this.nota.relaciones = newRel;

        this.comprobantesCollection.sourceCollection = newRel;

        this.comprobantesCollection.refresh;

/*
        this.notaService.AgregarComprobante(this.nota.identity, comprobante.identity, uuid)
            .then(r => {
                this.listComprobantes();
            })
            .catch(error => this.handleError(error));
*/
    }

    getDivisaDefault() {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(r => {
                this.nota.idDivisa = r.identity; 
            })
            .catch(error => this.handleError(error));

    }


    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.asignarComprobante(this.selectorComprobante.selectedItem);
        }
    }

    onMenuComprobantesClicked(menu: Menu, args: any) {
        if (menu.selectedIndex == 0) {
            //Eliminar
            this.comprobantes = this.comprobantesCollection.currentItem;
            this.notaService.EliminarComprobante(this.comprobantesCollection.currentItem.idNotaRelacion)
                .then(r => {
                    this.comprobantesCollection.remove(this.comprobantes);
                })
                .catch(error => this.handleError(error));
        }

    }

    loadUsosCFDI() {
        this.contribuyenteService.loadUsosCFDI(this.context.app.defaultCompany.identity)
            .then(usos => this.usosCFDI = usos)
            .catch(error => this.handleError(error));
    }

    onContribuyenteSelected(contribuyente: ContribuyenteInfo) {
        this.nota.idUsoCFDI = contribuyente.idUsoCFDI;
    }


}

