﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { PeriodoInfo, PeriodoItem } from '../models/periodo';
import { PeriodoDetail } from '../models/periodo-detail';
import { PeriodoService } from '../services/periodo.service';
import { EjercicioService } from '../services/ejercicio.service';
import { EjercicioInfo } from '../models/ejercicio';
import * as moment from 'moment';

@Component({
    selector: 'contabilidad-periodos-editor',
    templateUrl: './periodos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PeriodosEditor)
        },
        {
            provide: DocumentService, useExisting: PeriodoService
        }
    ]
})
export class PeriodosEditor extends DocumentEditor {
    periodo = new PeriodoInfo();
    ejercicio = new EjercicioInfo();
    meses: { id: number, nombre: string }[] = [
        { "id": 1, "nombre": "Enero" },
        { "id": 2, "nombre": "Febrero" },
        { "id": 3, "nombre": "Marzo" },
        { "id": 4, "nombre": "Abril" },
        { "id": 5, "nombre": "Mayo" },
        { "id": 6, "nombre": "Junio" },
        { "id": 7, "nombre": "Julio" },
        { "id": 8, "nombre": "Agosto" },
        { "id": 9, "nombre": "Septiembre" },
        { "id": 10, "nombre": "Octubre" },
        { "id": 11, "nombre": "Noviembre" },
        { "id": 12, "nombre": "Diciembre" }
    ]

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private periodoService: PeriodoService,
        private ejercicioService: EjercicioService,
    ) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'meses': ['', Validators.required],
            'fechaIni': ['', Validators.required],
            'fechaFin': ['', Validators.required]
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.ejercicioService.getDetail(params['idEjercicio'])
                .then(x => {
                    this.ejercicio = x;
                    this.ejercicio.fechaIni = moment(this.ejercicio.fechaIni).toDate();
                    this.ejercicio.fechaFin = moment(this.ejercicio.fechaFin).toDate();
                    this.periodo.idEjercicio = x.identity;
                    this.periodo.fechaEjercicio = x.fechaSys;
                })
                .catch(error => this.handleError(error))
        });
    }

    onCancelingEdit() {
        this.router.navigate(['accounting/ejercicios/' + this.process.id + '/' + this.ejercicio.identity]);
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.periodoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.periodo = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.periodoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.periodo = doc;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.periodoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.periodo = doc;
                if (this.periodo.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.periodo.fechaIni = moment(this.periodo.fechaIni).startOf('day').toDate();
        this.periodo.fechaFin = moment(this.periodo.fechaFin).startOf('day').toDate();
        if (this.periodo.fechaIni >= this.ejercicio.fechaIni && this.periodo.fechaIni <= this.ejercicio.fechaFin && this.periodo.fechaFin >= this.periodo.fechaIni) {
            this.periodoService.update(this.periodo)
                .then(updatedDoc => {
                    this.context.app.hideSpinner();
                    //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                    this.reset();
                    if (!args.closeEditor && this.periodo.identity == '') {
                        this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                    }
                    else {
                        if (args.closeEditor)
                            this.router.navigate(['../'], { relativeTo: this.route });
                        else
                            this.showAlert('La información ha sido actualizada con éxito', 'success');
                    }
                    this.periodo = updatedDoc;
                })
                .catch(error => this.handleError(error));
        } else {
            this.context.app.hideSpinner();
            this.showAlert('Las fechas deben de estar entre el rango de fechas del ejercicio', 'warning');
        }
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.periodoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.periodoService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.periodo.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

}