﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef} from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, SucursalService as SucursalKernelService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { EmpresaService} from '../services/empresa.service';
import { EmpresaInfo, EmpresaItem} from '../models/empresa';

import { SucursalItem } from '../models/sucursal';
import { SucursalService } from '../services/sucursal.service';
import { CobroTipoService } from '../services/cobro-tipo.service';
import { CobroTipoItem } from '../models/cobro-tipo';

import { Control } from 'wijmo/wijmo';

@Component({
    selector: 'credito-empresa-editor',
    templateUrl: './empresas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EmpresasEditor)
        },
        {
            provide: CatalogService, useExisting: EmpresaService
        }
    ]
})
export class EmpresasEditor extends CatalogEditor {
    empresa: EmpresaInfo = new EmpresaInfo(); 
    @Input() sucursales: SucursalItem[] = [];    
    cobrosTipos: CobroTipoItem[] = [];
    tiposBloqueo: { id: string, nombre: string }[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private empresaService: EmpresaService, private sucursalService: SucursalService,
        private sucursalKernelService: SucursalKernelService, private cobroTipoService: CobroTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idImpuestoAnticipos': [0],
            'idCobroTipoRedondeo': [0],
            'cobranzaRedondeo': [0],
            'idCobroTipoRedondeoComprobante':[0],
            'idConceptoItemRedondeoComprobante': [0],
            'tipoBloqueoDeudores': ['N', Validators.required],
            'diasBloqueoPorPago':[0]
         });

        this.tiposBloqueo =
            [
                { "id": 'N', "nombre": "No aplica" },
                { "id": 'L', "nombre": "Limite de crédito" },
                { "id": 'P', "nombre": "Falta de pago(dias)" }
            ];
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadCobrosTipos();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;   
                this.loadSucursales();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.empresaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;         
                this.loadSucursales();
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.empresaService.update(this.empresa)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.empresa.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.empresa = info;
            })
            .catch(error => this.handleError(error));        
    }

    //
    //Manejo de Estados
    selectTab(arg: any) {
        Control.invalidateAll();
    }

    loadSucursales() {
        let request: SearchRequest = new SearchRequest();
        request.paged = false;
        //Agregar condición de empresa actual
        let condicionEmpresa: SearchCondition = { field: "idEmpresa", operator: "=", valueType: "int", value: this.empresa.identity.toString() };
        request.conditions.push(condicionEmpresa);
        //
        this.context.app.showSpinner();
        this.sucursalKernelService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.sucursales = response.rows;
            })
            .catch(error => this.handleError(error));
    }

    loadCobrosTipos() {
        this.cobroTipoService.loadList(this.context.app.defaultSite.identity, true)
            .then(cobrosTipos => this.cobrosTipos = cobrosTipos)
            .catch(error => this.handleError(error));
    }

}