﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { DepositoInfo, DepositoItem } from '../models/deposito';
import { DepositoDetail } from '../models/deposito-detail';
import { DepositoTipoItem } from '../models/deposito-tipo';
import { DepositoTipoService } from '../services/deposito-tipo.service';

import { DepositoService } from '../services/deposito.service';
import { DepositoDetailEditor } from './parts/depositos-detail.editor';
import { CuentaBancariaItem } from '../../accounting/models/cuenta-bancaria';
import { CuentaBancariaService } from '../../accounting/services/cuenta-bancaria.service';
import { moment } from 'ngx-bootstrap/chronos/test/chain';

@Component({
    selector: 'credito-depositos-editor',
    templateUrl: './depositos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DepositosEditor)
        },
        {
            provide: DocumentService, useExisting: DepositoService
        }
    ]
})
export class DepositosEditor extends DocumentEditor {
    deposito = new DepositoInfo();
    depositosTipos: DepositoTipoItem[] = [];    
    cuentasBancarias: CuentaBancariaItem[] = [];
    tipoOrdenante: string;
    tiposOrdenantes: { id: string, nombre: string }[] = [];

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: DepositoDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private depositoService: DepositoService, private depositoTipoService: DepositoTipoService, private cuentaBancariaService: CuentaBancariaService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idDepositoTipo': ['', Validators.required],
            'idCuentaBancaria': ['', Validators.required],
            'ordenante': ['', Validators.required],
            'idGrupoDeudor': [''],
            'idDeudor': [''],
            'monto': ['', Validators.required],
            'observaciones': [''],
        });

        this.tiposOrdenantes =
            [
                { "id": "G", "nombre": "Grupo Deudor" },
                { "id": "D", "nombre": "Deudor" },                
            ];

    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.loadDepositosTipos();
        this.loadCuentasBancarias();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.depositoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.deposito = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.depositoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.deposito = doc;
                this.detailsEditor.clientIndexStart = this.deposito.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();        
        this.depositoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.tipoOrdenante = doc.idDeudor > 0 ? 'D' : 'G';
                this.deposito = doc;                
                this.detailsEditor.clientIndexStart = this.deposito.details.length;
                if (this.deposito.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();        
        this.deposito.fechaAfectacion = new Date();
        this.depositoService.update(this.deposito)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.deposito.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.deposito = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        if (args.step.tag == 'APPLY') {
            this.context.app.hideSpinner();
            this.showAlert('El documento solo se aplica desglosando el cobro en el proceso de cobranza', 'warning');
        }
        else {
            this.depositoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
                .then(_ => {
                    //Volver a cargar documento
                    this.onViewingDocument(args.document.id);
                })
                .catch(error => this.handleError(error));
        }
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();
        if (document.workflowStep == 'APPLY') {
            this.context.app.hideSpinner();
            this.showAlert('El documento en estatus aplicado solo puede ser cancelado en el proceso de cobranza', 'warning');
        }
        else
            this.depositoService.previousStep(document.id, null)
                .then(_ => {
                    //Volver a cargar documento
                    this.onViewingDocument(document.id);
                })
                .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.deposito.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new DepositoDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: DepositoDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: DepositoDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: DepositoDetail[]) {
        this.deposito.details = updatedDetails;
    }

    onSavingDetail(detail: DepositoDetail) {
        this.detailsList.updateItem(detail);
    }

    ///////
    loadDepositosTipos() {
        this.depositoTipoService.loadList(this.context.app.defaultSite.identity)
            .then(depositosTipos => this.depositosTipos = depositosTipos)
            .catch(error => this.handleError(error));
    }

    loadCuentasBancarias() {
        this.cuentaBancariaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cuentasBeneficiario => {
                this.cuentasBancarias = cuentasBeneficiario;
            })
            .catch(error => this.showAlert(error, 'danger'));
    }

    selectedOrdenante(item: any) {
        this.tipoOrdenante = item;        
    }

}

