﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Wijmo
import { Control } from 'wijmo/wijmo';

//Features
import { RutasPreventaService } from '../services/rutas-preventa.service';
import { RutasPreventaInfo, RutasPreventaItem, RepartoDiarioItem } from '../models/rutas-preventa';
import { RutasRepartoInfo, RutasRepartoItem } from '../models/rutas-reparto';
import { RutasRepartoSelectorComponent } from '../controls/ruta-reparto-selector.component';
import { RutasRepartoListComponent } from '../dialogs/ruta-reparto-list.component';
import { FlexGrid } from 'wijmo/wijmo.grid';

@Component({
    selector: 'distribucion-rutas-preventa-editor',
    templateUrl: './rutas-preventa.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RutasPreventaEditor)
        },
        {
            provide: CatalogService, useExisting: RutasPreventaService
        }
    ]
})
export class RutasPreventaEditor extends CatalogEditor {
    rutasPreventa: RutasPreventaInfo = new RutasPreventaInfo();
    tipoReparto: { id: string, nombre: string }[] = [];
    mostrarReparto: boolean = false;

    repartoDiaCurrent: RepartoDiarioItem;

    @ViewChild('rutaRepartoSelector') rutaRepartoSelector: RutasRepartoSelectorComponent;
    @ViewChild('grdRutasDiarias') grdRutasDiarias: FlexGrid;
    
    @ViewChild('rutasRepartoList') rutasRepartoList: RutasRepartoListComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private rutasPreventaService: RutasPreventaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'estVen': ['', AztecaValidators.selected],
            'clienteGenerico': ['', AztecaValidators.selected],
            'idDispositivoCaptura': ['', AztecaValidators.selected],
            'idVehiculo': ['', AztecaValidators.selected],
            'idCuenta': [''],
            'idSegmento': ['', AztecaValidators.selected],
            'idGama': ['', AztecaValidators.selected],
            'idRutaReparto': ['', AztecaValidators.selected],
            'tipoReparto': ['', Validators.required]
        });

        this.tipoReparto = [
            { "id": "F", "nombre": "Fija" },
            { "id": "D", "nombre": "Por dia" },
            { "id": "I", "nombre": "Inferida" },
            { "id": "S", "nombre": "Inteligente" }
        ];
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
    }

    onCreatingItem() {

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.rutasPreventaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.rutasPreventa = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.rutasPreventaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.rutasPreventa = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.rutasPreventaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.rutasPreventa = info;
                this.onTipoRepartoChanged(this.rutasPreventa.tipoReparto);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.rutasPreventaService.update(this.rutasPreventa)
            .then(info => {
                this.context.app.hideSpinner();
                this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.rutasPreventa.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.rutasPreventa = info;
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    onTipoRepartoChanged(args) {
        if (args == 'F') {
            this.mostrarReparto = true;
            this.catalogForm.controls.idRutaReparto.setValidators(AztecaValidators.selected);
            this.catalogForm.controls.idRutaReparto.updateValueAndValidity();
        }
        if (args == 'D') {
            this.mostrarReparto = false;
            if (this.rutaRepartoSelector != undefined)
                this.rutaRepartoSelector.clear();
            this.catalogForm.controls.idRutaReparto.setValidators(null);
            this.catalogForm.controls.idRutaReparto.updateValueAndValidity();
            this.loadRepartoDiarioDefault();
        }
        if (args == 'I') {
            this.mostrarReparto = false;
            if (this.rutaRepartoSelector != undefined)
                this.rutaRepartoSelector.clear();
            this.catalogForm.controls.idRutaReparto.setValidators(null);
            this.catalogForm.controls.idRutaReparto.updateValueAndValidity();
        }
        if (args == 'S') {
            this.mostrarReparto = false;
            if (this.rutaRepartoSelector != undefined)
                this.rutaRepartoSelector.clear();
            this.catalogForm.controls.idRutaReparto.setValidators(null);
            this.catalogForm.controls.idRutaReparto.updateValueAndValidity();
        }
    }

    loadRepartoDiarioDefault() {
        if (this.rutasPreventa.repartoDiario && this.rutasPreventa.repartoDiario.length == 0) {
            this.rutasPreventa.repartoDiario= [ { dia: 1, diaText: "Lunes", idRutaReparto: 0, rutaReparto: "" },
                                                { dia: 2, diaText: "Martes", idRutaReparto: 0, rutaReparto: "" },
                                                { dia: 3, diaText: "Miercoles", idRutaReparto: 0, rutaReparto: "" },
                                                { dia: 4, diaText: "Jueves", idRutaReparto: 0, rutaReparto: "" },
                                                { dia: 5, diaText: "Viernes", idRutaReparto: 0, rutaReparto: "" },
                                                { dia: 6, diaText: "Sabado", idRutaReparto: 0, rutaReparto: "" },
                                                { dia: 7, diaText: "Domingo", idRutaReparto: 0, rutaReparto: "" } ];
        }
    }

    onRowButtonClick(repartoDia: RepartoDiarioItem) {
        this.repartoDiaCurrent = repartoDia;
        this.rutasRepartoList.show();
    }

    onRutaRepartoSelected(rutaReparto: RutasRepartoItem) {
        this.repartoDiaCurrent.idRutaReparto = rutaReparto.id;
        this.repartoDiaCurrent.rutaReparto = rutaReparto.codigo;
        this.grdRutasDiarias.refresh();
    }


    rutaRepartoDiarioChanged(rutaRep: RutasRepartoInfo, repartoDia: any)
    {
        repartoDia.item.idRutaReparto = rutaRep.identity;
    }


}