﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { TripulacionInfo, TripulacionItem } from '../models/tripulacion';
import { TripulacionDetail } from '../models/tripulacion-detail';
import { TripulacionService } from '../services/tripulacion.service';
import { TripulacionDetailEditor } from './parts/tripulaciones-detail.editor';

import { RutasRepartoSelectorComponent } from '../controls/ruta-reparto-selector.component';
import { RutasPreventaSelectorComponent } from '../controls/ruta-preventa-selector.component';
import { RutasRepartoInfo } from '../models/rutas-reparto';
import { RutasPreventaInfo } from '../models/rutas-preventa';


@Component({
    selector: 'distribucion-tripulaciones-editor',
    templateUrl: './tripulaciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TripulacionesEditor)
        },
        {
            provide: DocumentService, useExisting: TripulacionService
        }
    ]
})
export class TripulacionesEditor extends DocumentEditor {
    tripulacion = new TripulacionInfo();

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: TripulacionDetailEditor;


    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private tripulacionService: TripulacionService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idDispositivo': [0],
            'idVehiculo':[0]
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {
        this.route.params.forEach(params => {
            if (params['idRuta']) {
                this.tripulacion.idRuta = +params['idRuta'];
            }
            if (params['tipoTripulacion']) {
                this.tripulacion.tipoTripulacion = params['tipoTripulacion'];
            }
            if (params['idVehiculo']) {
                this.tripulacion.idVehiculo = +params['idVehiculo'];
            }
            if (params['idDispositivo']) {
                this.tripulacion.idDispositivoCaptura = +params['idDispositivo'];
            }

        });
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.tripulacionService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.tripulacion = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.tripulacionService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.tripulacion = doc;
                this.detailsEditor.clientIndexStart = this.tripulacion.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.tripulacionService.getDetail(id)
            .then(doc => {

                this.context.app.hideSpinner();
                this.tripulacion = doc;
                this.detailsEditor.clientIndexStart = this.tripulacion.details.length;
                if (this.tripulacion.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();

        this.tripulacionService.update(this.tripulacion)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.tripulacion.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.tripulacion = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.tripulacionService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.tripulacionService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.tripulacion.identity);
            }
            if (args.heading == 'Detalle' && this.tripulacion.identity == '') {
                this.tripulacionService.detailNewDocument(this.tripulacion.tipoTripulacion, this.context.app.defaultSite.identity)
                    .then(ret => {
                        this.tripulacion.details = ret;
                    });
            }

            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        //this.detailsEditor.detail = new TripulacionDetail();
        //this.detailsEditor.show();
    }

    onEditingDetail(detail: TripulacionDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: TripulacionDetail) {
        detail.idEmpleado = 0;
        detail.nombreEmpleado = '';
        detail.codigoEmpleado = '';
        detail.base = false;
        Control.invalidateAll();
        detail.state = "DELETED";
        //this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: TripulacionDetail[]) {
        this.tripulacion.details = updatedDetails;
    }

    onSavingDetail(detail: TripulacionDetail) {
        this.detailsList.updateItem(detail);
    }



}

