﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';

import * as _ from 'lodash';

//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { ValeInfo, ValeItem } from '../models/vale';
import { ValeDetail } from '../models/vale-detail';
import { ValeService } from '../services/vale.service';
import { ValeDetailEditor } from './parts/vales-detail.editor';

import { ImportesInfo } from '../../accounting/models/importes';
import { ValeTipoItem, ValeTipoInfo } from '../models/vale-tipo'

import { ClienteInfo, ClienteRutasItem } from '../models/cliente';
import { ClienteService } from '../services/cliente.service';
import { RutasPreventaItem } from '../models/rutas-preventa';
import { RutasRepartoItem } from '../models/rutas-reparto';

import { ValeTipoService } from '../services/vale-tipo.service'
import { Empaquetado } from '../../inventory';

import { TipoCapturaService } from '../services/tipo-captura.service'

import { DivisaService } from '../../accounting/services/divisa.service';

import * as moment from 'moment';


@Component({
    selector: 'distribucion-vales-editor',
    templateUrl: './vales.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ValesEditor)
        },
        {
            provide: DocumentService, useExisting: ValeService
        }
    ]
})
export class ValesEditor extends DocumentEditor {
    vale = new ValeInfo();
    tiposVales: ValeTipoItem[] = [];
    //
    get importes() {
        return this.importesDesglosados();
    }

    rutasItems: ClienteRutasItem[] = [];
    rutasPreventa: RutasPreventaItem[] = [];
    rutasReparto: RutasRepartoItem[] = [];
    idAlmacenCarga: number;


    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: ValeDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private valeService: ValeService,
        private valeTipoService: ValeTipoService,
        private clienteService: ClienteService,
        private tipoCapturaService: TipoCapturaService,
        private divisaService: DivisaService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idCliente': [0, AztecaValidators.selected],
            'idValeTipo': [0, Validators.required],
            'idRutaPreventa': [0, AztecaValidators.selected],
            'idRutaReparto': [0, AztecaValidators.selected],
            'fechaEntrega': [new Date, (control) => this.validateFecha(control)],
            'observaciones': [''],
        });

        //Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        this.documentForm.controls['idRutaPreventa'].valueChanges.subscribe(value => {
            setTimeout(_ => this.loadRutasReparto(value));
        });

        //Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        this.documentForm.controls['idRutaReparto'].valueChanges.subscribe(value => {
            setTimeout(_ => this.onRutaRepartoChanged(value));
        });

    }


    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.loadTiposVales();
        this.applyPolicies();
        this.setTipoCaptura();
        this.getDivisaDefault()
    }

    onCreatingDocument() {
        this.vale.fechaEntrega = moment(new Date).utc().toDate();
        this.vale.fechaEntrega.setHours(0);
        this.vale.fechaEntrega.setMinutes(0);
        this.vale.fechaEntrega.setSeconds(0);

        this.vale.horaInicio = moment(new Date).utc().toDate();

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.valeService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.vale = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.valeService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.vale = doc;
                this.detailsEditor.clientIndexStart = this.vale.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.valeService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.vale = doc;

                this.loadRutas(this.vale.idCliente);

                this.vale.details.forEach(det => {
                    let paquete = new Empaquetado(det.cantidad, det.producto.articulo.unidadMedida.factor);
                    det.cajas = paquete.cantidad;
                    det.botellas = paquete.unidades;
                });

                setTimeout(_ => this.loadRutasReparto(doc.idRutaPreventa), 1000);
                setTimeout(_ => this.onRutaRepartoChanged(doc.idRutaReparto), 2000);

                this.detailsEditor.clientIndexStart = this.vale.details.length;
                if (this.vale.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {

        //esperar wait
        this.validateDetail()
            .then(value => {
                if (value == true) {
                    this.context.app.showSpinner();
                    this.valeService.update(this.vale)
                        .then(updatedDoc => {
                            this.context.app.hideSpinner();
                            //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                            this.reset();
                            if (!args.closeEditor && this.vale.identity == '') {
                                this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                            }
                            else {
                                if (args.closeEditor)
                                    this.router.navigate(['../'], { relativeTo: this.route });
                                else
                                    this.showAlert('La información ha sido actualizada con éxito', 'success');
                            }
                            this.vale = updatedDoc;
                            this.vale.details.forEach(det => {
                                let paquete = new Empaquetado(det.cantidad, det.producto.articulo.unidadMedida.factor);
                                det.cajas = paquete.cantidad;
                                det.botellas = paquete.unidades;
                            });
                        })
                        .catch(error => this.handleError(error));
                }
            })
            .catch(error => this.handleError(error));

    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.valeService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.valeService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.vale.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        if (this.vale.idCliente > 0) {
            this.detailsEditor.detail = new ValeDetail();
            this.detailsEditor.show();
        }
        else
            this.showAlert("Debe seleccionar un cliente para poder agregar productos", "warning");
    }

    onEditingDetail(detail: ValeDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: ValeDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: ValeDetail[]) {
        this.vale.details = updatedDetails;
    }

    onSavingDetail(detail: ValeDetail) {
        this.detailsList.updateItem(detail);
    }

    ///metodos locales
    importesDesglosados(): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();

        let elementos = this.vale.details.filter(item => item.state != 'DELETED');//this.combineDetail();

        if (elementos.length > 0)
            this.vale.totalCantidad = elementos.map(item => item.cantidad).reduce((a, b) => a + b);

        elementos.forEach(item => {
            desglose.subtotal += item.precioUnitario * item.cantidad;
            desglose.descuentos += (item.descuentos.descuento1 + item.descuentos.descuento2 + item.descuentos.descuento3 + item.descuentos.descuento4 + item.descuentos.descuento5) * item.cantidad;
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "T").length > 0)
                desglose.impuestosTrasladados += item.impuestos.filter(tras => tras.tipoAplicacion == "T").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "R").length > 0)
                desglose.impuestosRetenidos += item.impuestos.filter(tras => tras.tipoAplicacion == "R").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
        });

        desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;


        return desglose;
    }    

    loadTiposVales() {
        this.valeTipoService.getTiposValeCbo(this.context.app.defaultSite.identity)
            .then(tipos => this.tiposVales = tipos)
            .catch(error => {
                this.handleError(error)
            });
    }

    clienteSelectedChanged(cliente: ClienteInfo) {
        if (cliente) {
            this.vale.cliente = cliente;
            this.vale.idListaPrecio = cliente.clienteVentas.idListaPrecios;
            this.loadRutas(cliente.identity);
            if (this.vale.identity == '')
                this.vale.details = [];
        }
    }

    loadRutas(idCliente: number) {
        this.clienteService.getRutas(idCliente)
            .then(rutas => {
                //
                this.rutasItems = rutas;

                //Agrupamos y cargamos Rutas de Preventa
                var result = _(this.rutasItems).groupBy('idRutaPreventa').transform(function (result, current) {
                    result.push({
                        id: current[0].idRutaPreventa,
                        nombre: current[0].rutaPreventa,
                    });
                }, []).value();

                this.rutasPreventa = result;
            })
            .catch(error => this.handleError(error));
    }

    loadRutasReparto(idRutaPreventa: number) {
        var result = _(this.rutasItems.filter(item => item.idRutaPreventa == idRutaPreventa)).groupBy('idRutaReparto').transform(function (result, current) {
            result.push({
                id: current[0].idRutaReparto,
                nombre: current[0].rutaReparto,
                idAlmacenCarga: current[0].idAlmacenCarga,
            });
        }, []).value();

        this.rutasReparto = result;
    }

    setTipoCaptura() {
        this.tipoCapturaService.getDetailByCode(this.context.app.defaultSite.identity, 'MANUAL')
            .then(tipo => {
                this.vale.idTipoCaptura = tipo.identity;
            })
            .catch(error => this.handleError(error));
    }

    onRutaRepartoChanged(idRutaReparto: number) {
        let rutaRep: RutasRepartoItem = this.rutasReparto.find(rr => rr.id == idRutaReparto)
        if (rutaRep)
            this.idAlmacenCarga = rutaRep.idAlmacenCarga;
    }


    validateFecha(control: FormControl) {
        let FechaHoy: Date;
        FechaHoy = moment(new Date).hour(0).minute(0).second(0).toDate();

        if (control.value) {
            let Fecha = control.value;
            Fecha = moment(Fecha).hour(0).minute(0).second(0).toDate();
            if (Fecha.getTime() >= FechaHoy.getTime())
                return null;
            else
                return 'La fecha no debe ser anterior a hoy';
        }
    }

    getDivisaDefault() {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(r => {
                this.vale.idDivisa = r.identity;
            })
            .catch(error => this.handleError(error));

    }


    async validateDetail() {
        return <boolean>await this.checkDetails();
    };

    checkDetails() {
        let result: boolean = true;

        //validamos que todos los articulos tengan serie en caso de que se maneje
        if (this.vale.details.filter(det => det.state != 'DELETED').length == 0) {
            this.showAlert("Debe calcular al menos una partida de productos al vale", 'warning');
            result = false;
        }

        return result;
    }

}

