﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { Clasificacion } from 'framework3/general';
//Features
import { ArticuloStockService } from '../services/articulo-stock.service';
import { ArticuloStockInfo, ArticuloStockItem } from '../models/articulo-stock';
import { UnidadMedidaService } from '../services/unidad-medida.service';
import { UnidadMedidaInfo, UnidadMedidaItem } from '../models/unidad-medida';
import { UnidadMedidaTipoService } from '../services/unidad-medida-tipo.service';
import { UnidadMedidaTipoItem } from '../models/unidad-medida-tipo';
import { MetodoValuacion } from '../models/metodo-valuacion';

@Component({
    selector: 'inventory-articulos-stock-editor',
    templateUrl: './articulos-stock.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticulosStockEditor)
        },
        {
            provide: CatalogService, useExisting: ArticuloStockService
        }
    ]
})
export class ArticulosStockEditor extends CatalogEditor {
    articuloStock: ArticuloStockInfo = new ArticuloStockInfo();
    tiposUnidad: UnidadMedidaTipoItem[] = [];
    unidadesMedida: UnidadMedidaItem[] = [];
    clasificadores: Clasificacion[] = [];
    metodosValuacion = MetodoValuacion.list;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, formBuilder: FormBuilder,
        protected context: Context, private location: Location, private articuloStockService: ArticuloStockService,
        private unidadMedidaService: UnidadMedidaService, private tipoUnidadService: UnidadMedidaTipoService,
        protected changeDetector: ChangeDetectorRef) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'gtin': [''],
            'idTipoUnidad': [0],
            'idUnidadMedida': [0],
            'categoriaFiscal': [0, Validators.required],
            'metodoValuacion': [''],
            'manejaSeries': [false],
            'manejaCaducidad': [false],
            'manejaValuacion': [false],
            'manejaLotes': [false],
            'descripcion': [''],
            'clasificaciones': [[], AztecaValidators.validClasifications]
        });
        //Hacer requerido el método de valuación sólo cuando se manejen lotes
        this.catalogForm.controls['manejaLotes'].valueChanges.subscribe(value => {
            if (value == true) {
                this.catalogForm.controls['metodoValuacion'].setValidators(Validators.required);
            }
            else {
                this.catalogForm.controls['metodoValuacion'].setValue('');
                this.catalogForm.controls['metodoValuacion'].setValidators(null);
            }
            this.catalogForm.controls['metodoValuacion'].updateValueAndValidity();
        });
        //Si se manejan series, la unidad de medida no será requerida
        this.catalogForm.controls['manejaSeries'].valueChanges.subscribe(value => {
            if (value) {
                this.catalogForm.controls['idUnidadMedida'].setValidators(null);
            }
            else {
                this.catalogForm.controls['idUnidadMedida'].setValidators(AztecaValidators.selected);
            }
            this.catalogForm.controls['idUnidadMedida'].updateValueAndValidity();            
        });
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        //Cargar catálogos
        this.loadClasificadores();
        this.loadTiposUnidad();
    }

    ///////////// METODOS /////////////////////////////////
    loadClasificadores() {
        this.articuloStockService.loadClassifiers(this.context.app.defaultSite.identity)
            .then(clasificadores => this.clasificadores = clasificadores)
            .catch(error => this.handleError(error));
    }

    loadTiposUnidad() {
        this.tipoUnidadService.loadList()
            .then(tipos => this.tiposUnidad = tipos)
            .catch(error => this.handleError(error));
    }

    onTipoUnidadChanged(idTipoUnidad) {
        this.loadUnidadesMedida(idTipoUnidad);
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        this.unidadMedidaService.loadList(idTipoUnidad)
            .then(unidades => this.unidadesMedida = unidades)
            .catch(error => this.handleError(error));
    }

    ///////////// HANDLERS /////////////////////////////////

    onCreatingItem() {
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.articuloStockService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.articuloStock = info;
                this.setReadonlyMode();
                this.loadUnidadesMedida(info.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.articuloStockService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.articuloStock = info;
                this.loadUnidadesMedida(info.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.articuloStockService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.articuloStock = info;
                this.loadUnidadesMedida(info.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.articuloStockService.update(this.articuloStock)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.articuloStock.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.articuloStock = info;
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'CONFIG') {
            this.router.navigate(['inventory/articulos-stock-configuracion/' + this.process.id + '/' + this.articuloStock.identity.toString()]);
        }
        else if (item.name == 'PICS') {
            this.router.navigate(['inventory/articulos-fotografias/' + this.process.id + '/' + this.articuloStock.identity.toString()]);
        }
        else if (item.name == 'STOCK') {
            this.router.navigate(['inventory/articulos-existencias/' + this.process.id + '/' + this.articuloStock.identity.toString()]);
        }
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

}