﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton, AzComboboxComponent } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { MetaAlmacenService} from '../services/meta-almacen.service';
import { MetaAlmacenInfo, MetaAlmacenItem } from '../models/meta-almacen';
import { MetaAlmacenSucursalItem } from '../models/metaalmacen-sucursal-item';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
//Inventario
import { AreaItem } from '../models/area';
import { AlmacenItem } from '../models/almacen';
import { AreaService } from '../services/area.service';
import { AlmacenService } from '../services/almacen.service';
//import { AsignarAlmacen } from '../parts/almacen-asignar.component';


@Component({
    selector: 'inventario-meta-almacen-editor',
    templateUrl: './meta-almacenes.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => MetaAlmacenesEditor)
        },
        {
            provide: CatalogService, useExisting: MetaAlmacenService
        }
    ]
})
export class MetaAlmacenesEditor extends CatalogEditor {
    metaAlmacen: MetaAlmacenInfo = new MetaAlmacenInfo();       
    idArea: number;
    idAlmacen: number;
    areas: AreaItem[] = [];
    almacenes: AlmacenItem[] = [];

    almacenesCollection: CollectionView = new CollectionView();

    @ViewChild('cboAreas') cboAreas: AzComboboxComponent;
    @ViewChild('cboAlmacenes') cboAlmacenes: AzComboboxComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private metaAlmacenService: MetaAlmacenService, private areaService: AreaService, private almacenService: AlmacenService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idArea': [0],
            'idAlmacen': [0]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////
    private loadAreas() {
        this.context.app.showSpinner();
        this.areaService.loadListByUser(this.context.app.defaultSite.identity, this.context.session.user.identity)
            .then(areas => {
                this.areas = areas;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));

    }

    private loadAlmacenes(idArea: number) {
        this.idAlmacen = 0;
        this.context.app.showSpinner();
        this.almacenService.loadListByUser(idArea, this.context.session.user.identity)
            .then(almacenes => {
                this.almacenes = almacenes.filter(almacen => almacen.salida);
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadAreas();
    }

    onCreatingItem() {

        this.metaAlmacenService.getMetaAlmacenesSucursales(0, this.context.app.defaultCompany.identity)
            .then(almacenes => {
                this.metaAlmacen.almacenesSucursal = almacenes;
                this.almacenesCollection.sourceCollection = this.metaAlmacen.almacenesSucursal;
                this.almacenesCollection.refresh;
            })
            .catch(error => this.handleError(error));
    }
        

    onConfirmar() {

    }



    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.metaAlmacenService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.metaAlmacen = info;
                this.almacenesCollection.sourceCollection = this.metaAlmacen.almacenesSucursal;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.metaAlmacenService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.metaAlmacen = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.metaAlmacenService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.metaAlmacen = info;                
                this.almacenesCollection.sourceCollection = this.metaAlmacen.almacenesSucursal;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.metaAlmacenService.update(this.metaAlmacen)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.metaAlmacen.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
            })
            .catch(error => this.handleError(error));        
    }

    onRemoveItem(args: EditorEventArgs) {

        let currentSucursal: any = this.almacenesCollection.currentItem;
        currentSucursal.idAlmacen = 0;
        currentSucursal.idArea = 0;
        currentSucursal.nombreAlmacen = '';
        currentSucursal.nombreArea = '';

        Control.invalidateAll();


    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {

            let currentSucursal: any = this.almacenesCollection.currentItem;
            currentSucursal.idAlmacen = this.idAlmacen;
            currentSucursal.idArea = this.idArea;
            currentSucursal.nombreAlmacen = this.cboAlmacenes.selectedItem.nombre;
            currentSucursal.nombreArea = this.cboAreas.selectedItem.nombre;

            Control.invalidateAll();

        }
    }

}