﻿import { Component, Input, Output, EventEmitter, ViewChild, SimpleChange, ChangeDetectorRef } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators, ControlValueAccessor, FormControlName, NG_VALUE_ACCESSOR } from '@angular/forms';
//
import { BaseControl, AztecaValidators } from 'framework3/controls';
//
import { StockInfo } from '../models/stock';
import { UnidadMedidaInfo, UnidadMedidaItem } from '../models/unidad-medida';
import { UbicacionInfo } from '../models/ubicacion';
import { QuantityInputComponent } from '../controls/quantity-input.component';
import { DateTime } from 'wijmo/wijmo';

@Component({
    selector: 'inventory-articulo-stock-entrada',
    templateUrl: './articulo-stock-entrada.component.html'
})
export class ArticuloStockEntradaComponent extends BaseControl implements ControlValueAccessor {
    private form: FormGroup;
    private selectedUM: UnidadMedidaItem;
    /////////// INPUTS
    @Input() stockConfig: StockInfo;
    @Input() idAlmacen: number = 0;
    @Input() unidadesMedida: UnidadMedidaItem[] = [];
    @Input() cantidad: number = 0;
    @Input() unidades: number = 0;
    @Input() idUnidadMedida: number = 0;
    @Input() costoUnitario: number = 0;
    @Input() numeroSerie: string = '';
    @Input() fechaCaducidad: Date = new Date();
    @Input() idUbicacion: number;
    
    @Input() costoEnabled: boolean = true;
    @Input() unidadMedidaEnabled: boolean = true;

    @Input() enabledCantidad: boolean = true;
    /////////// OUTPUS
    @Output() cantidadChange = new EventEmitter<number>();
    @Output() unidadesChange = new EventEmitter<number>();
    @Output() unidadMedidaChange = new EventEmitter<number>();
    @Output() costoUnitarioChange = new EventEmitter<number>();
    @Output() numeroSerieChange = new EventEmitter<string>();
    @Output() fechaCaducidadChange = new EventEmitter<Date>();
    @Output() idUbicacionChange = new EventEmitter<number>();
    @Output() validChange = new EventEmitter<boolean>();

    fechaHoy: Date;
    //

    @ViewChild(QuantityInputComponent) quantity: QuantityInputComponent;
    private get valid(): boolean {
        return this.form.valid && (!this.quantity || this.quantity.total > 0);
    }

    constructor(private formBuilder: FormBuilder, private cdRef: ChangeDetectorRef) {
        super();
        //
        this.form = formBuilder.group({
            'idUnidadMedida': [0],
            'numeroSerie': [''],
            'fechaCaducidad': [null],
            'costoUnitario': [0],
            'idUbicacion': [0]
        });
        //Suscribirse a cambios en la unidad de medida ya que esta determina varios comportamientos en los campos y validaciones
        this.form.controls['idUnidadMedida'].valueChanges.subscribe(value => {
            setTimeout(_ => this.setCurrentUnidad(value));
        });

        this.fechaHoy = new Date();
      
    }

    ngAfterViewInit() {
        this.form.statusChanges.subscribe(_ => {
            this.validChange.emit(this.valid);
        });
    }


    ngOnChanges(changes: SimpleChange) {
        
        let controlSerie = this.form.controls['numeroSerie'];
        let controlUnidadMedida = this.form.controls['idUnidadMedida'];
        if (changes['idUnidadMedida']) {
            controlUnidadMedida.setValue(this.idUnidadMedida);
        }
        if (changes['numeroSerie']) {
            controlSerie.setValue(this.numeroSerie);
        }
        if (changes['fechaCaducidad'] && !this.fechaCaducidad) {
            this.fechaCaducidad = new Date();
            this.cdRef.detectChanges();
        }
        if (changes['unidadesMedida']) {
            setTimeout(_ => this.setCurrentUnidad(this.idUnidadMedida));
        }
        if (this.stockConfig && changes['stockConfig']) {
            //Establecer validadores dependiendo de las opciones del stock
            //Series
            if (this.stockConfig.manejaSeries) {
                controlSerie.setValidators(Validators.required);
                controlUnidadMedida.setValidators(AztecaValidators.selected);
                //Los articulos que se controlan por numero de serie siempre son piezas y su cantidad se establece a 1
                this.cantidadChange.emit(1);
                this.unidadesChange.emit(0);
            }
            else {
                controlSerie.setValidators(null);
                controlUnidadMedida.setValidators(null);
                this.numeroSerieChange.emit('');
                controlSerie.updateValueAndValidity();
            }
            //Si no se maneja control de caducidad, actualizar valor como nulo
            if (!this.stockConfig.manejaCaducidad) {
                this.fechaCaducidadChange.emit(null);
            }
            this.form.updateValueAndValidity();
        }
    }

    /////////// METHODS

    setCurrentUnidad(idUnidadMedida: number) {        
        this.selectedUM = this.unidadesMedida.find(u => u.id == idUnidadMedida);
        if (this.quantity) {
            this.quantity.unidadMedida = this.selectedUM;
        }
        this.validChange.emit(this.valid);
    }

    /////////// HANDLERS
    onUnidadMedidaChanged(idUnidadMedida: number) {
        this.unidadMedidaChange.emit(idUnidadMedida);
        this.validChange.emit(this.valid);
    }

    onCantidadChange(cantidad: number) {
        this.cantidadChange.emit(cantidad);
        this.validChange.emit(this.valid);
    }

    onUnidadesChange(unidades: number) {
        this.unidadesChange.emit(unidades);
        this.validChange.emit(this.valid);
    }

    onCostoChange(costo: number) {
        this.costoUnitarioChange.emit(costo);
    }

    onNumSerieChange(numSerie: string) {
        this.numeroSerieChange.emit(numSerie);
    }

    onFechaCaducidadChange(fecha: Date) {
        if (this.stockConfig && this.stockConfig.manejaCaducidad) {
            this.fechaCaducidadChange.emit(fecha);
        }
        else {
            this.fechaCaducidadChange.emit(null);
        }   
    }

    onUbicacionChange(ubicacion: UbicacionInfo) {
        if (ubicacion) {
            this.idUbicacionChange.emit(ubicacion.identity);
        }
        else {
            this.idUbicacionChange.emit(0);
        }
        
    }

    //Implementación de control value accesor
    writeValue(value: any) {
    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() { }

}
