﻿import { Component, Input, Output, EventEmitter, ViewChild, SimpleChange } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators, ControlValueAccessor, FormControlName, NG_VALUE_ACCESSOR } from '@angular/forms';
//
import { Globalize } from 'wijmo/wijmo';
//
import { BaseControl, AztecaValidators } from 'framework3/controls';
//
import { StockInfo } from '../models/stock';
import { LoteInfo } from '../models/lote';
import { MetodoValuacion } from '../models/metodo-valuacion';
import { UnidadMedidaInfo, UnidadMedidaItem } from '../models/unidad-medida';
import { UbicacionInfo } from '../models/ubicacion';
import { SerieInfo } from '../models/serie';

import { QuantityInputComponent } from './quantity-input.component';
import { QuantityLabelComponent } from './quantity-label.component'
import { ArticuloSeriesComponent } from './articulo-series.component';
import { ArticuloLotesComponent } from './articulo-lotes.component';

@Component({
    selector: 'inventory-articulo-stock-salida',
    templateUrl: './articulo-stock-salida.component.html'
})
export class ArticuloStockSalidaComponent extends BaseControl implements ControlValueAccessor {
    private form: FormGroup;
    private selectedUM: UnidadMedidaItem;
    private costoUnitario: string = '$0.00';
    private disponible: string = '0';    
    /////////// INPUTS
    @Input() stockConfig: StockInfo;
    @Input() idArticuloStock: number = 0;
    @Input() idAlmacen: number = 0;
    @Input() unidadesMedida: UnidadMedidaItem[] = [];
    @Input() cantidad: number = 0;
    @Input() unidades: number = 0;
    @Input() idUnidadMedida: number = 0;        

    /////////// OUTPUS
    @Output() cantidadChange = new EventEmitter<number>();
    @Output() unidadesChange = new EventEmitter<number>();
    @Output() idUnidadMedidaChange = new EventEmitter<number>();        
    @Output() validChange = new EventEmitter<boolean>();
    @Output() lotesChange = new EventEmitter<LoteInfo[]>();
    @Output() seriesChange = new EventEmitter<SerieInfo[]>();
    //

    @ViewChild(QuantityInputComponent) quantity: QuantityInputComponent;
    @ViewChild(QuantityLabelComponent) quantityLbl: QuantityLabelComponent;
    @ViewChild(ArticuloSeriesComponent) seriesControl: ArticuloSeriesComponent;
    @ViewChild(ArticuloLotesComponent) lotesControl: ArticuloLotesComponent;

    private get valid(): boolean {
        let validControls = true;
        if (this.quantity && this.showQuantity() && this.quantity.total <= 0) {
            validControls = false;
        }
        if (this.seriesControl && this.showSeries() && this.seriesControl.count() <= 0) {
            validControls = false;
        }
        if (this.lotesControl && this.showLotes() && this.lotesControl.count() <= 0) {
            validControls = false;
        }
        return this.form.valid && validControls;
    }

    constructor(private formBuilder: FormBuilder) {
        super();
        //
        this.form = formBuilder.group({
            'idUnidadMedida': [0],            
        });
        //Suscribirse a cambios en la unidad de medida ya que esta determina varios comportamientos en los campos y validaciones
        this.form.controls['idUnidadMedida'].valueChanges.subscribe(value => {
            setTimeout(_ => this.setCurrentUnidad(value));
        });
    }

    ngAfterViewInit() {
        this.form.statusChanges.subscribe(_ => {
            setTimeout(_ => this.validChange.emit(this.valid));
        });        
    }

    ngOnChanges(changes: SimpleChange) {        
        let controlUnidadMedida = this.form.controls['idUnidadMedida'];
        if (changes['idUnidadMedida']) {
            if (this.stockConfig && this.stockConfig.manejaLotes && this.stockConfig.metodoValuacion == MetodoValuacion.NINGUNO) {
                setTimeout(_ => this.setCurrentUnidad(this.idUnidadMedida));   
            }
            else {
                controlUnidadMedida.setValue(this.idUnidadMedida);
            }
        }
        if (changes['unidadesMedida']) {
            setTimeout(_ => this.setCurrentUnidad(this.idUnidadMedida));
        }
        if (this.stockConfig && changes['stockConfig']) {            
            //Establecer validadores dependiendo de las opciones del stock
            //Series
            if (this.stockConfig.manejaSeries) {
                if (this.seriesControl) {
                    this.seriesControl.clear();
                }
                controlUnidadMedida.setValidators(null);
            }
            else {                
                controlUnidadMedida.setValidators(AztecaValidators.selected);
            }            
            this.form.updateValueAndValidity();
        }
    }

    /////////// METHODS

    setCurrentUnidad(idUnidadMedida: number) {
        this.selectedUM = this.unidadesMedida.find(u => u.id == idUnidadMedida);
        if (this.quantity) {
            this.quantity.unidadMedida = this.selectedUM;
        }
        //Actualizar valor de existencia disponible y el costo unitario basado en la unidad de medida seleccionada
        if (this.stockConfig && this.selectedUM) {
            this.disponible = (this.stockConfig.disponible / this.selectedUM.factor).toString();
            this.quantityLbl.Show( + this.disponible, this.selectedUM);
            this.costoUnitario = Globalize.formatNumber((this.stockConfig.costoPromedio * this.selectedUM.factor), 'c');
            if (this.idUnidadMedida == 0 || this.form.controls['idUnidadMedida'].value != this.idUnidadMedida) {
                this.form.controls['idUnidadMedida'].setValue(this.selectedUM.id);
            }
        }
        this.validChange.emit(this.valid);
    }

    private showQuantity(): boolean {
        if (this.stockConfig) {
            if (this.stockConfig.manejaLotes) {
                if (this.stockConfig.metodoValuacion == MetodoValuacion.NINGUNO || this.stockConfig.metodoValuacion == '') {
                    return false;
                }
                else {
                    return true;
                }
            }
            else {
                if (this.stockConfig.manejaSeries) {
                    return false;
                }
                else {
                    return true;
                }
            }
        }
        else {
            return false;
        }
    }

    private showSeries(): boolean {
        if (this.stockConfig) {            
            if (this.stockConfig.manejaSeries) {
                //Si está controlado por series, revisar que además no esté controlado por lotes y algún método de valuación
                if (!this.stockConfig.manejaLotes || (this.stockConfig.metodoValuacion == MetodoValuacion.NINGUNO || this.stockConfig.metodoValuacion == '')) {
                    return true;
                }
                else {
                    return false;
                }
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
    }

    private showLotes() {
        if (this.stockConfig) {
            if (this.stockConfig.manejaLotes && !this.stockConfig.manejaSeries && (this.stockConfig.metodoValuacion == MetodoValuacion.NINGUNO || this.stockConfig.metodoValuacion == '')) {
                return true;
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
    }



    /////////// HANDLERS

    onCantidadChange(cantidad: number) {
        this.cantidadChange.emit(cantidad);
        this.validChange.emit(this.valid);
    }

    onUnidadesChange(unidades: number) {
        this.unidadesChange.emit(unidades);
        this.validChange.emit(this.valid);
    }

    onLotesChange(lotes: LoteInfo[]) {
        this.lotesChange.emit(lotes);
        this.validChange.emit(this.valid);
    }

    onSeriesChange(series: SerieInfo[]) {
        this.seriesChange.emit(series);
        this.validChange.emit(this.valid);
    }

    //Implementación de control value accesor
    writeValue(value: any) {
    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() { }

}
