﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Inventario
import { AlmacenInfo } from '../models/almacen';
import { ConceptoInfo, ConceptoItem } from '../models/concepto';
//
import { AlmacenService } from '../services/almacen.service';
import { ConceptoService } from '../services/concepto.service';
import { EntradaService } from '../services/entrada.service';
//
import { EntradaInfo, EntradaItem } from '../models/entrada';
import { EntradaDetail } from '../models/entrada-detail';
import { Empaquetado } from '../models/empaquetado';
//
import { EntradaDetailEditor } from './parts/entrada-detail.editor';
import { DetailQuantityEditor } from './parts/detail-quantity.editor';


@Component({
    selector: 'inventario-entradas-editor',
    templateUrl: './entradas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EntradasEditor)
        },
        {
            provide: DocumentService, useExisting: EntradaService
        }
    ]
})
export class EntradasEditor extends DocumentEditor {
    entrada = new EntradaInfo();
    conceptos: ConceptoItem[] = [];
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: EntradaDetailEditor;
    @ViewChild(DetailQuantityEditor) quantityEditor: DetailQuantityEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private entradaService: EntradaService,
        private almacenService: AlmacenService,
        private conceptoService: ConceptoService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idConcepto': [0, Validators.required],
            'flete': [0],
            'observaciones': ['']
        });
    }

    ////////// METODOS

    loadAlmacenInfo() {
        console.log('Almacen');
        this.almacenService.getDetailByID(this.entrada.idAlmacen)
            .then(almacen => this.entrada.almacen = almacen)
            .catch(error => this.handleError(error));
    }

    loadConceptos() {
        console.log('Conceptos');
        this.conceptoService.loadListByUser(this.context.app.defaultCompany.identity, this.context.session.user.identity, 'E')
            .then(conceptos => this.conceptos = conceptos)
            .catch(error => this.handleError(error));
    }

    ////////// HANDLERS
    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        //
        setTimeout(_ => this.loadConceptos());
    }

    onCreatingDocument() {
        //Obtener almacen especificado en el dialogo de nueva entrada
        this.route.params.forEach(params => {
            if (params['idAlmacen']) {
                this.entrada.idAlmacen = +params['idAlmacen'];
            }
        });
        if (this.entrada.idAlmacen > 0) {
            this.loadAlmacenInfo();
        }
        else {
            this.showAlert('No se ha especificado un almacén destino', 'warning');
        }   
        
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.entradaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.entrada = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.entradaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.entrada = doc;
                this.detailsEditor.clientIndexStart = this.entrada.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.entradaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.entrada = doc;
                this.detailsEditor.clientIndexStart = this.entrada.details.length;
                if (this.entrada.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.entradaService.update(this.entrada)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.entrada.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.entrada = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.entradaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.entradaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.entrada.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new EntradaDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: EntradaDetail) {
        this.quantityEditor.unidadMedida = detail.unidadMedida;
        this.quantityEditor.cantidad = detail.cantidad;
        this.quantityEditor.unidades = detail.unidades;
        this.quantityEditor.show();
    }

    onDeletingDetail(detail: EntradaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: EntradaDetail[]) {
        this.entrada.details = updatedDetails;
    }

    onSavingDetail(detail: EntradaDetail) {
        this.detailsList.updateItem(detail);
    }

}

