﻿import { Component, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//Inventario
import { SalidaDetail } from '../../models/salida-detail';
import { ArticuloStockInfo } from '../../models/articulo-stock';
import { StockInfo } from '../../models/stock';
import { LoteInfo } from '../../models/lote';
import { SerieInfo } from '../../models/serie';
import { UnidadMedidaItem } from '../../models/unidad-medida';
import { UbicacionItem, UbicacionInfo } from '../../models/ubicacion';
import { MetodoValuacion } from '../../models/metodo-valuacion';

//
import { ArticuloStockService } from '../../services/articulo-stock.service';
import { UnidadMedidaService } from '../../services/unidad-medida.service';
import { AlmacenService } from '../../services/almacen.service';

@Component({
    selector: 'inventario-salida-detail-editor',
    templateUrl: './salida-detail.editor.html'
})
export class SalidaDetailEditor extends BasicEditor {    
    unidadesMedida: UnidadMedidaItem[] = [];
    stockConfig: StockInfo;
    validInput: boolean = false;
    lotes: LoteInfo[] = [];
    series: SerieInfo[] = [];
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new SalidaDetail();
    //Objeto original que se editará
    @Input() detail: SalidaDetail = new SalidaDetail();

    @Input() idAlmacen: number = 0;

    constructor(protected context: Context, private builder: FormBuilder, private unidadService: UnidadMedidaService, private almacenService: AlmacenService) {
        super(context);

        this.form = builder.group({
            'idArticulo': [0, AztecaValidators.selected]
        });
    }

    /////////////////// METODOS ///////////////////////////////

    loadUnidadesMedida(idTipoUnidad: number) {
        //Si el articulo esta controlado por series, no se puede cambiar la unidad de medida, por lo tanto solo se debe utilizar la unidad de medida el articulo,
        //en caso contrario, cargar lista de unidades de medida del mismo tipo
        if (this.stockConfig.manejaSeries) {
            let unidadArticulo = this.detailCopy.articuloStock.unidadMedida;
            let newUnidad: UnidadMedidaItem = {
                id: unidadArticulo.identity, codigo: unidadArticulo.code, idUnidadMedidaTipo: unidadArticulo.idUnidadMedidaTipo, base: unidadArticulo.base,
                factor: unidadArticulo.factor, fraccionable: unidadArticulo.fraccionable, paquete: unidadArticulo.paquete, activo: true
            };
            let newUnidades = [];
            newUnidades.push(newUnidad);
            this.unidadesMedida = newUnidades;
            this.detailCopy.idUnidadMedida = unidadArticulo.identity;
        }
        else {
            this.unidadService.loadList(idTipoUnidad)
                .then(unidadesMedida => {
                    this.unidadesMedida = unidadesMedida;
                    this.detailCopy.idUnidadMedida = this.detailCopy.articuloStock.idUnidadMedida;
                })
                .catch(error => this.handleError(error));
        }
    }

    loadStock(idArticulo: number) {
        this.almacenService.loadStock(this.idAlmacen, idArticulo)
            .then(stock => {
                this.stockConfig = stock;
                this.loadUnidadesMedida(this.detailCopy.articuloStock.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }


    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
        this.form.reset({ idArticulo: 0 });
        this.form.markAsPristine();
    }

    onLoadingItem(detail: SalidaDetail) {
        this.detailCopy = <SalidaDetail>this.copyItem(this.detail);
        this.form.updateValueAndValidity();
    }

    onUpdatingItem(detail: SalidaDetail) {
        this.unidadService.getDetailFromCache(this.detailCopy.idUnidadMedida)
            .then(unidad => {
                //Si el artículo maneja lotes y ningún método de valuación, el usuario podrá especificar indicar manualmente los lotes y por lo cual generar más de un detalle
                if (!this.stockConfig.manejaSeries && this.stockConfig.manejaLotes && (this.stockConfig.metodoValuacion == MetodoValuacion.NINGUNO || this.stockConfig.metodoValuacion == '')) {
                    let details: SalidaDetail[] = [];
                    //Crear una partida por cada lote que el usuario haya seleccionado
                    this.lotes.filter(lote => lote.cantidad > 0 || lote.unidades > 0).forEach(lote => {
                        let detail = new SalidaDetail();
                        detail._clientID = this.createClientID();
                        detail.idArticuloStock = this.detailCopy.idArticuloStock;
                        detail.cantidad = lote.cantidad;
                        detail.unidades = lote.unidades;
                        detail.costoUnitario = lote.costo;
                        detail.idLote = lote.id;                        
                        detail.idUnidadMedida = this.detailCopy.idUnidadMedida;
                        //
                        detail.articuloStock = this.detailCopy.articuloStock;
                        detail.stock = this.stockConfig;
                        detail.unidadMedida = unidad;
                        //
                        details.push(detail);
                    });
                    this.savingItems.emit(details);
                }
                //Si el artículo maneja series, crear un detalle por cada número de serie
                else if (this.stockConfig.manejaSeries && (this.stockConfig.metodoValuacion == MetodoValuacion.NINGUNO || this.stockConfig.metodoValuacion == '')) {
                    let details: SalidaDetail[] = [];
                    //Crear una partida por cada lote que el usuario haya seleccionado
                    this.series.forEach(item => {
                        let detail = new SalidaDetail();
                        detail._clientID = this.createClientID();
                        detail.idArticuloStock = this.detailCopy.idArticuloStock;
                        detail.numeroSerie = item.numeroSerie;
                        detail.cantidad = 1;
                        detail.unidades = 0;
                        detail.costoUnitario = item.costo;
                        detail.idLote = item.idLote;
                        detail.idUnidadMedida = this.detailCopy.idUnidadMedida;
                        //
                        detail.articuloStock = this.detailCopy.articuloStock;
                        detail.stock = this.stockConfig;
                        detail.unidadMedida = unidad;
                        //
                        details.push(detail);
                    });
                    this.savingItems.emit(details);
                }
                else {
                    this.detailCopy.stock = this.stockConfig;
                    this.detailCopy.unidadMedida = unidad;
                    if (this.detailCopy._clientID == 0) {
                        this.detailCopy._clientID = this.createClientID();
                    }
                    else {
                        this.detailCopy.state = ChangeState.MODIFIED;
                    }
                    this.savingItem.emit(this.detailCopy);
                    this.detailCopy = new SalidaDetail();
                }
            })
            .catch(error => this.handleError(error));
    }

    onArticuloChanged(articulo: ArticuloStockInfo) {
        if (articulo) {
            this.detailCopy.articuloStock = articulo;
            this.loadStock(articulo.identity);
        }
        else {
            this.unidadesMedida = [];
            this.stockConfig = new StockInfo();
        }
    }

    onInputValidChange(valid: boolean) {
        this.validInput = valid;
    }

    onLotesChange(lotes: LoteInfo[]) {
        this.lotes = lotes;
    }

    onSeriesChange(series: SerieInfo[]) {
        this.series = series;
    }
}
