﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Inventario
import { SalidaInfo, SalidaItem } from '../models/salida';
import { SalidaDetail } from '../models/salida-detail';
import { SalidaService } from '../services/salida.service';

import { SalidaDetailEditor } from './parts/salida-detail.editor';
import { DetailQuantityEditor } from './parts/detail-quantity.editor';
//
import { AlmacenInfo } from '../models/almacen';
import { ConceptoInfo, ConceptoItem } from '../models/concepto';
import { Empaquetado } from '../models/empaquetado';

//
import { AlmacenService } from '../services/almacen.service';
import { ConceptoService } from '../services/concepto.service';


@Component({
    selector: 'inventario-salidas-editor',
    templateUrl: './salidas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SalidasEditor)
        },
        {
            provide: DocumentService, useExisting: SalidaService
        }
    ]
})
export class SalidasEditor extends DocumentEditor {
    salida = new SalidaInfo();
    conceptos: ConceptoItem[] = [];
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: SalidaDetailEditor;
    @ViewChild(DetailQuantityEditor) quantityEditor: DetailQuantityEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private salidaService: SalidaService,
        private almacenService: AlmacenService,
        private conceptoService: ConceptoService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idConcepto': [0, Validators.required],
            'flete': [0],
            'observaciones': ['']
        });
    }

    ////////// METODOS

    loadAlmacenInfo() {
        this.almacenService.getDetailByID(this.salida.idAlmacen)
            .then(almacen => this.salida.almacen = almacen)
            .catch(error => this.handleError(error));
    }

    loadConceptos() {
        this.conceptoService.loadListByUser(this.context.app.defaultCompany.identity, this.context.session.user.identity, 'S')
            .then(conceptos => this.conceptos = conceptos)
            .catch(error => this.handleError(error));
    }

    ////////// HANDLERS

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        //
        setTimeout(_ => this.loadConceptos());
    }

    onCreatingDocument() {
        //Obtener almacen especificado en el dialogo de nueva entrada
        this.route.params.forEach(params => {
            if (params['idAlmacen']) {
                this.salida.idAlmacen = +params['idAlmacen'];
            }
        });
        if (this.salida.idAlmacen > 0) {
            this.loadAlmacenInfo();
        }
        else {
            this.showAlert('No se ha especificado un almacén destino', 'warning');
        }   
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.salidaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.salida = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.salidaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.salida = doc;
                this.detailsEditor.clientIndexStart = this.salida.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.salidaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.salida = doc;
                this.detailsEditor.clientIndexStart = this.salida.details.length;
                if (this.salida.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.salidaService.update(this.salida)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.salida.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.salida = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.salidaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.salidaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.salida.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new SalidaDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: SalidaDetail) {
        if (!detail.stock.manejaSeries) {
            this.quantityEditor.unidadMedida = detail.unidadMedida;
            this.quantityEditor.cantidad = detail.cantidad;
            this.quantityEditor.unidades = detail.unidades;
            this.quantityEditor.show();
        }
    }

    onDeletingDetail(detail: SalidaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: SalidaDetail[]) {
        this.salida.details = updatedDetails;
    }

    onSavingDetail(detail: SalidaDetail) {
        this.detailsList.updateItem(detail);
    }

    onSavingDetails(details: SalidaDetail[]) {
        this.detailsList.updateItems(details);
    }

    onUpdateQuantity(value: any) {
        let detail = <SalidaDetail>this.detailsList.currentItem;
        if (detail.unidadMedida.paquete) {
            let cantidad = <Empaquetado>value;
            detail.cantidad = cantidad.paquetes;
            detail.unidades = cantidad.unidades;
        }
        else {
            detail.cantidad = <number>value;
        }
        this.detailsList.updateItem(detail);
    }

}

