﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo, SucursalService, SucursalInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { TransferenciaInfo, TransferenciaItem } from '../models/transferencia';
import { TransferenciaDetail } from '../models/transferencia-detail';
import { TransferenciaService } from '../services/transferencia.service';
import { TransferenciaDetailEditor } from './parts/transferencias-tit-detail.editor';
import { TrensferenciaAceptarComponent } from '../parts/transferencia-aceptar.component';

import { AreaInfo } from '../models/area'
import { AlmacenService } from '../services/almacen.service';

import { Empaquetado } from '../models/empaquetado';

@Component({
    selector: 'inventario-transferencias-tit-editor',
    templateUrl: './transferencias-tit.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TransferenciasTitEditor)
        },
        {
            provide: DocumentService, useExisting: TransferenciaService
        }
    ]
})
export class TransferenciasTitEditor extends DocumentEditor {
    transferencia = new TransferenciaInfo();
    areaOrigen = new AreaInfo();

    argsCurrentDocument: WorkflowEventArgs;    

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: TransferenciaDetailEditor;
    //
    @ViewChild('dlgTransferenciaAceptar') dlgTransferenciaAceptar: TrensferenciaAceptarComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private transferenciaService: TransferenciaService,
        private almacenService: AlmacenService,
        private sucursalService: SucursalService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idAreaOrigen': [0],
            'idAlmacenOrigen': [0],
            'observaciones': ['', Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {
        //Obtener almacen especificado en el dialogo de nueva entrada
        this.route.params.forEach(params => {
            if (params['idAlmacen']) {
                this.transferencia.idAlmacenOrigen = +params['idAlmacen'];
            }
            if (params['idSucursal']) {
                this.transferencia.idSucursalDestino = +params['idSucursal'];
            }
        });
        if (this.transferencia.idAlmacenOrigen > 0) {
            this.loadAlmacenInfo();
            this.loadSucursalInfo();
        }
        else {
            this.showAlert('No se ha especificado un almacén destino', 'warning');
        }   

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.transferenciaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.transferencia = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.transferenciaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.transferencia = doc;
                this.detailsEditor.clientIndexStart = this.transferencia.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.transferenciaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.transferencia = doc;
                this.detailsEditor.clientIndexStart = this.transferencia.details.length;
                if (this.transferencia.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.transferenciaService.update(this.transferencia)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.transferencia.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.transferencia = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.argsCurrentDocument = args;
        args.step.activities.forEach(activity => {
            if (activity.tag == 'ACEPTAR') {
                this.dlgTransferenciaAceptar.showAceptar(this.transferencia.idSucursalDestino);
            }
            else {
                this.context.app.showSpinner();
                this.transferenciaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
                    .then(_ => {
                        //Volver a cargar documento
                        this.onViewingDocument(args.document.id);
                    })
                    .catch(error => this.handleError(error));
            }
        });

    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.transferenciaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.transferencia.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new TransferenciaDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: TransferenciaDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: TransferenciaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: TransferenciaDetail[]) {
        this.transferencia.details = updatedDetails;
    }

    onSavingDetail(detail: TransferenciaDetail) {
        this.detailsList.updateItem(detail);
    }

    onSavingDetails(details: TransferenciaDetail[]) {
        this.detailsList.updateItems(details);
    }


    onUpdateQuantity(value: any) {
        let detail = <TransferenciaDetail>this.detailsList.currentItem;
        if (detail.unidadMedida.paquete) {
            let cantidad = <Empaquetado>value;
            detail.cantidad = cantidad.paquetes;
            detail.unidades = cantidad.unidades;
        }
        else {
            detail.cantidad = <number>value;
        }
        this.detailsList.updateItem(detail);
    }

    ///////////////// METODOS

    loadAlmacenInfo() {
        this.almacenService.getDetailByID(this.transferencia.idAlmacenOrigen)
            .then(almacen => this.transferencia.almacenOrigen = almacen)
            .catch(error => this.handleError(error));
    }

    loadSucursalInfo() {
        this.sucursalService.getDetailByID(this.transferencia.idSucursalDestino)
            .then(sucursal => this.transferencia.sucursalDestino = sucursal)
            .catch(error => this.handleError(error));
    }


}

