﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { TraspasoInfo, TraspasoItem } from '../models/traspaso';
import { TraspasoDetail } from '../models/traspaso-detail';
import { TraspasoService } from '../services/traspaso.service';
import { TraspasoDetailEditor } from './parts/traspasos-tit-detail.editor';

import { AreaInfo } from '../models/area'
import { ConceptoItem } from '../models/concepto'
import { ConceptoService } from '../services/concepto.service'
import { Empaquetado } from '../models/empaquetado';

@Component({
    selector: 'inventario-traspasos-tit-editor',
    templateUrl: './traspasos-tit.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TraspasosTitEditor)
        },
        {
            provide: DocumentService, useExisting: TraspasoService
        }
    ]
})
export class TraspasosTitEditor extends DocumentEditor {
    traspaso = new TraspasoInfo();
    areaOrigen = new AreaInfo();
    areaDestino = new AreaInfo();
    conceptosItems: ConceptoItem[]= [];
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: TraspasoDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private conceptoService: ConceptoService,
        private traspasoService: TraspasoService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idAreaOrigen': [0],
            'idAlmacenOrigen': [0],
            'idAreaDestino': [0],
            'idAlmacenDestino': [0],
            'idConcepto': [0, Validators.required],
            'observaciones': ['', Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.conceptoService.loadListByUser(this.context.app.defaultCompany.identity, this.context.session.user.identity, 'T')
            .then(conceptos => {
                this.conceptosItems = conceptos;
            })
            .catch(error => this.handleError(error));

    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.traspasoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.traspaso = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.traspasoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.traspaso = doc;
                this.detailsEditor.clientIndexStart = this.traspaso.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.traspasoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.traspaso = doc;
                this.detailsEditor.clientIndexStart = this.traspaso.details.length;
                if (this.traspaso.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.traspasoService.update(this.traspaso)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.traspaso.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.traspaso = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.traspasoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.traspasoService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.traspaso.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new TraspasoDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: TraspasoDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: TraspasoDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: TraspasoDetail[]) {
        this.traspaso.details = updatedDetails;
    }

    onSavingDetail(detail: TraspasoDetail) {
        this.detailsList.updateItem(detail);
    }

    onSavingDetails(details: TraspasoDetail[]) {
        this.detailsList.updateItems(details);
    }

    onUpdateQuantity(value: any) {
        let detail = <TraspasoDetail>this.detailsList.currentItem;
        if (detail.unidadMedida.paquete) {
            let cantidad = <Empaquetado>value;
            detail.cantidad = cantidad.paquetes;
            detail.unidades = cantidad.unidades;
        }
        else {
            detail.cantidad = <number>value;
        }
        this.detailsList.updateItem(detail);
    }


    //////////////////IMPLEMENTACION/////////////////////////

    onAreaOrigenChanged(area: AreaInfo) {
        this.areaOrigen = area;
    }
    onAreaDestinoChanged(area: AreaInfo) {
        this.areaDestino = area;
    }

}

