﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

// wijmo
import { Control } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid } from 'wijmo/wijmo.grid';

import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { AplicacionService } from '../services/aplicacion.service';
import { AplicacionInfo, AplicacionItem, VersionItem, AplicacionConfiguracionItem, AplicacionPoliticaItem } from '../models/aplicacion';

import { AplicacionVersionEditor } from './parts/aplicacion-version.editor';

import { ConfiguracionSelectorComponent } from '../controls/configuracion-selector.component';
import { ConfiguracionInfo } from '../models/configuracion';
import { PoliticaSelectorComponent } from '../controls/politica-selector.component';
import { PoliticaInfo } from '../models/politica';



@Component({
    selector: 'mantenimiento-aplicacion-editor',
    templateUrl: './aplicaciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AplicacionesEditor)
        },
        {
            provide: CatalogService, useExisting: AplicacionService
        }
    ]
})
export class AplicacionesEditor extends CatalogEditor {
    aplicacion: AplicacionInfo = new AplicacionInfo();
    //
    versionEdit: VersionItem = new VersionItem();

    versionesCollection: TrackableCollection = new TrackableCollection();

    @ViewChild(AplicacionVersionEditor) dlgVersion: AplicacionVersionEditor;
    @ViewChild('grdVersiones') grid: FlexGrid;

    configuracionesCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorConfiguracion') selectorConfiguracion: ConfiguracionSelectorComponent;

    politicasCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorPolitica') selectorPolitica: PoliticaSelectorComponent;

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private aplicacionService: AplicacionService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'package': ['', Validators.required],
            'idConfiguracion': [0],
            'idPolitica': [0],
        });
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
    }

    onCreatingItem() {

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.aplicacionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.aplicacion = info;

                this.configuracionesCollection.sourceCollection = info.configuraciones;
                this.configuracionesCollection.refresh;

                this.politicasCollection.sourceCollection = info.politicas;
                this.politicasCollection.refresh;


                this.aplicacion.versiones.forEach(item => {
                    item.lanzamiento = moment(item.lanzamiento).toDate();
                });
                this.versionesCollection = new TrackableCollection(this.aplicacion.versiones);
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.aplicacionService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.aplicacion = info;
                this.aplicacion.versiones.forEach(item => {
                    item.lanzamiento = moment(item.lanzamiento).toDate();
                });
                this.versionesCollection = new TrackableCollection(this.aplicacion.versiones);
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.aplicacionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.aplicacion = info;

                this.configuracionesCollection.sourceCollection = info.configuraciones;
                this.configuracionesCollection.refresh;

                this.politicasCollection.sourceCollection = info.politicas;
                this.politicasCollection.refresh;

                this.aplicacion.versiones.forEach(item => {
                    item.lanzamiento = moment(item.lanzamiento).toDate();
                });
                this.versionesCollection = new TrackableCollection(this.aplicacion.versiones);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.aplicacionService.update(this.aplicacion)
            .then(info => {
                this.context.app.hideSpinner();
                this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.aplicacion.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.aplicacion = info;
            })
            .catch(error => this.handleError(error));
    }


    onAgregarVersion() {
        this.versionEdit = new VersionItem();
        this.dlgVersion.show();
    }

    onSavedVersion() {
        let newVersiones: VersionItem[] = this.aplicacion.versiones.slice();
        newVersiones.push(this.cloneInfo(this.versionEdit));
        this.aplicacion.versiones = newVersiones;
        this.versionesCollection = new TrackableCollection(this.aplicacion.versiones);
    }

    onRowDeleteVersion(version: any) {
        this.versionesCollection.remove(version);
    }

    onRemoveConfiguracion(item) {
        this.configuracionesCollection.remove(item);
        this.configuracionesCollection.refresh();
    }

    onRemovePolitica(item) {
        this.politicasCollection.remove(item);
        this.politicasCollection.refresh();
    }


    menuItemClicked(menu: Menu, args: any) {
        let item = this.versionesCollection.currentItem;
        if (item != null) {
            if (menu.selectedIndex == 0) {
                this.context.app.showSpinner()
                this.aplicacionService.getLinkDescargaQr(item.id)
                    .then(result => {
                        var content = window.location.protocol + "//" + window.location.hostname + ":" + window.location.port + "/file-server/" + result;
                        window.open(content);
                        this.context.app.hideSpinner();
                    })
                    .catch(e => this.handleError(e));
            }
        }
    }

    tabSelected(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    cloneInfo<T>(element: any): T {
        return <T>JSON.parse(JSON.stringify(element));
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD-CONFIGURACION') {
            var configuracionNew: AplicacionConfiguracionItem = new AplicacionConfiguracionItem();
            var configuracionInf: ConfiguracionInfo = new ConfiguracionInfo();

            configuracionNew.idConfiguracion = this.selectorConfiguracion.selectedItem.identity;
            configuracionNew.configuracion = this.selectorConfiguracion.selectedItem;

            var configuracionSlice = this.aplicacion.configuraciones.slice();

            configuracionSlice.push(configuracionNew);

            this.aplicacion.configuraciones = configuracionSlice;

            this.configuracionesCollection.sourceCollection = configuracionSlice;
            this.configuracionesCollection.refresh;
        }
        if (button.name == 'ADD-POLITICA') {
            var politicaNew: AplicacionPoliticaItem = new AplicacionPoliticaItem();
            var politicaInf: PoliticaInfo = new PoliticaInfo();

            politicaNew.idPolitica = this.selectorPolitica.selectedItem.identity;
            politicaNew.politica = this.selectorPolitica.selectedItem;

            var politicaSlice = this.aplicacion.politicas.slice();

            politicaSlice.push(politicaNew);

            this.aplicacion.politicas = politicaSlice;

            this.politicasCollection.sourceCollection = politicaSlice;
            this.politicasCollection.refresh;
        }


    }

}