﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Features
import { DispositivoGrupoService } from '../services/dispositivo-grupo.service';
import { DispositivoGrupoInfo, DispositivoGrupoItem, DispositivoGrupoConfiguracionItem, DispositivoGrupoPoliticaItem } from '../models/dispositivo-grupo';
import { SistemaOperativoItem } from '../models/sistema-operativo';
import { SistemaOperativoService } from '../services/sistema-operativo.service';
import { MarcaItem, MarcaSelectorComponent, MarcaInfo, EquipoInformaticaTipoItem } from '../../maintenance';

import { ConfiguracionSelectorComponent } from '../controls/configuracion-selector.component';
import { ConfiguracionInfo } from '../models/configuracion';
import { PoliticaSelectorComponent } from '../controls/politica-selector.component';
import { PoliticaInfo } from '../models/politica';


@Component({
    selector: 'mantenimiento-dispositivo-grupo-editor',
    templateUrl: './dispositivos-grupos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DispositivosGruposEditor)
        },
        {
            provide: CatalogService, useExisting: DispositivoGrupoService
        }
    ]
})
export class DispositivosGruposEditor extends CatalogEditor {
    dispositivoGrupo: DispositivoGrupoInfo = new DispositivoGrupoInfo();
    idServicioTipo: number = 4;
    marca: MarcaInfo = new MarcaInfo();
    sistemasOperativos: SistemaOperativoItem[] = [];

    configuracionesCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorConfiguracion') selectorConfiguracion: ConfiguracionSelectorComponent;

    politicasCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorPolitica') selectorPolitica: PoliticaSelectorComponent;


    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private dispositivoGrupoService: DispositivoGrupoService, private sistemaOperativoService: SistemaOperativoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'marca': [''],
            'modelo': [''],
            'versionOS': ['', Validators.required],
            'memoria': [0, Validators.required],
            'almacenamiento': [0, Validators.required],
            'idSubMarca': [0, AztecaValidators.selected],
            'alto': [0, Validators.required],
            'ancho': [0, Validators.required],
            'espesor': [0, AztecaValidators.selected],
            'idSistemaOperativo': [0, Validators.required],
            'idConfiguracion': [0],
            'idPolitica': [0]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////
    onSelectedItemChanged(marca: MarcaInfo) {
        if (marca) {
            this.marca = marca;
        }
    }

    loadSistemasOperativos() {
        this.sistemaOperativoService.loadList(this.context.app.defaultSite.identity)
            .then(so => this.sistemasOperativos = so)
            .catch(error => this.handleError(error));
    }

    
	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadSistemasOperativos();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.dispositivoGrupoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.dispositivoGrupo = info;  

                this.configuracionesCollection.sourceCollection = info.configuraciones;
                this.configuracionesCollection.refresh;

                this.politicasCollection.sourceCollection = info.politicas;
                this.politicasCollection.refresh;


                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.dispositivoGrupoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.dispositivoGrupo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.dispositivoGrupoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.dispositivoGrupo = info;      

                this.configuracionesCollection.sourceCollection = info.configuraciones;
                this.configuracionesCollection.refresh;

                this.politicasCollection.sourceCollection = info.politicas;
                this.politicasCollection.refresh;

            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.dispositivoGrupoService.update(this.dispositivoGrupo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.dispositivoGrupo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.dispositivoGrupo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onRemoveConfiguracion(item) {
        this.configuracionesCollection.remove(item);
        this.configuracionesCollection.refresh();
    }

    onRemovePolitica(item) {
        this.politicasCollection.remove(item);
        this.politicasCollection.refresh();
    }


    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD-CONFIGURACION') {
            var configuracionNew: DispositivoGrupoConfiguracionItem = new DispositivoGrupoConfiguracionItem();
            var configuracionInf: ConfiguracionInfo = new ConfiguracionInfo();

            configuracionNew.idConfiguracion = this.selectorConfiguracion.selectedItem.identity;
            configuracionNew.configuracion = this.selectorConfiguracion.selectedItem;

            var configuracionSlice = this.dispositivoGrupo.configuraciones.slice();

            configuracionSlice.push(configuracionNew);

            this.dispositivoGrupo.configuraciones = configuracionSlice;

            this.configuracionesCollection.sourceCollection = configuracionSlice;
            this.configuracionesCollection.refresh;
        }
        if (button.name == 'ADD-POLITICA') {
            var politicaNew: DispositivoGrupoPoliticaItem = new DispositivoGrupoPoliticaItem();
            var politicaInf: PoliticaInfo = new PoliticaInfo();

            politicaNew.idPolitica = this.selectorPolitica.selectedItem.identity;
            politicaNew.politica = this.selectorPolitica.selectedItem;

            var politicaSlice = this.dispositivoGrupo.politicas.slice();

            politicaSlice.push(politicaNew);

            this.dispositivoGrupo.politicas = politicaSlice;

            this.politicasCollection.sourceCollection = politicaSlice;
            this.politicasCollection.refresh;
        }

    }

}