﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Features
import { DispositivoService } from '../services/dispositivo.service';
import { DispositivoGrupoService } from '../services/dispositivo-grupo.service'
import { DispositivoInfo, DispositivoItem, DispositivoAplicacionItem } from '../models/dispositivo';
import { DispositivoGrupoInfo } from '../models/dispositivo-grupo';
import { RedTelefonicaItem } from '../models/red-telefonica';
import { RedTelefonicaService } from '../services/red-telefonica.service';
import { AplicacionSelectorComponent } from '../controls/aplicacion-selector.component';
import { AplicacionInfo } from '../models/aplicacion';


@Component({
    selector: 'mantenimiento-dispositivo-editor',
    templateUrl: './dispositivos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DispositivosEditor)
        },
        {
            provide: CatalogService, useExisting: DispositivoService
        }
    ]
})
export class DispositivosEditor extends CatalogEditor {
    dispositivo: DispositivoInfo = new DispositivoInfo(); 
    idEquipoGrupo: number = 0;
    idDispositivoGrupo: number = 0;
    dispositivoGrupo: DispositivoGrupoInfo = new DispositivoGrupoInfo;
    redesTelefonicas: RedTelefonicaItem[] = [];

    configuracionesCollection: TrackableCollection = new TrackableCollection();
    politicasCollection: TrackableCollection = new TrackableCollection();
    aplicacionesCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorAplicacion') selectorAplicacion: AplicacionSelectorComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private dispositivoService: DispositivoService, private dispositivoGrupoService: DispositivoGrupoService, private redTelefonicaService: RedTelefonicaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'imei': ['', Validators.required],
            'idLocalizacion': [0, Validators.required],
            'idRedTelefonica': ['Falta catálogo de redes telefónicas'],
            'noLinea': ['', Validators.required],
            'idAplicacion': [0]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////
    loadEquipoGrupo() {
        this.dispositivoGrupoService.getDetailByID(this.idDispositivoGrupo)
            .then(info => {
                this.dispositivoGrupo = info;
            })
            .catch(error => this.handleError(error));
    }

    loadRedesTelefonicas() {
        this.redTelefonicaService.loadList(this.context.app.defaultSite.identity)
            .then(redes => this.redesTelefonicas = redes)
            .catch(error => this.handleError(error));
    }


	///////////////////////// HANDLERS ///////////////////////////////////////
    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipoGrupo = params['idEquipoGrupo'];
            this.idDispositivoGrupo = params['idDispositivoGrupo'];
        });
    }


    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadEquipoGrupo();
        this.applyPolicies();
        this.loadRedesTelefonicas();
    }

    onCreatingItem() {
        this.dispositivo.equipo.idEquipoGrupo = this.idEquipoGrupo;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.dispositivoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.dispositivo = info;  

                this.configuracionesCollection.sourceCollection = info.configuraciones;
                this.configuracionesCollection.refresh;

                this.politicasCollection.sourceCollection = info.politicas;
                this.politicasCollection.refresh;

                this.aplicacionesCollection.sourceCollection = info.aplicaciones;
                this.aplicacionesCollection.refresh;


                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.dispositivoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.dispositivo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.dispositivoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.dispositivo = info;   

                this.configuracionesCollection.sourceCollection = info.configuraciones;
                this.configuracionesCollection.refresh;

                this.politicasCollection.sourceCollection = info.politicas;
                this.politicasCollection.refresh;

                this.aplicacionesCollection.sourceCollection = info.aplicaciones;
                this.aplicacionesCollection.refresh;


            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.dispositivoService.update(this.dispositivo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.dispositivo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.dispositivo = info;
            })
            .catch(error => this.handleError(error));        
    }

 
    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onRemoveAplicacion(item) {
        this.aplicacionesCollection.remove(item);
        this.aplicacionesCollection.refresh();
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD-APLICACION') {

            var aplicacionNew: DispositivoAplicacionItem = new DispositivoAplicacionItem();
            var aplicacionInf: AplicacionInfo = new AplicacionInfo();

            aplicacionNew.idAplicacion = this.selectorAplicacion.selectedItem.identity;
            aplicacionNew.package = this.selectorAplicacion.selectedItem.package;
            aplicacionNew.nombre = this.selectorAplicacion.selectedItem.nombre;
            aplicacionNew.aplicacion = this.selectorAplicacion.selectedItem;

            var aplicacionesSlice = this.dispositivo.aplicaciones.slice();

            aplicacionesSlice.push(aplicacionNew);

            this.dispositivo.aplicaciones = aplicacionesSlice;

            this.aplicacionesCollection.sourceCollection = aplicacionesSlice;
            this.aplicacionesCollection.refresh;

        }


    }

}