﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection} from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { LlantaService} from '../services/llanta.service';
import { LlantaInfo, LlantaItem } from '../models/llanta';
import { LlantaPosicionService } from '../services/llanta-posicion.service';
import { LlantaPosicionItem } from '../models/llanta-posicion'
import { RefaccionInfo } from '../models/refaccion'
import { RefaccionArticulo } from '../models/refaccion-articulo';
import { MarcaInfo } from '../models/marcas';
import { MarcaService } from '../services/marcas.service';


@Component({
    selector: 'mantenimiento-llanta-editor',
    templateUrl: './llantas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => LlantasEditor)
        },
        {
            provide: CatalogService, useExisting: LlantaService
        }
    ]
})
export class LlantasEditor extends CatalogEditor {
    llanta: LlantaInfo = new LlantaInfo();        
    idEquipoGrupo: number = 0;
    idMarca: number = 0;
    marca: MarcaInfo = new MarcaInfo();
    posiciones: LlantaPosicionItem[] = [];
    //
    articulosCollection: TrackableCollection = new TrackableCollection();

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private llantaService: LlantaService,
        private llantaPosicionService: LlantaPosicionService, private marcaService: MarcaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'idLocalizacion': [0],
            'numEconomico': [''],
            'numSerie': [''],
            'idLlantaPosicion': [0],
            'profundidad': [0],
            'refaccion': [0],
            'idRefaccion': [0],
            'idVehiculo': [0],
            'dOT': ['']

        });
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipoGrupo = params['idEquipoGrupo'];
            this.idMarca = params['idMarca'];
        });
    }


	///////////////////////// METODOS ///////////////////////////////////////

    loadPosiciones() {
        this.llantaPosicionService.loadList(this.context.app.defaultSite.identity)
            .then(pos => this.posiciones = pos)
            .catch(error => this.handleError(error));
    }

    loadMarca() {
        this.marcaService.getDetailFromCache(this.idMarca)
            .then(mar => this.marca = mar)
            .catch(error => this.handleError(error));
    }

    onSelectedRefaccionChanged(refaccion: any) {
        var ref: RefaccionInfo;
        ref = <RefaccionInfo>refaccion;

        this.llanta.pieza.idRefaccion = ref.identity;

        let articulosMarca = ref.articulos.filter(item => item.idMarca == this.idMarca);

        this.articulosCollection.sourceCollection = articulosMarca;
        this.articulosCollection.refresh;
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadPosiciones();
        this.loadMarca();
    }

    onCreatingItem() {
        this.llanta.pieza.equipo.idEquipoGrupo = this.idEquipoGrupo;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.llantaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.llanta = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.llantaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.llanta = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.llantaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.llanta = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();

        var arts: RefaccionArticulo[];
        arts = this.articulosCollection.sourceCollection;
        let articulo = arts.filter(item => item.predeterminado);
        if (articulo.length == 1) {
            this.llanta.pieza.idArticuloStock = articulo[0].idArticuloStock;
        }
        else {
            this.showAlert("Debe seleccionar un solo artículo como refacción", "danger");
        }
        
        this.llantaService.update(this.llanta)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.llanta.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.llanta = info;
            })
            .catch(error => this.handleError(error));        
    }

}