﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
import { Control, CollectionView } from 'wijmo/wijmo';
//Bootstrap

//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { MarcaService } from '../services/marcas.service';
import { MarcaInfo, MarcaItem } from '../models/marcas';
import { TipoServicioService } from '../services/tipos-servicio.service';
import { TipoServicioItem } from '../models/tipos-servicio';

import { SubMarca } from '../models/submarca';
import { SubMarcaEditor } from '../parts/submarca.editor';

@Component({
    selector: 'maintenance-marcas-editor',
    templateUrl: './marcas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => MarcasEditor)
        },
        {
            provide: CatalogService, useExisting: MarcaService
        }
    ]
})
export class MarcasEditor extends CatalogEditor {
    marca: MarcaInfo = new MarcaInfo();
    tiposServicio: TipoServicioItem[] = [];
    currentSubMarca: SubMarca = new SubMarca();
    subMarcasCollection: TrackableCollection = new TrackableCollection();
    //
    @ViewChild('submarcaEditor') submarcaEditor: SubMarcaEditor;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, formBuilder: FormBuilder,
        protected context: Context, private location: Location, private marcaService: MarcaService,
        protected changeDetector: ChangeDetectorRef, private tipoService: TipoServicioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idServicioTipo': [0, AztecaValidators.selected]
        });        
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadTiposServicio();
    }

    loadTiposServicio() {
        this.tipoService.loadList()
            .then(tipos => this.tiposServicio = tipos)
            .catch(error => this.handleError(error));
    }

    eliminarSubMarca(idSubMarca: number) {
        this.marcaService.deleteSubMarca(idSubMarca)
            .catch(error => this.handleError(error))
    }

    onCreatingItem() {
        this.subMarcasCollection.sourceCollection = this.marca.subMarcas
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.marcaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.marca = info;
                this.subMarcasCollection.sourceCollection = this.marca.subMarcas;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.marcaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.marca = info;
                this.subMarcasCollection.sourceCollection = this.marca.subMarcas;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.marcaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.marca = info;
                this.subMarcasCollection.sourceCollection = this.marca.subMarcas;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.marcaService.update(this.marca)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.marca.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.marca = info;
            })
            .catch(error => this.handleError(error));
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.currentSubMarca = new SubMarca();
            this.submarcaEditor.show();
        }
        
    }



    selectMarcasTab(args) {
        if (args.heading) {
            //refresca los controles de wijmo
            Control.invalidateAll();
        }
    }

    onSubMarcaDelete(current: SubMarca) {
        this.subMarcasCollection.remove(current);
    }

    onSubmarcaUpdated(submarca: SubMarca) {
        let newSubMarca: SubMarca[] = this.marca.subMarcas.slice();
        newSubMarca.push(submarca);
        this.marca.subMarcas = newSubMarca;
        this.subMarcasCollection.sourceCollection = newSubMarca;
    }
}


