﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection, TrackableItem } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { Clasificacion } from 'framework3/general';
//Features
import { VehiculoService } from '../services/vehiculo.service';
import { VehiculoInfo, VehiculoItem, VehiculoLlantaItem } from '../models/vehiculo';

import { GrupoVehiculosService } from '../services/grupo-vehiculos.service';
import { GrupoVehiculosInfo, GrupoVehiculosItem } from '../models/grupo-vehiculos';

import { LlantaPosicionService } from '../services/llanta-posicion.service';
import { LlantaPosicionItem } from '../models/llanta-posicion'
import { LocalizacionInfo, LocalizacionItem, LocalizacionSelectorComponent, LlantasSelectorComponent } from '../../maintenance';

@Component({
    selector: 'mantenimiento-vehiculos-editor',
    templateUrl: './vehiculos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => VehiculosEditor)
        },
        {
            provide: CatalogService, useExisting: VehiculoService
        }
    ]
})
export class VehiculosEditor extends CatalogEditor {
    vehiculo: VehiculoInfo = new VehiculoInfo();
    idEquipoGrupo: number = 0;
    idGrupoVehiculo: number = 0;
    vehiculoGrupo: GrupoVehiculosInfo = new GrupoVehiculosInfo();
    clasificadores: Clasificacion[] = [];
    idLlanta: number = 0;

    posiciones: LlantaPosicionItem[] = [];
    idLlantaPosicion: number = 0;
    llantasCollection: TrackableCollection = new TrackableCollection();


    @ViewChild('selectorLocalizacion') selectorMarca: LocalizacionSelectorComponent;
    @ViewChild('selectorLlanta') selectorLlanta: LlantasSelectorComponent;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private vehiculoService: VehiculoService, private grupoVehiculoService: GrupoVehiculosService,
        private llantaPosicionService: LlantaPosicionService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': [''],
            'placas': ['', Validators.required],
            'color': ['', Validators.required],
            'perfil': ['QUEDA PENDIENTE HASTA COMPLETAR MODULO DE MANTENIMIENTO'],
            'numEconomico': ['', Validators.required],
            'numSerie': ['', Validators.required],
            'numSerieMotor': ['', Validators.required],
            'idLocalizacion': [0, Validators.required],
            'idLlanta': [0],
            'idLlantaPosicion':[0],
            'clasificaciones': [[], AztecaValidators.validClasifications]
        });
    }
    //TODO: Implementar Combo de Perfil de Mantenimiento al terminar dicho catálogo

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idGrupoVehiculo = params['idVehiculoGrupo'];
            this.idEquipoGrupo = params['idEquipoGrupo'];
        });
    }

    loadClasificadores() {
        this.vehiculoService.loadClassifiers(this.context.app.defaultSite.identity)
            .then(clasificadores => this.clasificadores = clasificadores)
            .catch(error => this.handleError(error));
    }

    loadVehiculoGrupo() {
        this.grupoVehiculoService.getDetailByID(this.idGrupoVehiculo)
            .then(info => {
                this.vehiculoGrupo=info;
            })
            .catch(error => this.handleError(error));
    }

  

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadClasificadores();
        this.loadVehiculoGrupo();
        this.loadPosiciones();
    }

    onCreatingItem() {
        this.vehiculo.equipo.idEquipoGrupo = this.idEquipoGrupo;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.vehiculoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.vehiculo = info;  

                this.llantasCollection.sourceCollection = info.llantas;
                this.llantasCollection.refresh;

                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.vehiculoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.vehiculo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.vehiculoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.vehiculo = info;   

                this.llantasCollection.sourceCollection = info.llantas;
                this.llantasCollection.refresh;


            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.vehiculoService.update(this.vehiculo)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.vehiculo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.vehiculo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'DOCUMENTOS') {
            this.router.navigate(['maintenance/documentos', this.vehiculo.idEquipo, this.process.id]);
        }
        if (item.name == 'FOTOGRAFIAS') {
            this.router.navigate(['maintenance/fotografias', this.vehiculo.idEquipo, this.process.id]);
        }

    }


    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onRemoveLlanta(item) {
        this.llantasCollection.remove(item);
        this.llantasCollection.refresh();
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD-LLANTA') {
            var llantaNew: VehiculoLlantaItem = new VehiculoLlantaItem();
            var vehiculoInf: VehiculoInfo = new VehiculoInfo();
          
            llantaNew.idLlanta = this.selectorLlanta.selectedItem.identity;
            llantaNew.idLlantaPosicion = this.selectorLlanta.selectedItem.idLlantaPosicion;
            llantaNew.modelo = this.selectorLlanta.selectedItem.pieza.equipo.grupo.modelo;
            llantaNew.nombre = this.selectorLlanta.selectedItem.pieza.equipo.grupo.nombre;
            llantaNew.numEconomico = this.selectorLlanta.selectedItem.pieza.equipo.numEconomico;
            llantaNew.numSerie = this.selectorLlanta.selectedItem.pieza.equipo.numSerie;
            llantaNew.posicion = this.selectorLlanta.selectedItem.posicion.nombre;

            var llantasSlice = this.vehiculo.llantas.slice();

            llantasSlice.push(llantaNew);

            this.vehiculo.llantas = llantasSlice;

            this.llantasCollection.sourceCollection = llantasSlice;
            this.llantasCollection.refresh;

        }
    }

    loadPosiciones() {
        this.llantaPosicionService.loadList(this.context.app.defaultSite.identity)
            .then(pos => this.posiciones = pos)
            .catch(error => this.handleError(error));
    }


    onPosicionChanged(idPosicion) {
        this.llantaPosicionService.getDetailByID(idPosicion)
            .then(pos => {

                var vehiculoLlanta: VehiculoLlantaItem = this.vehiculo.llantas.find(i => i.idLlanta == this.llantasCollection.currentItem.idLlanta)
                vehiculoLlanta.idLlantaPosicion = pos.identity;
                vehiculoLlanta.posicion = pos.nombre;
                vehiculoLlanta.state = 'MODIFIED';

                this.llantasCollection.sourceCollection = this.vehiculo.llantas;
                this.llantasCollection.refresh;
                Control.invalidateAll();

            })
            .catch(error => this.handleError(error));
        
    }


}