﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

import { Control } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellRange, CellType, Column, CellRangeEventArgs } from 'wijmo/wijmo.grid';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ProductoService } from '../services/producto.service';
import { ProductoInfo, ProductoItem } from '../models/producto';

import { ArticuloStockService, UnidadMedidaService, UnidadMedidaTipoService, UnidadMedidaTipoItem, UnidadMedidaItem } from '../../inventory';
import { ProductoInsumo } from '../models/producto-insumo';
import { ProductoInsumoEditor } from '../parts/producto-insumo.editor';
import { ProductoSubproducto } from '../models/producto-subproducto';
import { ProductoSubproductoEditor } from '../parts/producto-subproducto.editor';

@Component({
    selector: 'produccion-productos-editor',
    templateUrl: './productos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ProductosEditor)
        },
        {
            provide: CatalogService, useExisting: ProductoService
        }
    ]
})
export class ProductosEditor extends CatalogEditor {
    producto: ProductoInfo = new ProductoInfo();
    unidadesMedidaTipo: UnidadMedidaTipoItem[] = [];
    unidadesMedida: UnidadMedidaItem[] = [];
    idArticuloStock: number;
    codigo: string;

    subProductosCollection: TrackableCollection = new TrackableCollection();
    insumosCollection: TrackableCollection = new TrackableCollection();
    //Current items
    currentInsumo: ProductoInsumo = new ProductoInsumo();
    currentSubproducto: ProductoSubproducto = new ProductoSubproducto();
    //Editors
    @ViewChild('menuInsumos') menuInsumos: Menu;
    @ViewChild('menuSubProductos') menuSubProductos: Menu;

    @ViewChild('insumoEditor') insumoEditor: ProductoInsumoEditor;
    @ViewChild('subproductoEditor') subproductoEditor: ProductoSubproductoEditor;
    //


    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef, private articuloStockService:ArticuloStockService,
        protected context: Context, private location: Location, private productoService: ProductoService, private unidadMedidaTipoService: UnidadMedidaTipoService, private unidadMedidaService: UnidadMedidaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idUnidadMedidaTipo': ['', Validators.required],
            'idUnidadMedida': ['', Validators.required],
            'idUnidadMedidaInv': ['', Validators.required],
            'GTIN': [''],
            'produccionMinima': ['', Validators.required],
            'multiplosProduccion': ['', Validators.required]
        });
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idArticuloStock = +params['idArticuloStock'];
            this.codigo = params['codigo'];
        });
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.loadUnidadesMedidaTipo();
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        if (this.codigo && this.codigo != '') {
            this.producto.code = this.codigo;
        }
        if (this.idArticuloStock > 0) {
            this.articuloStockService.getDetailByID(this.idArticuloStock)
                .then(articulo => this.producto.articulo = articulo)
                .catch(error => this.handleError(error));                
        }
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.productoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.producto = info;
                this.insumosCollection.sourceCollection = this.producto.insumos;
                this.subProductosCollection.sourceCollection = this.producto.subProductos;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.productoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.producto = info;
                this.insumosCollection.sourceCollection = this.producto.insumos;
                this.subProductosCollection.sourceCollection = this.producto.subProductos;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.productoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.producto = info;
                this.insumosCollection.sourceCollection = this.producto.insumos;
                this.subProductosCollection.sourceCollection = this.producto.subProductos;
                this.selectUnidadTipo(this.producto.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.productoService.update(this.producto)
            .then(info => {
                this.context.app.hideSpinner();
                this.producto = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    loadUnidadesMedidaTipo() {
        this.unidadMedidaTipoService.loadList()
            .then(unidades => this.unidadesMedidaTipo = unidades)
            .catch(error => this.handleError(error));
    }

    selectUnidadTipo(idUnidadMedida) {
        this.unidadMedidaService.loadList(idUnidadMedida)
            .then(unidades => this.unidadesMedida = unidades)
            .catch(error => this.handleError(error));
    }

    ////
    onTabSelect(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }
   

    ///////////// SUBPRODUCTOS///////////////////////
    gridSubPrductosinitialized(sender: FlexGrid, args: any) {
        var self = this;
        //Agregar manejador para menú contextual        
        sender.addEventListener(sender.hostElement, "contextmenu", (e: any) => {
            let ht: HitTestInfo = sender.hitTest(e);
            sender.select(new CellRange(ht.row, ht.col), true);
        });
    }

    //responde al click del boton añadir del tool bar para abrir ventana del editor de un nuevo subproducto
    onSubproductoToolbar(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.currentSubproducto = new ProductoSubproducto();
            this.currentSubproducto._clientID = 0;
            this.subproductoEditor.show();
        }
    }   

    onMenuItemSubProductosClicked(menu: Menu, args: any) {
        if (menu.selectedIndex == 0) {
            //Editar
            this.currentSubproducto = this.copySubProducto(this.subProductosCollection.currentItem);            
            this.subproductoEditor.show();
        }
        if (menu.selectedIndex == 2) {
            //Eliminar
            this.subProductosCollection.remove(this.subProductosCollection.currentItem);
        }
    }

    //cachar evento de que el editor de subproductos se ha aceptado  
    onSubproductoUpdated(subproducto: ProductoSubproducto) {
        let newSubproductos: ProductoSubproducto[] = this.producto.subProductos.slice();
        if (subproducto._clientID == 0) {
            subproducto._clientID = this.producto.subProductos.length + 1;
            newSubproductos.push(subproducto)
        }
        else {
            let index: number = newSubproductos.findIndex(item => item._clientID == subproducto._clientID);
            newSubproductos[index] = subproducto;
        }
              
        this.producto.subProductos = newSubproductos;
        this.subProductosCollection.sourceCollection = newSubproductos;
    }

    copySubProducto(original: ProductoSubproducto): ProductoSubproducto {
        return JSON.parse(JSON.stringify(original));
    }

    ///////////// INSUMOS ////////////////////////////
    gridInsumosinitialized(sender: FlexGrid, args:any) {
        var self = this;
        //Agregar manejador para menú contextual        
        sender.addEventListener(sender.hostElement, "contextmenu", (e: any) => {
            let ht: HitTestInfo = sender.hitTest(e);
            sender.select(new CellRange(ht.row, ht.col), true);            
        });
    }

    //responde al click del boton añadir del tool bar para abrir ventana del editor de un nuevo insumo
    onInsumosToolbar(button: ToolbarButton) {
        if (button.name == 'ADD') {            
            this.currentInsumo = new ProductoInsumo();            
            this.currentInsumo._clientID = 0;
            this.insumoEditor.show();
        }
    }    
    onMenuItemInsumoClicked(menu: Menu, args: any) {
        if (menu.selectedIndex == 0) {
            //Editar
            this.currentInsumo = this.copyInsumo(this.insumosCollection.currentItem);           
            this.insumoEditor.show();
        }
        if (menu.selectedIndex == 2) {
            //Eliminar
            this.insumosCollection.remove(this.insumosCollection.currentItem);
        }
    }

    //cachar evento de que el editor de insumos se ha aceptado  
    onInsumoUpdated(insumo: ProductoInsumo) {
        let newInsumos: ProductoInsumo[] = this.producto.insumos.slice();  
        if (insumo._clientID == 0) {
            insumo._clientID = this.producto.insumos.length + 1;
            newInsumos.push(insumo);
        }
        else {
            let index: number = newInsumos.findIndex(item => item._clientID == insumo._clientID);
            newInsumos[index] = insumo;
        }
        this.producto.insumos = newInsumos;
        this.insumosCollection.sourceCollection = newInsumos;
    }

    copyInsumo(original: ProductoInsumo): ProductoInsumo {
        return JSON.parse(JSON.stringify(original));
    }
   
   

}