﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { CentralService} from '../services/central.service';
import { CentralInfo, CentralItem} from '../models/central';
import { CajaService } from '../../treasury/services/caja.service';
import { CajaItem } from '../../treasury/models/caja';
import { ZonaCompraService } from '../services/zona-compra.service';
import { ZonaCompraItem } from '../models/zona-compra';


@Component({
    selector: 'compras-central-editor',
    templateUrl: './centrales.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CentralesEditor)
        },
        {
            provide: CatalogService, useExisting: CentralService
        }
    ]
})
export class CentralesEditor extends CatalogEditor {
    central: CentralInfo = new CentralInfo(); 
    cajas: CajaItem[] = [];
    zonasCompra: ZonaCompraItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private centralService: CentralService, private cajaService: CajaService, private zonaCompraService: ZonaCompraService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idCaja': ['', AztecaValidators.selected],
            'idZonaCompra': ['', AztecaValidators.selected],
            'idRemitente': ['', AztecaValidators.selected],
            'idOperador': [''],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadCajas();
        this.loadZonasCompra();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.centralService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.central = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.centralService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.central = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.centralService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.central = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.centralService.update(this.central)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.central.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.central = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadCajas() {
        this.cajaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cajas => {
                this.cajas = cajas;
            })
            .catch(error => this.showAlert(error, 'danger'));
    }

    loadZonasCompra() {
        this.zonaCompraService.loadListByUser(this.context.app.defaultSite.identity)
            .then(zonas => {
                this.zonasCompra = zonas;
            })
            .catch(error => this.showAlert(error, 'danger'));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['members'], { relativeTo: this.route });
        }
        else if (option.name == 'DEPTOS') {
            this.router.navigate(['departamentos'], { relativeTo: this.route });
        }
    }

}