﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { CentroCostoService} from '../services/centro-costo.service';
import { CentroCostoInfo, CentroCostoItem} from '../models/centro-costo';

import { SegmentoNegocioService } from '../../accounting/services/segmento-negocio.service';
import { SegmentoNegocioItem} from '../../accounting/models/segmento-negocio';
import { Servicio, Articulo } from '../models/categorias-presupuestales';
import { ArticuloItem } from '../models/articulo';
import { ServicioItem } from '../models/servicio';
import { Control } from 'wijmo/wijmo';
import { ServicioListComponent } from '../dialogs/servicio-list.component';
import { ArticuloListComponent } from "../dialogs/articulo-list.component";

@Component({
    selector: 'compras-centro-costo-editor',
    templateUrl: './centros-costo.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CentrosCostoEditor)
        },
        {
            provide: CatalogService, useExisting: CentroCostoService
        }
    ]
})
export class CentrosCostoEditor extends CatalogEditor {
    centroCosto: CentroCostoInfo = new CentroCostoInfo();        
    segmentosNegocio: SegmentoNegocioItem[] = [];

    articulosCollection: TrackableCollection = new TrackableCollection();
    serviciosCollection: TrackableCollection = new TrackableCollection();

    @ViewChild(forwardRef(() => ArticuloListComponent)) articuloDialog: ArticuloListComponent;
    @ViewChild(forwardRef(() => ServicioListComponent)) servicioDialog: ServicioListComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private centroCostoService: CentroCostoService, private segmentoNegocioService: SegmentoNegocioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idSegmentoNegocio': [0, AztecaValidators.selected],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadSegmentosNegocio();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.centroCostoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.centroCosto = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.centroCostoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.centroCosto = info;

                this.articulosCollection.sourceCollection = this.centroCosto.articulos;
                this.articulosCollection.refresh();

                this.serviciosCollection.sourceCollection = this.centroCosto.servicios;
                this.serviciosCollection.refresh();
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.centroCostoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.centroCosto = info;        

                this.articulosCollection.sourceCollection = this.centroCosto.articulos;
                this.articulosCollection.refresh();

                this.serviciosCollection.sourceCollection = this.centroCosto.servicios;
                this.serviciosCollection.refresh();
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.centroCostoService.update(this.centroCosto)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.centroCosto.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
                this.centroCosto = info;

                this.articulosCollection.sourceCollection = this.centroCosto.articulos;
                this.articulosCollection.refresh();

                this.serviciosCollection.sourceCollection = this.centroCosto.servicios;
                this.serviciosCollection.refresh();
            })
            .catch(error => this.handleError(error));        
    }

    loadSegmentosNegocio() {
        this.segmentoNegocioService.loadList(this.context.app.defaultCompany.identity)
            .then(items => this.segmentosNegocio = items)
            .catch(error => this.handleError(error));        
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onToolbarButtonClick(args: ToolbarButton) {
        if (args.name == 'ADD-ARTICULO')
            this.onAgregarArticulo();
        else if (args.name == 'ADD-SERVICIO')
            this.onAgregarServicio();
    }

    onAgregarArticulo() {
        this.articuloDialog.show();
    }

    onAgregarServicio() {
        this.servicioDialog.show();
    }

    onSelectedArticuloChanged(articulos: ArticuloItem[]) {
        for (var articulo of articulos) {
            if (this.centroCosto.articulos.filter(X => X.id == articulo.id).length == 0) {
                var articulosSlice = this.centroCosto.articulos.slice();
                var articuloNew = new Articulo();
                articuloNew.id = articulo.id;
                articuloNew.codigo = articulo.codigo;
                articuloNew.nombre = articulo.nombre;
                articuloNew.activo = true;
                articulosSlice.push(articuloNew);

                this.centroCosto.articulos = articulosSlice;
                this.articulosCollection.sourceCollection = articulosSlice;
                this.articulosCollection.refresh();
            }
        }
    }

    onSelectedServicioChanged(servicios: ServicioItem[]) {
        for (var servicio of servicios) {
            if (this. centroCosto.servicios.filter(X => X.id == servicio.id).length == 0) {
                var serviciosSlice = this.centroCosto.servicios.slice();
                var servicioNew = new Servicio();
                servicioNew.id = servicio.id;
                servicioNew.codigo = servicio.codigo;
                servicioNew.nombre = servicio.nombre;
                servicioNew.activo = true;
                serviciosSlice.push(servicioNew);

                this.centroCosto.servicios = serviciosSlice;
                this.serviciosCollection.sourceCollection = serviciosSlice;
                this.serviciosCollection.refresh();
            }
        }
    }

    onRemoveArticulo(item) {
        this.articulosCollection.remove(item);
        this.articulosCollection.refresh();
    }

    onRemoveServicio(item) {
        this.serviciosCollection.remove(item);
        this.serviciosCollection.refresh();
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['members'], { relativeTo: this.route });
        }
    }

}