﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, WorkflowStep} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { FormatoTipoService, FormatoItem } from 'framework3/general';

//Features
import { EmpresaService} from '../services/empresa.service';
import { EmpresaInfo, EmpresaItem } from '../models/empresa';
import { SucursalItem } from '../models/sucursal';
import { SucursalService } from '../services/sucursal.service';
import { EjercicioTipoItem } from '../../accounting/models/ejercicio-tipo';
import { EjercicioTipoService } from '../../accounting/services/ejercicio-tipo.service';

import { Control } from 'wijmo/wijmo';
import { DocumentTypeService } from 'framework3/core/services/document-type.service'

@Component({
    selector: 'compras-empresa-editor',
    templateUrl: './empresas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EmpresasEditor)
        },
        {
            provide: CatalogService, useExisting: EmpresaService
        }
    ]
})
export class EmpresasEditor extends CatalogEditor {
    empresa: EmpresaInfo = new EmpresaInfo();    
    @Input() sucursales: SucursalItem[] = [];
    formatosImpresion: FormatoItem[] = [];
    ejerciciosTipos: EjercicioTipoItem[] = [];
    ordenSteps: WorkflowStep[] = [];
    requisicionesSteps: WorkflowStep[] = [];

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private empresaService: EmpresaService, private sucursalService: SucursalService, private formatoService: FormatoTipoService, private ejercicioTipoService: EjercicioTipoService,
        private documentTypeService: DocumentTypeService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': [''],
            'nombre': [''],
            'idRemitentePortal': [0],
            'idRemitenteOrdenes': [0],
            'idFormatoPresupuesto':[0],
            'idFormatoOrden':[0],
            'correoRecepcionCFD': ['', AztecaValidators.email],
            'servidorPopCFD': [''],
            'puertoPopCFD': [0],
            'passwordPopCFD': [''],
            'idEjercicioTipo': [0, AztecaValidators.selected],
            'idWorkflowStepRequisicionOrdenar': [0, AztecaValidators.selected],
            'idWorkflowStepOrdenRecibir': [0, AztecaValidators.selected],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();        
        this.loadEjerciciosTipos();
        this.loadOrdenesSteps();
        this.loadRequisicionesSteps();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;            
                this.loadSucursales();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.empresaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;           
                this.loadSucursales();
                this.loadFormatosImpresion();
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.empresaService.update(this.empresa)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.empresa.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.empresa = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadSucursales() {
        let request: SearchRequest = new SearchRequest();
        request.paged = false;
        //Agregar condición de empresa actual
        let condicion: SearchCondition = { field: "IdEmpresa", operator: "=", valueType: "int", value: this.empresa.identity.toString() };
        request.conditions.push(condicion);
        //
        this.context.app.showSpinner();
        this.sucursalService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.sucursales = response.rows;
                console.log(JSON.stringify(this.sucursales));
            })
            .catch(error => this.handleError(error));
    }

    loadFormatosImpresion() {
        this.formatoService.loadFormatos('PURCHASING_FORMATOS', this.empresa.identity)
            .then(formatos => {
                this.formatosImpresion = formatos;
                this.formatosImpresion.unshift({ id: 0, nombre: '[NINGUNO]', activo: true, codigo: '' });
            })
            .catch(error => this.handleError(error));
    }

    //
    //Manejo de Estados
    selectSucursalesTab(arg: any) {
        Control.invalidateAll();
    }
    //
    loadEjerciciosTipos() {
        this.ejercicioTipoService.loadList(this.context.app.defaultSite.identity)
            .then(tipos => this.ejerciciosTipos = tipos)
            .catch(error => this.handleError(error));
    }

    loadOrdenesSteps() {
        this.documentTypeService.getDetailByCode(this.context.app.defaultSite.identity,"PURCHASING_ORDER")
            .then(orden => this.ordenSteps = orden.workflowSteps)
            .catch(error => this.handleError(error));
    }

    loadRequisicionesSteps() {
        this.documentTypeService.getDetailByCode(this.context.app.defaultSite.identity, "PURCHASING_REQ")
            .then(req => this.requisicionesSteps = req.workflowSteps)
            .catch(error => this.handleError(error));
    }

}