﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { AdelantoInfo, AdelantoItem } from '../models/adelanto';
import { AdelantoDetail } from '../models/adelanto-detail';
import { AdelantoService } from '../services/adelanto.service';
import { AdelantoDetailEditor } from './parts/adelantos-detail.editor';
import { OrdenService } from '../services/orden.service';
import { OrdenInfo, OrdenItem } from '../models/orden';
import { OrdenDetail } from '../models/orden-detail';
import { OrdenDetailArticulo } from '../models/orden-detail-articulo';
import { OrdenDetailServicio } from '../models/orden-detail-servicio';
import { ImportesInfo } from '../../accounting/models/importes';


@Component({
    selector: 'compras-adelantos-editor',
    templateUrl: './adelantos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AdelantosEditor)
        },
        {
            provide: DocumentService, useExisting: AdelantoService
        }
    ]
})
export class AdelantosEditor extends DocumentEditor {
    adelanto = new AdelantoInfo();
    orden = new OrdenInfo();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: AdelantoDetailEditor;
   
    get proveedorText() {
        return this.orden.proveedor.code + '-' + this.orden.proveedor.nombre;
    }

    get importes() {
        let importesTMP = this.importesDesglosados();
        return importesTMP;
    }

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private adelantoService: AdelantoService,
        private ordenService: OrdenService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'porcentaje': [0, Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {
        //Cargar parámetros que se proporcionan cuando es un nuevo documento
        this.route.params.forEach(params => {            
            this.adelanto.idOrden = params['idOrden'];
        });

        setTimeout(_ => this.cargarInfomacionEntidades());
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.adelantoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.adelanto = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.adelantoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.adelanto = doc;
                this.detailsEditor.clientIndexStart = this.adelanto.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.adelantoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.adelanto = doc;
                this.cargarInfomacionEntidades();
                this.detailsEditor.clientIndexStart = this.adelanto.details.length;
                if (this.adelanto.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.adelantoService.update(this.adelanto)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.adelanto.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.location.back();
                        //this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.adelanto = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.adelantoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.adelantoService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onCancelingEdit() {
        this.location.back();
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.adelanto.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new AdelantoDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: AdelantoDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: AdelantoDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: AdelantoDetail[]) {
        this.adelanto.details = updatedDetails;
    }

    onSavingDetail(detail: AdelantoDetail) {
        this.detailsList.updateItem(detail);
    }


    ///////////////////////////////////metodos generados
    cargarInfomacionEntidades() {     
        this.ordenService.getDetail(this.adelanto.idOrden)
            .then(orden => {
                this.orden = orden;
            })
            .catch(error => this.handleError(error));
    }

    importesDesglosados(): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        let elementos = this.combineDetail().filter(item => item.state != 'DELETED');

        elementos.forEach(item => {
            desglose.subtotal += item.costoUnitario * item.cantidad;
            desglose.descuentos += (item.descuentos.descuento1 + item.descuentos.descuento2 + item.descuentos.descuento3 + item.descuentos.descuento4 + item.descuentos.descuento5) * item.cantidad;

            if (item.impuestos.filter(tras => tras.tipoAplicacion == "T").length > 0)
                desglose.impuestosTrasladados += item.impuestos.filter(tras => tras.tipoAplicacion == "T").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "R").length > 0)
                desglose.impuestosRetenidos += item.impuestos.filter(tras => tras.tipoAplicacion == "R").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
        });

        desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;

        //this.totalOC = desglose.total;
        //this.visualValidacion();
        return desglose;
    }

    combineDetail(): OrdenDetail[] {
        let articulos = this.orden.details.map(d => <OrdenDetailArticulo>d).filter(d => d.idArticulo && d.idArticulo > 0);
        let servicios = this.orden.details.map(d => <OrdenDetailServicio>d).filter(d => d.idServicio && d.idServicio > 0);

        let defaultDate: Date = new Date(1901, 1, 1);
        let newDetail: OrdenDetail[] = [];

        servicios.forEach(s => newDetail.push(s));
        articulos.forEach(a => newDetail.push(a));
        
        return newDetail;
    }

}

