﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent, DownloadPopupComponent } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
//Locales
import { ComprobanteFiscalService as ComprobanteBaseService } from '../../accounting';
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service';
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';

@Component({
    selector: 'purchasing-almacen-digital',
    templateUrl: './almacen-digital.component.html'
})
export class AlmacenDigitalComponent extends ProcessComponent {
    form: FormGroup;
    menuItems: MenuItem[] = [];    
    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();
    pageSize: number = 20;
    maxRows: number = 3000;
    totalRows: number = 0;
    comprobantes: ComprobanteFiscalItem[] = [];
    selectedGroup: FilterGroup;
    filterGroups: FilterGroup[] = [];

    currentComprobante = new ComprobanteFiscalInfo();
    
    //
    @ViewChild(DownloadPopupComponent) downloadPopup: DownloadPopupComponent;
    @ViewChild('viewDialog') viewDialog: ModalDirective;
    @ViewChild(DocumentListComponent) documentList: DocumentListComponent;
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;


    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder,
        private comprobanteService: ComprobanteFiscalService, private comprobanteBaseService: ComprobanteBaseService) {
        super(context, router, route);
        //Crear formulario de parámetros
        this.form = formBuilder.group({
            'fechaInicial': [new Date()],
            'fechaFinal': [new Date()]
        });                        
    }

    ngOnInit() {
        this.createMenu();        
    }

    ngAfterViewInit() {
        this.fechaInicial = moment().subtract(7, 'day').toDate();
        this.fechaFinal = moment().add(1, 'day').toDate();
        setTimeout(_ => this.refreshList());
    }

    /////////////////////// METODOS ////////////////////////////////////

    refreshList() {
        this.context.app.showSpinner();

        //Disparar búsqueda inicial. Si el manejo se realiza del lado del servidor, obtener sólo la primer página, si es local obtener toda la información disponible
        let request: SearchRequest = new SearchRequest();
        request.pageIndex = 0;
        request.pageSize = this.pageSize;
        request.paged = false;
        request.maxRows = this.maxRows;
        //Agregar condiciones de fecha
        let condicionFechaIni: SearchCondition = { field: 'fechaComprobante', operator: '>=', valueType: ConditionValueType.DATE_TIME, value: moment(this.fechaInicial).format() };
        let condicionFechaFin: SearchCondition = { field: 'fechaComprobante', operator: '<=', valueType: ConditionValueType.DATE_TIME, value: moment(this.fechaFinal).format() };
        request.conditions.push(condicionFechaIni, condicionFechaFin);

        this.comprobanteService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.comprobantes = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    createMenu() {
        //Agregar opciones estándar del catálogo
        let mnuView: MenuItem = { icon: "glyphicon glyphicon-folder-open", text: "Ver", name: "OPEN", smallText: "ver resumen del comprobante" };
        let mnuPdf: MenuItem = { icon: "glyphicon glyphicon-download-alt", text: "Imprimir", name: "PDF", smallText: "generar PDF del comprobante" };
        let mnuXml: MenuItem = { icon: "glyphicon glyphicon-download-alt", text: "Descargar XML", name: "XML", smallText: "descargar en formato XML" };
        let mnuExport: MenuItem = { icon: "glyphicon glyphicon-export", text: "Exportar", name: "EXPORT", smallText: "exportar XML a sistema externo" };

        this.menuItems.push(mnuView, mnuPdf, /*mnuXml,*/ mnuExport);
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });

    }

    viewComprobante() {
        this.context.app.showSpinner();
        this.comprobanteService.getDetail(this.documentList.currentDocument.id)
            .then(comprobante => {                
                this.currentComprobante = comprobante;
                this.context.app.hideSpinner();
                this.viewDialog.config.backdrop = false;
                this.viewDialog.show();
            })
            .catch(error => this.handleError(error));
    }

    exportXml() {
        this.context.app.showSpinner();
        this.comprobanteService.export(this.documentList.currentDocument.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alertsBar.showAlert('El comprobante ha sido exportado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    /////////////////////// HANDLERS //////////////////////////////////

    onMainToolbar(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.router.navigate(['./upload'], { relativeTo: this.route });
        }
    }

    optionClick(option: MenuItem) {
        if (option.name == 'OPEN') {
            this.viewComprobante();
        }
        if (option.name == 'PDF') {
            this.context.app.showSpinner();
            this.comprobanteService.getDetail(this.documentList.currentDocument.id)
                .then(comprobante => {
                    this.comprobanteBaseService.generarPdfTemporal(comprobante.comprobanteBase.identity)
                        .then(file => {
                            this.context.app.hideSpinner();
                            window.open('/file-server/' + file.uri, '_blank');
                        })
                        .catch(error => this.handleError(error));            
                })
                .catch(error => this.handleError(error));
        }
        if (option.name == 'XML') {

        }
        if (option.name == 'EXPORT') {
            this.exportXml();
        }
    }

    onValidateMenu() {

    }

    currentItemChangedHandler() {

    }

    handleError(error: any) {
        console.log(error);
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'error');
        this.context.app.hideSpinner();
    }

    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }


}
