﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { ImportesInfo } from '../../accounting/models/importes';
import { GastoInfo, GastoItem } from '../models/gasto';
import { GastoDetail } from '../models/gasto-detail';
import { GastoComprobarItem } from '../models/gasto-comprobar-item';
import { GastoService } from '../services/gasto.service';
import { GastoDetailEditor } from './parts/gastos-detail.editor';


import { DivisaService } from '../../accounting/services/divisa.service';
import { DivisaInfo } from '../../accounting/models/divisa';
import { EmpresaService } from '../../accounting/services/empresa.service';
import { EmpresaInfo } from '../../accounting/models/empresa';

import { BeneficiarioSelectorComponent } from '../../treasury/controls/beneficiario-selector.component';
import { BeneficiarioInfo } from '../../treasury/models/beneficiario';

import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';


@Component({
    selector: 'compras-gastos-editor',
    templateUrl: './gastos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => GastosEditor)
        },
        {
            provide: DocumentService, useExisting: GastoService
        }
    ]
})
export class GastosEditor extends DocumentEditor {
    gasto = new GastoInfo();
    idDivisaDefault: number = 0;
    tipoCambio: string;
    previewCollectionDetallesComprobantes: CollectionView = new CollectionView();

    get importesG() {
        return this.importesDesglosados('G');
    }
    get importesC() {
        return this.importesDesglosados('C');
    }



    detailMode = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandSingle];
    detailModeMulti = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandMulti];

    //
    @ViewChild(AlertsBarComponent) alertsBarComprobante: AlertsBarComponent;

    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: GastoDetailEditor;
    @ViewChild('selectorBeneficiario') selectorBeneficiario: BeneficiarioSelectorComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private gastoService: GastoService,
        private divisaService: DivisaService,
        private empresaService: EmpresaService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idBeneficiario': [0, Validators.required],
            'idDivisa': [0, Validators.required],

        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.getDivisaDefault(false);


    }

    onCreatingDocument() {
        this.route.params.forEach(params => {
            if (params['idPresupuesto']) {
                this.gasto.idPresupuesto = params['idPresupuesto'];
            }
            if (params['idDepartamento']) {
                this.gasto.idDepartamento = params['idDepartamento'];
            }
            if (params['idGastoTipo']) {
                this.gasto.idGastoTipo = params['idGastoTipo'];
            }

        });
        this.getDivisaDefault(true);
        this.loadNewDetalle(this.gasto.idPresupuesto);

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.gastoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();

                this.previewCollectionDetallesComprobantes.sourceCollection = doc.comprobantes;

                this.gasto = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.gastoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.gasto = doc;
                this.detailsEditor.clientIndexStart = this.gasto.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.gastoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();

                this.previewCollectionDetallesComprobantes.sourceCollection = doc.comprobantes;

                this.gasto = doc;
                this.detailsEditor.clientIndexStart = this.gasto.details.length;
                if (this.gasto.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));

       
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();

        this.gasto.tipoBeneficiario = this.selectorBeneficiario.selectedItem.tipoBeneficiario;

        this.gastoService.update(this.gasto)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.gasto.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.gasto = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.gastoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.gastoService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.gasto.identity);
            }
            if (args.heading == 'Comprobacion') {
            }


            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        /*
        this.detailsEditor.detail = new GastoDetail();
        this.detailsEditor.show();
        */
    }

    onEditingDetail(detail: GastoDetail) {
        /*
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
        */
    }

    onDeletingDetail(detail: GastoDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: GastoDetail[]) {
        this.gasto.details = updatedDetails;
    }

    onSavingDetail(detail: GastoDetail) {
        this.detailsList.updateItem(detail);
    }

    loadNewDetalle(idPresupuesto: string) {
        this.gastoService.loadDetalleNew(idPresupuesto)
            .then(det => {
                this.gasto.details = det;
            })
            .catch(error => this.handleError(error));
    }

    getDivisaDefault(creating: boolean) {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(r => {
                this.idDivisaDefault = r.identity;
                if (creating) { this.gasto.idDivisa = r.identity; }

            })
            .catch(error => this.handleError(error));

    }

    ///metodos locales
    importesDesglosados(origen: string): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        if (origen == 'G') {

            let elementos = this.gasto.details.filter(item => item.state != 'DELETED');//this.combineDetail();

            if (elementos.length > 0)
                elementos.forEach(item => {
                desglose.subtotal += item.importe;
            });

            desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;

        }
        if (origen == 'C') {

            let elementos = this.gasto.comprobantes.filter(item => item.state != 'DELETED');//this.combineDetail();

            if (elementos.length > 0)
                elementos.forEach(item => {
                desglose.subtotal += item.subTotal;
                desglose.descuentos += item.descuentos;
                    desglose.impuestosTrasladados += item.impuestosTrasladados;
                    desglose.impuestosRetenidos += item.impuestosRetenidos;
            });

            desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;


        }
        return desglose;
    }    

    onBeneficiarioSelected(args: any) {
        var beneficiario: BeneficiarioInfo;
        beneficiario = args;
        if (!beneficiario.active) {
            this.showAlert("No no se puede hacer un gasto para un beneficiario inactivo", "danger");
            beneficiario = null;
            this.selectorBeneficiario.displayValue = null;
            this.selectorBeneficiario.selectedItem = null;
            this.selectorBeneficiario.valueMember = null;
            Control.invalidateAll();
        }
            
    }

    onDivisaSelected(args: any) {
        if (this.gasto.idDivisa == this.idDivisaDefault) {
            this.gasto.tipoCambio = 1;
            this.tipoCambio = this.gasto.tipoCambio.toString();
        }
        else {
            this.divisaService.loadTiposCambio(this.context.app.defaultCompany.identity)
                .then(tipos => {
                    let tCambio = tipos.find(search => search.idDivisaOrigen == this.gasto.idDivisa && search.idDivisaDestino == this.idDivisaDefault);
                    if (tCambio == undefined) {
                        this.showAlert("No está definido el tipo de cambio para la divisa seleccionada", "danger");
                        this.tipoCambio = null;
                        this.gasto.idDivisa = null;
                        this.gasto.tipoCambio = 0;
                    }
                    else {
                        this.empresaService.getDetailFromCache(this.context.app.defaultCompany.identity)
                            .then(emp => {
                                if (emp.tipoCambio == 'O') {
                                    this.gasto.tipoCambio = tCambio.precioOtro;
                                    this.tipoCambio = tCambio.precioOtro.toString();
                                }
                                else if (emp.tipoCambio == 'V') {
                                    this.gasto.tipoCambio = tCambio.precioVenta;
                                    this.tipoCambio = tCambio.precioVenta.toString();
                                }
                                else if (emp.tipoCambio == 'C') {
                                    this.gasto.tipoCambio = tCambio.precioCompra;
                                    this.tipoCambio = tCambio.precioCompra.toString();
                                }
                            })
                    }
                });
        }
    }



}

