﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import { MenuItem } from 'framework3/controls';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { GastoInfo, GastoItem } from '../models/gasto';
import { GastoDetail } from '../models/gasto-detail';
import { GastoService } from '../services/gasto.service';
import { PolizaPerfilListComponent, PolizaPerfilItem, PolizaInfo, PolizaService } from '../../accounting/index';
import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';

@Component({
    selector: 'compras-gastos-manager',
    templateUrl: './gastos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => GastosManager)
        },
        {
            provide: DocumentService, useExisting: GastoService
        }
    ]
})
export class GastosManager extends DocumentManager {
    gastos: GastoItem[];
    currentPoliza: PolizaInfo = new PolizaInfo();

    @ViewChild('viewDialog') dlgPolizaViewer: ModalDirective;
    @ViewChild(PolizaPerfilListComponent) dlgPolizaPerfil: PolizaPerfilListComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private gastoService: GastoService, private polizaSvc: PolizaService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.gastoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.gastos = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['new'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.gastoService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.gastoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.gastoService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        
        if (option.name == 'ASIGNAR') {
            this.gastoService.getDetail(this.manager.currentDocument.id)
                .then(gasto => {
                    if (gasto.currentStep.tag == 'AUTORIZAR') {
                        this.router.navigate(['purchasing/gasto-comprobar/' + this.process.id + '/' + this.manager.currentDocument.id]);
                    }
                    else {
                        this.showAlert('Para asignar comprobantes el gasto debe estar en Autorizado.', 'success');
                    }
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "CONTABILIZAR") {
            this.dlgPolizaPerfil.show();
        }
    }

    onSelectedPolizaPerfil(item: PolizaPerfilItem) {
        this.context.app.showSpinner();
        this.polizaSvc.loadPolizaByDocumento(this.manager.currentDocument.id, item.id, this.context.app.defaultSite.identity)
            .then(result => {
                this.currentPoliza = result;
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.config.backdrop = false;
                this.dlgPolizaViewer.show();
            }).catch(error => this.handleError(error));
    }

    onClickGuardar() {
        this.context.app.showSpinner();
        if (this.currentPoliza.idPolizaOrigen == 0) this.currentPoliza.idPolizaOrigen = 1 //Manual
        this.polizaSvc.update(this.currentPoliza)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.hide();
                this.manager.showAlert('La poliza ha sido registrada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }


}

