﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem, PolicyInfo } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Locales
import { PresupuestosInfo, PresupuestosItem } from '../models/presupuestos';
import { PresupuestosDetail } from '../models/presupuestos-detail';
import { PresupuestosService } from '../services/presupuestos.service';

@Component({
    selector: 'azteca.compras-presupuestos-manager',
    templateUrl: './presupuestos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PresupuestosManager)
        },
        {
            provide: DocumentService, useExisting: PresupuestosService
        }
    ]
})
export class PresupuestosManager extends DocumentManager {
    presupuestos: PresupuestosItem[];

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private presupuestosService: PresupuestosService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    applyPolicy(policy: PolicyInfo): void {
        if (policy.valor == 'BUDGET_CHANGE') {
            let mnuChange: MenuItem = { icon: "glyphicon glyphicon-random", text: "Cambiar presupuesto", name: "BUDGET_CHANGE", smallText: "Cambiar los montos del presupuesto", enabled: true };
            this.manager.menuItems.push(mnuChange);
        }
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.presupuestosService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.presupuestos = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.presupuestosService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.presupuestosService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.presupuestosService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    ////////////////////////////////////        ////////////////////////////////////////
    onMenuItemClick(option: MenuItem) {
        if (option.name == 'UNLOCK') { // abrir el presupuesto
            this.context.app.showSpinner();
            this.presupuestosService.unlock(this.manager.currentDocument.id)
                .then(_ => {
                    this.context.app.hideSpinner();
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }

        if (option.name == 'LOCK') { //cerrar presupuesto
            this.context.app.showSpinner();
            this.presupuestosService.lock(this.manager.currentDocument.id)
                .then(_ => {
                    this.context.app.hideSpinner();
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }

        if (option.name == 'BUDGET_CHANGE') { //Cambiar
            this.router.navigate([ this.manager.currentDocument.id, 'change'], { relativeTo: this.route });
        }

        if (option.name == 'PDF') {
            this.context.app.showSpinner();
            this.presupuestosService.generarPdfImpresion(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    window.open('/file-server/' + file.uri, '_blank');
                })
                .catch(error => this.handleError(error));
        }

    }



}

