﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { MenuItem } from 'framework3/controls';
import { SearchRequest } from 'framework3/search';
//Locales
import { OrdenItem } from '../models/orden';
import { RecepcionInfo, RecepcionItem } from '../models/recepcion';
import { RecepcionDetail } from '../models/recepcion-detail';
import { RecepcionService } from '../services/recepcion.service';
import { ModalDirective } from 'ngx-bootstrap';
import { EmpresaService } from '../services/empresa.service';
import { CentralService } from '../services/central.service';
import { CentroRecepcionItem } from '../models/centro-recepcion';
import { OrdenesListComponent } from '../dialogs/ordenes-list.component';

import * as _ from 'lodash';

import { PolizaPerfilListComponent, PolizaPerfilItem } from '../../accounting/index';
import { PolizaInfo } from '../../accounting/index';
import { PolizaService } from '../../accounting/index';

@Component({
    selector: 'compras-recepciones-manager',
    templateUrl: './recepciones.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RecepcionesManager)
        },
        {
            provide: DocumentService, useExisting: RecepcionService
        }
    ]
})
export class RecepcionesManager extends DocumentManager {
    recepciones: RecepcionItem[];
    idProveedor: number = 0;
    idCentroRecepcion: number = 0;
    centrosRecepcion: CentroRecepcionItem[] = [];
    currentPoliza: PolizaInfo = new PolizaInfo();
    idWorkflowStepOrdenRecibir: number;

    //
    @ViewChild('viewDialog') dlgPolizaViewer: ModalDirective;
    @ViewChild('ordenesList') ordenesList: OrdenesListComponent;
    @ViewChild('proveedorDialog') proveedorDialog: ModalDirective;
    @ViewChild(PolizaPerfilListComponent) dlgPolizaPerfil: PolizaPerfilListComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private recepcionService: RecepcionService, private centralService: CentralService, private polizaSvc: PolizaService,
        private empresaService: EmpresaService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
        this.loadCentrales();
        this.loadEmpresa();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.recepcionService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.recepciones = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.idProveedor = 0;
        this.proveedorDialog.config.backdrop = false;
        this.proveedorDialog.show();
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id,'0','0'], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.recepcionService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.recepcionService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.recepcionService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    loadCentrales() {
        //this.centralService.loadListByUser(this.context.app.defaultSite.identity)
        //    .then(items => this.centrales = items)
        //    .catch(error => this.handleError(error));
        this.centralService.loadSecurityContext(this.context.app.defaultCompany.identity)
            .then(items => {                
                var result = _(items).groupBy('idCentroRecepcion').transform(function (result, current) {
                    result.push({
                        id: current[0].idCentroRecepcion,
                        nombre: current[0].centroRecepcion,
                    });
                }, []).value();

                this.centrosRecepcion = result;



            })
            .catch(error => this.handleError(error));

    }
       
    onProveedorSelected() {
        if (this.idCentroRecepcion > 0) {
            this.proveedorDialog.hide();
            //
            //this.ordenesList.config.backdrop = false;
            this.ordenesList.show();
        }
        else 
            this.showAlert("Se debe de seleccionar el proveedor y el centro de recepción", "danger");
        
    }
    
    onOrdenSelected(orden: OrdenItem) {
        //let item = this.manager.documentList.find(item => item.id == ingreso.id);
        //this.ingresosDialog.hide();
        if (orden != null) {
            this.router.navigate(['0', this.idCentroRecepcion, orden.id], { relativeTo: this.route });
        }
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'PDF') {
            this.context.app.showSpinner();
            this.recepcionService.generarPdfRecepcion(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    window.open('/file-server/' + file.uri, '_blank');
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "CONTABILIZAR") {
            this.dlgPolizaPerfil.show();
        }
    }

    cerrarProveedor() {
        this.proveedorDialog.hide();
    }

    onSelectedPolizaPerfil(item: PolizaPerfilItem) {
        this.context.app.showSpinner();
        this.polizaSvc.loadPolizaByDocumento(this.manager.currentDocument.id, item.id, this.context.app.defaultSite.identity)
            .then(result => {
                this.currentPoliza = result;
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.config.backdrop = false;
                this.dlgPolizaViewer.show();
            }).catch(error => this.handleError(error));
    }

    onClickGuardar() {
        this.context.app.showSpinner();
        if (this.currentPoliza.idPolizaOrigen == 0) this.currentPoliza.idPolizaOrigen = 1 //Manual
        this.polizaSvc.update(this.currentPoliza)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.hide();
                this.manager.showAlert('La poliza ha sido registrada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }

    loadEmpresa() {
        this.empresaService.getDetailByID(this.context.app.defaultCompany.identity)
            .then(info => {
                this.idWorkflowStepOrdenRecibir = info.idWorkflowStepOrdenRecibir;
            })
            .catch(error => this.handleError(error));
    }


}

