﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef, OnInit } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { PuestoInfo, PuestoItem } from '../models/puesto';
import { PuestoService } from '../services/puesto.service';
import { RiesgoPuestoService } from '../../accounting';


@Component({
    selector: 'rh-puestos-manager',
    templateUrl: './puestos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PuestosManager)
        },
        {
            provide: CatalogService, useExisting: PuestoService
        }
    ]
})
export class PuestosManager extends HierarchyManager implements OnInit {
    puestos: PuestoInfo[] = [];
    puestoEdicion: PuestoInfo = new PuestoInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    riesgosPuestos = [];


    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder, private puestoService: PuestoService, protected router: Router, protected route: ActivatedRoute,
        private riesgoPuestoSvc: RiesgoPuestoService) {
        super(context, router, route, changeDetector);

        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cuenta': [null],
            'departamento': [null],
            'riesgoPuesto': [0]
        });
    }

    ngOnInit() {
        this.context.app.showSpinner();
        setTimeout(() => {
            this.riesgoPuestoSvc.getRiesgoPuestoCbo(this.context.app.defaultSite.identity)
                .then(info => {
                    this.riesgosPuestos = info;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error)); 
        });
    }

    ////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.puestoService.loadHierarchy(this.context.app.defaultCompany.identity, 0)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.puestos = nodes;
            })
            .catch(error => this.handleError(error));
    }


    //////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadHierarchy();
        //Hacer llamadas de carga inicial de datos aquí
    }


    onNodesChanged(puestos: PuestoInfo[]) {
        this.puestos = puestos;
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.puestoEdicion = new PuestoInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.puestoEdicion = new PuestoInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.puestoEdicion.idBranch = this.context.app.defaultSite.identity;
        this.puestoEdicion.idCompany = this.context.app.defaultCompany.identity;
        //
        this.currentParent = parentNode;
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {
        this.puestoEdicion = this.puestoService.copy(node.data);
        this.showEditor();
    }

    onSavingNode() {
        //Si es un nuevo nodo
        if (this.puestoEdicion.clientID == 0) {
            this.addNode(this.puestoEdicion, this.currentParent);
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.puestoEdicion);
        }
        this.puestoEdicion = new PuestoInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<PuestoInfo>) {
        this.context.app.showSpinner();
        this.puestoService.saveHierarchy(changes)
            .then(puestos => {
                this.puestos = puestos;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}