﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem, ReporteInfo, ParametroInfo, ReportService } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { FichasTitInfo, FichasTitItem } from '../models/fichas-tit';
import { FichasTitDetail } from '../models/fichas-tit-detail';
import { FichasTitService } from '../services/fichas-tit.service';
import { FichaComponent } from '../components/ficha.component';
import { MenuItem, ContentViewerComponent } from 'framework3/controls';
import { error } from 'util';
import { PolizaPerfilListComponent, PolizaPerfilItem, PolizaService, PolizaInfo, FichaItem } from '../../accounting';
import { ModalDirective } from 'ngx-bootstrap';

@Component({
    selector: 'rh-fichas-tit-manager',
    templateUrl: './fichas-tit.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FichasTitManager)
        },
        {
            provide: DocumentService, useExisting: FichasTitService
        }
    ]
})
export class FichasTitManager extends DocumentManager {
    fichasTit: FichasTitItem[];
    argsCurrentDocument: WorkflowEventArgs;   
    currentPoliza: PolizaInfo = new PolizaInfo();
    titleContentView: string = "";
    fileNameContentView: string = "";
    message: string = "";

    @ViewChild('dlgFicha') dlgFicha: FichaComponent;
    @ViewChild(PolizaPerfilListComponent) dlgPolizaPerfil: PolizaPerfilListComponent;
    @ViewChild('viewDialog') dlgPolizaViewer: ModalDirective;
    @ViewChild('viewType') dlgFichaCargo: ModalDirective;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private fichasTitService: FichasTitService, private polizaSvc: PolizaService, private reportSvc: ReportService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.fichasTitService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.fichasTit = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.fichasTitService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.argsCurrentDocument = args;
        args.step.activities.forEach(activity => {
            if (activity.tag == 'APPLY') {
                if (this.fichasTit.filter(X => X.id == this.manager.currentDocument.id)[0].tipo == 'C') {
                    this.message = 'Esta Ficha no se Descuenta Via Nomina Fecha de Pago Compromiso ' + this.fichasTit.filter(X => X.id == this.manager.currentDocument.id)[0].cobro;
                    this.dlgFichaCargo.config.backdrop = false;
                    this.dlgFichaCargo.show();
                } else {
                    this.dlgFicha.idFichatit = args.document.id;
                    this.dlgFicha.show();
                }
            }
            else if (activity.tag == 'AUTHORIZE') {
                this.context.app.showSpinner();
                this.fichasTitService.getCuentaEmpleado(args.document.id).then(
                    idCuenta => {
                        this.context.app.hideSpinner();
                        if (idCuenta > 0) {
                            this.fichasTitService.armarCuentas(args.document.id).then(
                                a => {
                                    this.dlgPolizaPerfil.show();
                                }
                            ).catch(error => {
                                this.context.app.hideSpinner();
                                this.handleError(error);
                            }); 
                        }
                        else
                            this.showAlert('El empleado no tiene cuenta asignada', 'danger');
                    }).catch(error => {
                        this.context.app.hideSpinner();
                        this.handleError(error);
                    });
            }
        });
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.fichasTitService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onDescuentoSelected(args: number) {
        this.context.app.showSpinner();
        this.fichasTitService.nextStep(this.argsCurrentDocument.document.id, this.argsCurrentDocument.step.idWorkflowStep, args)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedPolizaPerfil(item: PolizaPerfilItem) {
        this.context.app.showSpinner();
        this.polizaSvc.loadPolizaByDocumento(this.manager.currentDocument.id, item.id, this.context.app.defaultSite.identity)
            .then(result => {
                this.currentPoliza = result;
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.config.backdrop = false;
                this.dlgPolizaViewer.show();
            }).catch(error => this.handleError(error));
    }


    onMenuItemClick(option: MenuItem) {
        if (option.name == 'DEACTIVATE') {
            this.context.app.showSpinner();
            this.fichasTitService.deactivate(this.manager.currentDocument.id)
                .then(_ => {
                    this.context.app.hideSpinner();
                    this.showAlert('El documento se desactivo con éxito', 'success');
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == 'DOCUEMNTS') {
            this.context.app.showSpinner();
            this.fichasTitService.getDetail(this.manager.currentDocument.id)
                .then(doc => {
                    this.context.app.hideSpinner();
                    this.router.navigate(['rh/documento-empleado', this.process.id, doc.idEmpleado], { queryParams: {process:this.process.codigo}});
                })
                .catch(error => {
                    this.context.app.hideSpinner();
                    this.handleError(error);
                });
        }
        else if (option.name == 'PDF') {

            this.context.app.showSpinner();
            let item = <FichasTitItem> this.manager.currentDocument;
            let informe = new ReporteInfo();
            let parametroFicha = new ParametroInfo();
            let parametroSucursal = new ParametroInfo();
            parametroFicha.name = "IdFichaTit";
            parametroFicha.values.push(item.id);
            parametroSucursal.name = "IdSucursal"
            parametroSucursal.values.push(this.context.app.defaultSite.identity.toString());
            informe.parametros.push(parametroFicha);
            informe.parametros.push(parametroSucursal);
            informe.path = "/Onili/RH/FichaCargo";
            informe.format = "pdf";
            this.reportSvc.loadReporte(informe)
                .then(info => {
                    this.context.app.hideSpinner();
                    let item = <FichaItem>this.manager.currentDocument;
                    this.titleContentView = "Ficha " + item.id;
                    this.fileNameContentView = item.id + ".pdf";
                    this.reportViewer.show("data:application/pdf; base64, " + info.content);
                })
                .catch(error => {
                    this.context.app.hideSpinner();
                    this.handleError(error);
                });

        }
    }

    onClickGuardar() {
        this.dlgPolizaViewer.hide();
        this.context.app.showSpinner();
        if (this.currentPoliza.idPolizaOrigen == 0) this.currentPoliza.idPolizaOrigen = 1 //Manual

        this.fichasTitService.nextStep(this.argsCurrentDocument.document.id, this.argsCurrentDocument.step.idWorkflowStep, this.currentPoliza)
            .then(_ => {
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.hide();
                this.refreshList();
            })
            .catch(error => {
                this.handleError(error)
            });
    }

    onAceptFicha() {
        this.context.app.showSpinner();
        this.fichasTitService.nextStep(this.argsCurrentDocument.document.id, this.argsCurrentDocument.step.idWorkflowStep, null)
            .then(_ => {
                this.dlgFichaCargo.hide();
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => {
                this.dlgFichaCargo.hide();
                this.handleError(error)
            });
    }

}

