﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { NominaInfo, NominaItem } from '../models/nomina';
import { NominaDetail } from '../models/nomina-detail';
import { NominaService } from '../services/nomina.service';
import { NominaDetailEditor } from './parts/nominas-detail.editor';
import { PeriodicidadPagoInfo, PeriodoItem } from '../models/periodicidad-pago';

@Component({
    selector: 'rh-nominas-editor',
    templateUrl: './nominas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => NominasEditor)
        },
        {
            provide: DocumentService, useExisting: NominaService
        }
    ]
})
export class NominasEditor extends DocumentEditor {
    nomina = new NominaInfo();
    periodos: PeriodoItem[] = [];
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: NominaDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private nominaService: NominaService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idClaseNomina': [null, AztecaValidators.selected],
            'idTipoNomina': [null, AztecaValidators.selected],
            'idPeriodicidadPago': [null, AztecaValidators.selected],
            'idPeriodicidadPagoPeriodo': [0, Validators.required]
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.nominaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nomina = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.nominaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nomina = doc;
                this.detailsEditor.clientIndexStart = this.nomina.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.nominaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nomina = doc;
                this.detailsEditor.clientIndexStart = this.nomina.details.length;
                if (this.nomina.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.nominaService.update(this.nomina)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.nomina.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.nomina = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.nominaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.nominaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.nomina.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new NominaDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: NominaDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: NominaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: NominaDetail[]) {
        this.nomina.details = updatedDetails;
    }

    onSavingDetail(detail: NominaDetail) {
        this.detailsList.updateItem(detail);
    }

    ////////////////////////////////////////////////////////

    onPeriodicidadUpdatead(item: PeriodicidadPagoInfo) {
        if (item) {
            this.periodos = item.periodos;
        }
        else {
            this.periodos = [];
        }
    }

}

