﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import * as moment from 'moment';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';
//Locales
import { ReciboNominaInfo, ReciboNominaItem } from '../models/recibo-nomina';
import { ReciboNominaDetail } from '../models/recibo-nomina-detail';
import { ReciboNominaService } from '../services/recibo-nomina.service';

@Component({
    selector: 'rh-recibos-nominas-manager',
    templateUrl: './recibos-nominas.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RecibosNominasManager)
        },
        {
            provide: DocumentService, useExisting: ReciboNominaService
        }
    ]
})
export class RecibosNominasManager extends DocumentManager {
    recibosNominas: ReciboNominaItem[];

    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;
    titleContentView: string = "";
    fileNameContentView: string = "";

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private reciboNominaService: ReciboNominaService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.reciboNominaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.recibosNominas = response.rows;
                this.recibosNominas.forEach(item => {
                    item.fechaPago = moment(item.fechaPago).toDate();
                });
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.reciboNominaService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.reciboNominaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.reciboNominaService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'PRINT') {
            this.context.app.showSpinner();
            this.reciboNominaService.printReciboPdf(this.manager.currentDocument.id)
                .then(report => {
                    let item = <ReciboNominaItem>this.manager.currentDocument;
                    this.context.app.hideSpinner();
                    this.titleContentView = "Recibo " + item.id;
                    this.fileNameContentView = item.id + ".pdf";
                    this.reportViewer.show("data:application/pdf; base64, " + report);
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == 'DOWNLOAD') {
            this.context.app.showSpinner();
            this.reciboNominaService.loadReciboXml(this.manager.currentDocument.id)
                .then(response => {
                    let item = <ReciboNominaItem>this.manager.currentDocument;
                    this.titleContentView = "Recibo " + item.id;
                    this.fileNameContentView = item.id + ".xml";
                    this.reportViewer.show("data:text/xml, " + response);
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

}

