﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, Period } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//
import * as moment from 'moment';
//Locales
import { SolicitudVacacionalInfo, SolicitudVacacionalItem } from '../models/solicitud-vacacional';
import { SolicitudVacacionalDetail } from '../models/solicitud-vacacional-detail';
import { SolicitudVacacionalService } from '../services/solicitud-vacacional.service';
import { SolicitudVacacionalDetailEditor } from './parts/solicitudes-vacacionales-detail.editor';
import { PeriodoVacacionalService } from '../services/periodo-vacacional.service';
import { FlexGrid, CellRangeEventArgs, FormatItemEventArgs, CellType } from 'wijmo/wijmo.grid';
//
import { EmpleadoInfo } from '../models/empleado';

@Component({
    selector: 'rh-solicitud-vacacional-editor',
    templateUrl: './solicitudes-vacacionales.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SolicitudesVacacionalesEditor)
        },
        {
            provide: DocumentService, useExisting: SolicitudVacacionalService
        }
    ]
})
export class SolicitudesVacacionalesEditor extends DocumentEditor {
    solicitudVacacional = new SolicitudVacacionalInfo();

    loadingDocument: boolean = false;
    idBorrador: number = DocumentStatus.DRAFT;
    empleadoCurrent = new  EmpleadoInfo();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: SolicitudVacacionalDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private solicitudVacacionalService: SolicitudVacacionalService, private periodoVacacionalService: PeriodoVacacionalService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idEmpleado': [0, AztecaValidators.selected],
            'fechaIni': [new Date, (control) => this.validateFecha(control)],
            'fechaFin': [new Date()],
        });
    }

    validateFecha(control: FormControl) {
        let FechaHoy: Date;
        FechaHoy = moment(new Date).hour(0).minute(0).second(0).millisecond(0).toDate();

        if (control.value) {
            let Fecha = control.value;
            Fecha = moment(Fecha).hour(0).minute(0).second(0).millisecond(0).toDate();
            if (Fecha.getTime() >= FechaHoy.getTime())
                return null;
            else
                return 'La fecha no debe ser anterior a hoy';
        }
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();

        this.detailsList.gridDetail.formatItem.addHandler(function (grid: FlexGrid, args: FormatItemEventArgs) {
            if (args.panel.cellType == CellType.Cell && grid.columns[args.col].binding == 'dias' &&
                grid.rows[args.row].dataItem.dias > grid.rows[args.row].dataItem.periodoVacacional.diasDisponibles ) {
                args.cell.style.backgroundColor = '#dc3545';
            }
            //else if (args.panel.cellType == CellType.Cell && grid.columns[args.col].binding == 'workflowStep' &&
            //    grid.rows[args.row].dataItem.diasSinAutorizar > 2) {
            //    args.cell.style.backgroundColor = '#dca535';
            //}
            else
                args.cell.style.backgroundColor = '';
        });
    }

    onCreatingDocument() {
        let FechaHoy: Date;
        FechaHoy = moment(new Date).hour(0).minute(0).second(0).millisecond(0).toDate();

        this.solicitudVacacional.fechaIni = FechaHoy;
        this.solicitudVacacional.fechaFin = FechaHoy;
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.solicitudVacacionalService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.solicitudVacacional = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.solicitudVacacionalService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.solicitudVacacional = doc;
                this.detailsEditor.clientIndexStart = this.solicitudVacacional.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();
        this.loadingDocument = true; //bandera para indicar que los controles se estan actualizando porque se esta cargando el documento
        this.solicitudVacacionalService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.solicitudVacacional = doc;
                this.detailsEditor.clientIndexStart = this.solicitudVacacional.details.length;
                if (this.solicitudVacacional.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();

                setTimeout(_ => this.loadingDocument = false, 6000);
                
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {

        this.validateDetail()
            .then(value => {
                if (value == true) {

                    this.context.app.showSpinner();
                    this.solicitudVacacionalService.update(this.solicitudVacacional)
                        .then(updatedDoc => {
                            this.context.app.hideSpinner();
                            //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                            this.reset();
                            if (!args.closeEditor && this.solicitudVacacional.identity == '') {
                                this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                            }
                            else {
                                if (args.closeEditor)
                                    this.router.navigate(['../'], { relativeTo: this.route });
                                else
                                    this.showAlert('La información ha sido actualizada con éxito', 'success');
                            }
                            this.solicitudVacacional = updatedDoc;
                        })
                        .catch(error => this.handleError(error));

                }
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.solicitudVacacionalService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.solicitudVacacionalService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.solicitudVacacional.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        //this.detailsEditor.detail = new SolicitudVacacionalDetail();
        //this.detailsEditor.show();

        //Obtener el periodo vacacional correspondiente
        this.solicitudVacacionalService.calcularRango(this.solicitudVacacional.idEmpleado, this.solicitudVacacional.fechaIni, this.solicitudVacacional.fechaFin, this.empleadoCurrent.idBranch)
            .then(result => {                
                this.solicitudVacacional.details = result;

                if (this.solicitudVacacional.details.filter(detail => detail.dias > detail.periodoVacacional.diasDisponibles).length > 0)
                    this.showAlert('Se han excedido los dias permitidos para el periodo', 'warning');                            

            })
            .catch (error => this.handleError(error));

        //let fechaCount: Date = this.solicitudVacacional.fechaIni;
        //this.periodoVacacionalService.obtenerByFechas(this.solicitudVacacional.idEmpleado, this.solicitudVacacional.fechaIni, this.solicitudVacacional.fechaFin)
        //    .then(result => {
        //        let vacaciones: SolicitudVacacionalDetail[] = [];

        //        result.forEach(periodo => {                    
        //            //Generamos los detalles del documento de solicitud en base al periodo de fechas seleccionado
        //            let detail = new SolicitudVacacionalDetail();
        //            let excedido: boolean = false;
        //            //
        //            detail.idPeriodoVacacional = periodo.id;                    
        //            detail.fechaIni = fechaCount;
        //            //
        //            detail.periodoVacacional = periodo;

        //            //recorremos y validamos los dias solicitados
        //            while (fechaCount.getTime() <= this.solicitudVacacional.fechaFin.getTime() && fechaCount.getTime() <= periodo.fechaMax.getTime()  ) {
        //                detail.dias = detail.dias + 1;
        //                detail.fechaFin = fechaCount;
        //                //si no superamos los dias de disfrute
        //                if (periodo.diasDisfrute < periodo.diasAutorizados + detail.dias) {
        //                    if (excedido == false)
        //                        this.showAlert('Se han excedido los dias permitidos para el periodo', 'warning');                            
        //                    excedido = true;
        //                }
        //                //    
        //                fechaCount = moment(fechaCount).add(1, 'days').toDate();                        
        //            }
                                        
        //            vacaciones.push(detail);
        //        });

        //        this.solicitudVacacional.details = vacaciones;
        //    })
        //    .catch(error => this.handleError(error));
    }

    onEditingDetail(detail: SolicitudVacacionalDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: SolicitudVacacionalDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: SolicitudVacacionalDetail[]) {
        this.solicitudVacacional.details = updatedDetails;
    }

    onSavingDetail(detail: SolicitudVacacionalDetail) {
        this.detailsList.updateItem(detail);
    }

    ////////////// IMPLEMENTACION DE EVENTOS //////////////////
    onEmpleadoUpdated(empleado: EmpleadoInfo) {       
        this.empleadoCurrent = <EmpleadoInfo>empleado;
        this.reset();
    }

    onFechaChanged(value: Date) {        
        this.reset();
    }

    //metodos particulares
    async validateDetail() {
        return <boolean>await this.checkDetails();
    };
    
    checkDetails() {
        let result: boolean = true;

        if (this.solicitudVacacional.details.length == 0) {
            this.showAlert("No se han establecido periodos de vacaciones.", 'warning');
            result = false;
        }

        //validamos que todos los articulos tengan serie en caso de que se maneje
        if (this.solicitudVacacional.details.filter(detail => detail.dias > detail.periodoVacacional.diasDisponibles).length > 0) {
            this.showAlert("Existen periodos que exceden el número de dias disponibles.", 'warning');
            result = false;
        }

        return result;
    }

    reset() {
        if (this.solicitudVacacional.status == DocumentStatus.DRAFT && this.loadingDocument==false)
            this.solicitudVacacional.details = [];

    }
}

