﻿import { Component, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';

// moment
import * as moment from 'moment';

import { Context, BaseComponent, ProcessComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';
import { AlertsBarComponent, AztecaValidators, MenuItem } from 'framework3/controls';

// Wijmo
import * as wjcCore from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';

import { ChecadorService } from '../services/checador.service';
import { MotivoService } from '../services/motivo.service';

import { TransaccionItem } from '../models/transaccion';
import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';

@Component({
    selector: 'rh-checada-empleado',
    templateUrl: './checada-empleado.maintenance.html'
})
export class ChecadaEmpleadoMaintenance extends ProcessComponent {

    form: FormGroup;
    formChecada: FormGroup;

    idEmpleado: number = 0;
    fechaIni: Date = new Date();
    fechaFin: Date = new Date();
    valid: boolean = false;
    widthClass: string[] = [];

    cvTransacciones: wjcCore.CollectionView = new wjcCore.CollectionView();
    eliminadas: TransaccionItem[] = [];

    motivos: any[] = [];
    menuChecadaItems: MenuItem[] = [];


    // 
    idMotivo: number = 0;
    fecha: Date = new Date();
    hora: Date = new Date();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('dlgChecada') dlgChecada: ModalDirective;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        formBuilder: FormBuilder,
        private checadorSvc: ChecadorService,
        private motivoSvc: MotivoService
    ) {
        super(context, router, route);
        this.form = formBuilder.group({
            'idEmpleado': [null, AztecaValidators.selected],
            'fechaIni': [null, Validators.required],
            'fechaFin': [null, Validators.required]
        });

        this.formChecada = formBuilder.group({
            'idMotivo': ['', AztecaValidators.selected],
            'fecha': [null, Validators.required],
            'hora': [null, Validators.required]
        });

        this.cvTransacciones.sourceCollection = [];
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
        setTimeout(() => {
            this.motivoSvc.loadCbo(this.context.app.defaultSite.identity)
                .then(info => {
                    this.motivos = info;
                    this.motivos.unshift({ id: 0, codigo: '', nombre: '[Seleccione un motivo]', Descripcion: '' });
                })
                .catch(error => this.handleError(error));
        });
    }

    ngAfterViewInit() {
        if (!this.form) return;
        setTimeout(_ => {
            this.fechaIni = moment().subtract(16, 'day').toDate();
            this.fechaFin = moment().subtract(1, 'day').toDate();
            this.createMenuDetail();
        });

        this.form.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });
    }

    private changeState() {
        this.valid = this.form.valid;
    }

    createMenuDetail() {
        let mnuQuitar: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Eliminar", name: "QUITAR", smallText: "eliminar checada" };

        this.menuChecadaItems.push(mnuQuitar);
        this.menuChecadaItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    ////////////////////////////////////////////////////////

    onClickGuardar() {
        this.context.app.showSpinner();
        let items = this.cvTransacciones.items;
        let transacciones = items.concat(this.eliminadas);
        this.checadorSvc.updateTransaccionesEmpleado(transacciones)
            .then(result => {
                this.context.app.hideSpinner();
                this.onClickActualizar();
            })
            .catch(error => this.handleError(error));
    }

    onClickGuardarProcesar() {
        this.context.app.showSpinner();
        let items = this.cvTransacciones.items;
        let transacciones = items.concat(this.eliminadas);
        this.checadorSvc.procesarTransaccionesEmpleado(transacciones)
            .then(result => {
                this.context.app.hideSpinner();
                this.onClickActualizar();
                this.alerts.showAlert('Se agrego la tarea para procesar las checadas', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onClickCerrar() {
        this.location.back();
    }

    onClickActualizar() {
        this.context.app.showSpinner();
        this.checadorSvc.getTransaccionesEmpleado(this.fechaIni, this.fechaFin, this.idEmpleado)
            .then(result => {
                this.eliminadas = [];

                this.cvTransacciones.sourceCollection = result.filter(check=> check.manual==true);
                this.cvTransacciones.refresh();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onClickAgregar() {
        this.idMotivo = 0;
        this.fecha = new Date();
        this.hora = new Date();
        this.dlgChecada.config.backdrop = false;
        this.dlgChecada.show();
    }

    onClickMnuChecada(menu: Menu) {
        let option = this.menuChecadaItems[menu.selectedIndex];
        if (option.name == "QUITAR") {
            let item = <TransaccionItem>this.cvTransacciones.currentItem;
            if (item) {
                item.state = ChangeState.DELETED
                this.cvTransacciones.remove(item);
                this.eliminadas.push(item);
            }
        }
    }

    onClickChecadaAceptar() {
        this.context.app.showSpinner();
        let transaccion = <TransaccionItem> this.cvTransacciones.addNew();
        transaccion.idEmpleado = this.idEmpleado;
        transaccion.fecha = this.hora;
        transaccion.idMotivo = this.idMotivo;
        transaccion.manual = true;
        transaccion.usuario = this.context.session.user.code;
        transaccion.state = ChangeState.ADDED;
        this.cvTransacciones.commitNew();
        this.dlgChecada.hide();
        this.context.app.hideSpinner();
    }

    ////////////////////////////////////////////////////////

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

    private handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}