﻿import { Component, ViewChild } from '@angular/core';
import { FormsModule, FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
// Framework
import { Context, BaseComponent, ProcessComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';
import { Alert, AlertsBarComponent, AztecaValidators, MenuItem } from 'framework3/controls';
import { TrackableCollection } from 'framework3/editors';
// Bootstrap
import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';
// moment
import * as moment from 'moment';
// Wijmo
import * as wjcCore from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
// Features
import { KardexAsistenciaService } from "../services/kardex-asistencia.service";
import { IncidenciaService } from "../services/incidencia.service";
import { IncidenciaEmpleadoInfo } from '../models/incidencia-empleado';
import { EmpleadoInfo } from '../models/empleado';
import { IncidenciaInfo } from '../models/incidencia';
import { KardexAsistenciaInfo } from '../models/kardex-asistencia';
import { IncidenciaListComponent } from '../controls/incidencia-list.component';


@Component({
    selector: 'rh-kardex-empleado',
    templateUrl: './kardex-empleado.maintenance.html'
})
export class KardexEmpleadoMaintenance extends ProcessComponent {
    
    form: FormGroup;
    formFolio: FormGroup;

    idEmpleado: number = 0;
    fechaIni: Date = new Date();
    fechaFin: Date = new Date();
    valid: boolean = false;

    folio: string = "";
    observacion: string = "";

    widthClass: string[] = [];
    menuIncidenciaItems: MenuItem[] = [];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('lstDlgIncidencia') lstDlgIncidencia: IncidenciaListComponent;
    @ViewChild('dlgFolio') dlgFolio: ModalDirective;

    cvKardexAsistencia: wjcCore.CollectionView = new wjcCore.CollectionView();

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private kardexAsistenciaSvc: KardexAsistenciaService,
        private incidenciaSvc: IncidenciaService
    ) {
        super(context, router, route);
        this.form = formBuilder.group({
            'idEmpleado': [null, AztecaValidators.selected],
            'fechaIni': [null, Validators.required],
            'fechaFin': [null, Validators.required]
        });

        this.formFolio = formBuilder.group({
            'folio': ['', Validators.required],
            'observacion': ['', Validators.required]
        });
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
    }

    ngAfterViewInit() {
        if (!this.form) return;
        setTimeout(_ => {
            this.fechaIni = moment().subtract(16, 'day').toDate();
            this.fechaFin = moment().subtract(1, 'day').toDate();
            this.createMenuDetail();
        });

        this.form.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });
    }


    private changeState() {
        this.valid = this.form.valid;
    }

    closeEditor() {
        this.location.back();
    }

    createMenuDetail() {
        let mnuAsignar: MenuItem = { icon: "glyphicon glyphicon-pencil", text: "Asignar Incidencia", name: "ASIGNAR", smallText: "asignar incidencia" };
        let mnuQuitar: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Quitar Incidencia", name: "QUITAR", smallText: "quitar incidencia" };
        let mnuFolio: MenuItem = { icon: "glyphicon glyphicon-edit", text: "Asignar Folio", name: "FOLIO", smallText: "asignar folio" };        

        this.menuIncidenciaItems.push(mnuAsignar, mnuQuitar, mnuFolio);
        this.menuIncidenciaItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    ////////////////////////////////////////////////////////
    // Evnents handler

    onClickKadexActualizar() {
        if (this.valid) {
            this.context.app.showSpinner();
            this.kardexAsistenciaSvc.getKardexAsistencia(this.idEmpleado, this.fechaIni, this.fechaFin)
                .then(info => {
                    this.cvKardexAsistencia.sourceCollection = info;
                    this.cvKardexAsistencia.refresh();
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.cvKardexAsistencia.sourceCollection = [];
            this.cvKardexAsistencia.refresh();
        }   
    }

    onClickMnuIncidencia(menu: Menu) {
        let option = this.menuIncidenciaItems[menu.selectedIndex];
        if (option.name == "ASIGNAR") {
            this.lstDlgIncidencia.show();
        }
        else if (option.name == "QUITAR") {
            let item: KardexAsistenciaInfo = <KardexAsistenciaInfo>this.cvKardexAsistencia.currentItem;
            this.cvKardexAsistencia.editItem(item);
            item.idIncidencia = 0;
            item.incidencia = new IncidenciaInfo();
            item.folio = "";
            item.observaciones = "";
            item.manual = true;
            item.estatus = "";
            item.state = ChangeState.DELETED;
            this.cvKardexAsistencia.commitEdit();
        }
        else if (option.name == "FOLIO") {
            let item: KardexAsistenciaInfo = <KardexAsistenciaInfo>this.cvKardexAsistencia.currentItem;
            if (item) {
                this.folio = item.folio;
                this.observacion = item.observaciones;
                this.dlgFolio.config.backdrop = false;
                this.dlgFolio.show();
            }
        }
    }

    onClickFolioAceptar() {
        let item: KardexAsistenciaInfo = <KardexAsistenciaInfo>this.cvKardexAsistencia.currentItem;
        if (item) {
            this.cvKardexAsistencia.editItem(item);
            item.folio = this.folio;
            item.observaciones = this.observacion;
            item.state = ChangeState.MODIFIED;
            this.cvKardexAsistencia.commitEdit();
        }
        this.dlgFolio.hide();
    }

    onSelectedIncidencia(catalogItem: CatalogItem) {
        this.context.app.showSpinner();
        this.incidenciaSvc.getDetailByID(catalogItem.id)
            .then(info => {
                let item: KardexAsistenciaInfo = <KardexAsistenciaInfo>this.cvKardexAsistencia.currentItem;
                if (item) {
                    this.cvKardexAsistencia.editItem(item);
                    item.idIncidencia = info.identity;
                    item.incidencia = info;
                    item.estatus = info.estatus;
                    item.state = ChangeState.MODIFIED;
                    item.manual = true;
                    this.cvKardexAsistencia.commitEdit();
                }
                this.context.app.hideSpinner();
            }).catch(error => this.handleError(error));
    }

    saveHandler() {
        this.context.app.showSpinner();
        this.kardexAsistenciaSvc.updateKardexAsistencia(this.cvKardexAsistencia.sourceCollection)
            .then(result => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Kardex del empleado actualizado', 'success');
                this.onClickKadexActualizar();
            })
            .catch(error => this.handleError(error));
    }


    ////////////////////////////////////////////////////////

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }


    private handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}