﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, SucursalService as SucursalKernelService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';

//Features
import { EmpresaService} from '../services/empresa.service';
import { EmpresaInfo, EmpresaItem} from '../models/empresa';
import { SucursalItem } from '../models/sucursal';
import { SucursalService } from '../services/sucursal.service';

import { Control } from 'wijmo/wijmo';
import { FormatoItem, FormatoTipoService } from 'framework3/general';

@Component({
    selector: 'ventas-empresas-editor',
    templateUrl: './empresas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EmpresasEditor)
        },
        {
            provide: CatalogService, useExisting: EmpresaService
        }
    ]
})
export class EmpresasEditor extends CatalogEditor {
    empresa: EmpresaInfo = new EmpresaInfo();
    formatosVenta: FormatoItem[] = [];
    @Input() sucursales: SucursalItem[] = [];       

    solicitarMetodosPagoCredito: { id: boolean, nombre: string }[] = [];
    modalidadCobroContado: { id: string, nombre: string }[] = [];
    modalidadesPrecio: { id: string, nombre: string }[] = [];
    politicasDescuento: { id: string, nombre: string }[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private empresaService: EmpresaService, private sucursalService: SucursalService,
        private sucursalKernelService: SucursalKernelService, private formatoService: FormatoTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', ],
            'nombre': ['', ],
            'solicitarMetodosPago': ['', Validators.required],
            'modalidadCobroContado': ['', Validators.required],
            'idModalidadPrecios': ['', Validators.required],
            'idPoliticaDescuentos': ['', Validators.required],
            'impuestosIncluidos': [false],
            'idRemitentePortal':[0],
            'idFormatoVenta':[0]
        });

        this.solicitarMetodosPagoCredito =
            [
                { "id": true, "nombre": "Si" },
                { "id": false, "nombre": "No" }
            ];

        this.modalidadCobroContado =
            [
                { 'id': 'A', 'nombre': 'Al aplicar la venta' },
                { 'id': 'C', 'nombre': 'En el documento de cobranza' },
                { 'id': 'D', 'nombre': 'Decisión del usuario' }
            ];

        this.modalidadesPrecio =
            [
                { "id": "L", "nombre": "Utilizar listas de precio" },
                { "id": "Z", "nombre": "Utilizar precios de la zona de venta" }
            ];
        this.politicasDescuento =
            [
                { "id": "A", "nombre": "Descuentos ANTES de impuestos" },
                { "id": "D", "nombre": "Descuentos DESPUES de impuestos" }
            ];
    }

	///////////////////////// METODOS ///////////////////////////////////////

    loadFormatosVentas() {
        this.formatoService.loadFormatos('SALES', this.empresa.identity)
            .then(formatos => {
                this.formatosVenta = formatos;
                this.formatosVenta.unshift({ id: 0, nombre: '[NINGUNO]', activo: true, codigo: '' });
            })
            .catch(error => this.handleError(error));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;                
                this.setReadonlyMode();
                this.loadSucursales();
                this.loadFormatosVentas();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.empresaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresa = info;
                this.empresa.identity = info.empresaBase.identity;
                this.loadSucursales();
                this.loadFormatosVentas();
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.empresaService.update(this.empresa)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.empresa.identity == 0) {
                    this.router.navigate(['ventas/empresa', this.process.id, info.identity]);
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.empresa = info;
            })
            .catch(error => this.handleError(error));        
    }

    //Manejo de Estados
    selectSucursalesTab(arg: any) {
        Control.invalidateAll();
    }

    loadSucursales() {
        let request: SearchRequest = new SearchRequest();
        request.paged = false;
        //Agregar condición de empresa actual
        let condicionEmpresa: SearchCondition = { field: "idEmpresa", operator: "=", valueType: "int", value: this.empresa.identity.toString() };
        request.conditions.push(condicionEmpresa);
        //
        this.context.app.showSpinner();
        this.sucursalKernelService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.sucursales = response.rows;
            })
            .catch(error => this.handleError(error));
    }

}