﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, EditableListComponent, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Contabilidad
import { ImpuestoAsignacionInfo, ImpuestosAsignacionEditor } from '../../accounting';
//Ventas
import { ServicioService } from '../services/servicio.service';
import { ServicioInfo, ServicioItem } from '../models/servicio';
import { ImpuestoAsignacionZonaInfo } from '../models/impuesto-asignacion-zona';

import { UnidadMedidaService } from '../../inventory/services/unidad-medida.service';
import { UnidadMedidaInfo, UnidadMedidaItem } from '../../inventory/models/unidad-medida';
import { UnidadMedidaTipoItem, UnidadMedidaTipoInfo } from '../../inventory/models/unidad-medida-tipo';
import { UnidadMedidaTipoService } from '../../inventory/services/unidad-medida-tipo.service';
import { ZonaVentaItem } from '../models/zona-venta'
import { ZonaVentaService } from '../services/zona-venta.service'


@Component({
    selector: 'ventas-servicio-editor',
    templateUrl: './servicios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ServiciosEditor)
        },
        {
            provide: CatalogService, useExisting: ServicioService
        }
    ]
})
export class ServiciosEditor extends CatalogEditor {
    servicio: ServicioInfo = new ServicioInfo();
    idZonaVenta: number;
    impuestosCollection = new TrackableCollection();
    tiposUnidad: UnidadMedidaTipoItem[] = [];
    unidadesMedida: UnidadMedidaItem[] = [];
    unidadMedidaTipo: UnidadMedidaTipoInfo = new UnidadMedidaTipoInfo();
    zonasVenta: ZonaVentaItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private servicioService: ServicioService,
        private unidadMedidaService: UnidadMedidaService, private tipoUnidadService: UnidadMedidaTipoService, private zonaVentaService: ZonaVentaService) {


        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'categoriaFiscal': [0, AztecaValidators.selected],
            'idEstCom': [0],
            'idFlujo': [0],
            'idCuenta': [0],
            'impuestoIncluido': [false],
            'libre': [false],
            'idUnidadMedida': [0, AztecaValidators.selected],
            //
            'idZonaVenta': [0]
        });
    }

    @ViewChild('impuestosList') impuestosList: EditableListComponent;
    @ViewChild(ImpuestosAsignacionEditor) impuestoEditor: ImpuestosAsignacionEditor;

    ///////////////////////// METODOS ///////////////////////////////////////

    loadTiposUnidad() {

        this.tipoUnidadService.getDetailByCode(this.context.app.defaultSite.idBranch, "SVC")
            .then(tipos => {
                this.loadUnidadesMedida(tipos.identity);
            }

        )
            .catch(error => this.handleError(error));
        
    }

    loadZonasVenta() {
        this.zonaVentaService.loadList(this.context.app.defaultCompany.identity.toString())
            .then(zonasVenta => this.zonasVenta = zonasVenta)
            .catch(error => this.handleError(error));
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        this.unidadMedidaService.loadList(idTipoUnidad)
            .then(unidades => {
                this.unidadesMedida = unidades;
                this.unidadesMedida.unshift({ id: 0, codigo: '', nombre: '[Seleccionar]', activo: true, estatus: 'activo', factor: 0, base: false, fraccionable: false, paquete: false });
            })
            .catch(error => this.handleError(error));
    }


    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadTiposUnidad();
        this.loadZonasVenta();
    }

    onCreatingItem() {
        this.impuestosCollection.sourceCollection = this.servicio.impuestos;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.servicioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.servicio = info;
                this.setReadonlyMode();
                //this.impuestosCollection.sourceCollection = this.servicio.impuestos;
                this.impuestosCollection.sourceCollection = this.servicio.impuestos.filter(item => item.idZonaVenta == this.idZonaVenta);
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.servicioService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.servicio = info;
                this.impuestosCollection.sourceCollection = this.servicio.impuestos;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.servicioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.servicio = info;
                this.impuestosCollection.sourceCollection = this.servicio.impuestos.filter(item => item.idZonaVenta == this.idZonaVenta);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.servicioService.update(this.servicio)
            .then(info => {
                this.context.app.hideSpinner();
                this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.servicio.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.servicio = info;
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected() {
        Control.invalidateAll();
    }

    ///////////// IMPUESTOS /////////////////////////

    onImpuestosToolbar(button: ToolbarButton) {
        if (this.idZonaVenta > 0 && this.idZonaVenta != undefined)
            this.impuestoEditor.show();
        else
            this.showAlert("Debe seleccionar una zona de venta para incluir impuestos al servicio", "warning");
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'LP') {
            this.router.navigate(['sales/articulo-listas-precio/' + this.process.id + '/' + this.servicio.identity.toString()]);
        } else if (item.name == 'GD') {
            this.router.navigate(['sales/articulo-grupo-descuento/' + this.process.id + '/' + this.servicio.identity.toString()]);
        }
    }

    onRemoveItem(impuesto: ImpuestoAsignacionZonaInfo) {
        this.impuestosCollection.remove(impuesto);
    }

    onImpuestoUpdated(impuesto: ImpuestoAsignacionZonaInfo) {
        let newImpuestos = this.servicio.impuestos.slice();
        impuesto.idZonaVenta = this.idZonaVenta; //se asigna la zona de venta seleccionada
        newImpuestos.push(impuesto);
        this.servicio.impuestos = newImpuestos;
        this.impuestosCollection.sourceCollection = newImpuestos.filter(item => item.idZonaVenta == this.idZonaVenta);
    }

    onZonaVentaChanged(idZonaVenta: number) {
        this.impuestosCollection.sourceCollection = this.servicio.impuestos.filter(item => item.idZonaVenta == idZonaVenta);       
    }

}