﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { FormatoItem, FormatoTipoItem, FormatoTipoService } from 'framework3/general';
//Features
import { SucursalService} from '../services/sucursal.service';
import { SucursalInfo, SucursalItem } from '../models/sucursal';
import { UnidadNegocioItem } from '../models/unidad-negocio';
import { UnidadNegocioService } from '../services/unidad-negocio.service';

import { DivisaItem } from '../../accounting/models/divisa';
import { DivisaService } from '../../accounting/services/divisa.service';
import { ListaPreciosService } from '../services/lista-precios.service';
import { ListaPreciosItem } from '../models/lista-precios';
import { GrupoDescuentoService } from '../services/grupo-descuento.service';
import { GrupoDescuentoItem } from '../models/grupo-descuento';
import { Control } from 'wijmo/wijmo';
import { RegionItem } from '../models/region'
import { RegionService } from '../services/region.service'
import { ZonaVentaItem } from '../models/zona-venta'
import { ZonaVentaService } from '../services/zona-venta.service'


@Component({
    selector: 'ventas-sucursales-editor',
    templateUrl: './sucursales.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SucursalesEditor)
        },
        {
            provide: CatalogService, useExisting: SucursalService
        }
    ]
})
export class SucursalesEditor extends CatalogEditor {
    sucursal: SucursalInfo = new SucursalInfo();    
    @Input() unidadesNegocio: UnidadNegocioItem[] = [];

    divisas: DivisaItem[] = [];
    listasPrecio: FormatoItem[] = [];
    gruposDescuento: FormatoItem[] = [];
    regiones: RegionItem[] = [];
    zonasVenta: ZonaVentaItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private sucursalService: SucursalService,
        private divisaService: DivisaService, private formatoTipoService: FormatoTipoService, private listaPreciosService: ListaPreciosService, private grupoDescuentoService: GrupoDescuentoService,
        private unidadNegocioService: UnidadNegocioService, private regionService: RegionService, private zonaVentaService: ZonaVentaService,) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': [''],
            'idDivisaBase': ['', AztecaValidators.selected],
            'idListaPrecio': ['', AztecaValidators.selected],
            'idGrupoDescuento': ['', AztecaValidators.selected],                      
            'idRegion': ['', Validators.required],
            'idZonaVenta': ['', Validators.required],
            'idCuenta': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();  
        this.loadRegiones();
        this.loadZonasVenta();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.sucursalService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.sucursal = info;                
                this.setReadonlyMode();
                this.loadUnidadesNegocio();
                this.loadDivisas();
                this.loadListasPrecio();
                this.loadGruposDescuento();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.sucursalService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.sucursal = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.sucursalService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.sucursal = info;
                this.sucursal.identity = info.sucursalBase.identity;

                this.loadUnidadesNegocio();
                this.loadDivisas();
                this.loadListasPrecio();
                this.loadGruposDescuento();

            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.sucursalService.update(this.sucursal)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.sucursal.identity == 0) {
                    this.router.navigate(['../', this.process.id, info.identity]);
                }
                else {					
					if (args.closeEditor)
                        this.location.back();
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.sucursal = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadDivisas() {
        this.divisaService.loadList(this.sucursal.idCompany)
            .then(divisas => this.divisas = divisas)
            .catch(error => this.handleError(error));
    }
       

    //Manejo de Estados
    selectUnidadesNegocioTab(arg: any) {
        Control.invalidateAll();
    }

    loadUnidadesNegocio() {
        let request: SearchRequest = new SearchRequest();
        request.paged = false;
        //Agregar condición de empresa actual
        let condicionServidorSMTP: SearchCondition = { field: "idSucursal", operator: "=", valueType: "int", value: this.sucursal.identity.toString() };
        request.conditions.push(condicionServidorSMTP);
        //
        this.context.app.showSpinner();
        this.unidadNegocioService.search(this.sucursal.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.unidadesNegocio = response.rows;
            })
            .catch(error => this.handleError(error));
    }

    loadRegiones() {
        this.regionService.loadList(this.context.app.defaultCompany.identity)
            .then(regiones => this.regiones = regiones)
            .catch(error => this.handleError(error));
    }
    loadZonasVenta() {
        this.zonaVentaService.loadList(this.context.app.defaultCompany.identity.toString())
            .then(zonasVenta => this.zonasVenta = zonasVenta)
            .catch(error => this.handleError(error));
    }

    loadListasPrecio() {
        this.listaPreciosService.loadList(this.sucursal.identity)
            .then(listas => this.listasPrecio = listas)
            .catch(error => this.handleError(error));
    }
    loadGruposDescuento() {
        this.grupoDescuentoService.loadList(this.sucursal.identity)
            .then(grupos => this.gruposDescuento = grupos)
            .catch(error => this.handleError(error));
    }


    onCancelingEdit() {
        this.location.back();
    }


}