﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection, CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { ArticuloStockItem } from '../../inventory/models/articulo-stock';
import { ArticuloService } from '../services/articulo.service';
import { ArticuloStockSelectorComponent } from '../../inventory/controls/articulo-stock-selector.component';

@Component({
    selector: 'sales-nuevo-articulo',
    templateUrl: './nuevo-articulo.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticuloNuevoComponent)
        },
        {
            provide: CatalogService, useExisting: ArticuloService
        }
    ]
})
export class ArticuloNuevoComponent extends CatalogEditor {
    editorForm: FormGroup;
    articulosStock: ArticuloStockItem = new ArticuloStockItem();
    opcionesAutogenerar: { id: number, name: string }[] = [
        { "id": 1, "name": "Crear articulo nuevo en inventario y ventas" },
        { "id": 2, "name": "Ligar a articulo existente en inventario" }
    ];
    codigoValidador: string = '';
    //Inputs
    @Input() data: ArticuloStockItem = new ArticuloStockItem();
    @Input() autogenerado: boolean = false;
    //outputs
    @Output() dataUpdated: EventEmitter<ArticuloStockItem> = new EventEmitter<ArticuloStockItem>();
    //
    @ViewChild('dialog') articuloNuevoDialog: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('selectorArticulo') selectorArticulo: ArticuloStockSelectorComponent;

    constructor(private articulosService: ArticuloService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetector: ChangeDetectorRef) {
        super(context, router, route, changeDetector);
        this.editorForm = formBuilder.group({
            'codigo': [''],
            'nombre': [''],
            'unidadMedida': [''],
            'codigoValidador': [''],
            'opcionesAutogenerar': [0],
            'articulo': [''],
            'codigoLigado': [true]
        });

    }

    show() {
        if (this.autogenerado) {
            this.editorForm.controls['opcionesAutogenerar'].setValidators(AztecaValidators.selected);
        }
        else {
            this.editorForm.controls['codigo'].setValidators(Validators.required);
        }

        this.articuloNuevoDialog.config.backdrop = false;
        this.articuloNuevoDialog.show();
        setTimeout(_ => this.editorForm.reset());
        Control.invalidateAll();
    }

    onTextChange(text: string) {
        this.articulosService.GetArticuloStock(this.context.app.defaultSite.identity, text)
            .then(articulo => {
                this.data = articulo;
                this.articulosService.articuloAsignadoCheck(articulo.id).then(x => {
                    if (articulo.nombre.length == 0) {
                        this.codigoValidador = 'Código Nuevo';
                        // Nota: si es código es nuevo, habilita el boton de aceptar
                        this.editorForm.controls['codigoLigado'].clearValidators();
                        this.editorForm.controls['codigoLigado'].updateValueAndValidity();
                    } else if (x) {
                        //Nota: si el código ya fue ligado a un articulo, inhabilita el boton de aceptar
                        this.codigoValidador = 'El Código ya esta ligado a un articulo';
                        this.editorForm.controls['codigoLigado'].setValidators(Validators.requiredTrue);
                        this.editorForm.controls['codigoLigado'].updateValueAndValidity();
                    } else {
                        //Nota: si el codigo ya existe pero no esta ligado a un articulo, habilita el boton de aceptar
                        this.codigoValidador = 'El Código se asignara al articulo';
                        this.editorForm.controls['codigoLigado'].clearValidators();
                        this.editorForm.controls['codigoLigado'].updateValueAndValidity();
                    }
                }).catch(Er => this.handleError(Er))
            })
            .catch(error => this.handleError(error))
    }

    onUpdate() {
        if (this.autogenerado) {
            if (this.editorForm.controls.opcionesAutogenerar.value === 1) {
                this.router.navigate(['sales/articulos-ventas/' + this.process.id + '/0']);
            }
            else {
                this.router.navigate(['sales/articulos-ventas/' + this.process.id + '/0'], { queryParams: { idArticuloStock: this.editorForm.controls.articulo.value } });
            }
        }
        else {
            if (this.data.id > 0) {
                this.router.navigate(['sales/articulos-ventas/' + this.process.id + '/0'], { queryParams: { idArticuloStock: this.data.id } });
            }
            else {
                this.router.navigate(['sales/articulos-ventas/' + this.process.id + '/0'], { queryParams: { codigoNuevo: this.editorForm.controls.codigo.value } });
            }
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    onSelectedItemChanged(args: any) {
        if (args == 2) {
            this.editorForm.controls['articulo'].setValidators(AztecaValidators.selected);
        }
        else {
            this.editorForm.controls['articulo'].setValidators(Validators.nullValidator);
            this.codigoValidador = '';
            this.editorForm.controls['codigoLigado'].clearValidators();
            this.editorForm.controls['codigoLigado'].updateValueAndValidity();
        }
        //Nota: updateValueAndValidity() no actualiza el validador de todos los controles. Por eso se usa disable() y enable().
        this.editorForm.disable();
        this.editorForm.enable();
    }

    itemchanged(args: any) {
        this.articulosService.articuloAsignadoCheck(args.identity).then(x => {
            if (x) {
                //Nota: si el código ya fue ligado a un articulo, inhabilita el boton de aceptar
                this.codigoValidador = 'El Código ya esta ligado a un articulo';
                this.editorForm.controls['codigoLigado'].setValidators(Validators.requiredTrue);
                this.editorForm.controls['codigoLigado'].updateValueAndValidity();
            } else {
                //Nota: si el codigo ya existe pero no esta ligado a un articulo, habilita el boton de aceptar
                this.codigoValidador = 'El Código se asignara al articulo';
                this.editorForm.controls['codigoLigado'].clearValidators();
                this.editorForm.controls['codigoLigado'].updateValueAndValidity();
            }
        })
    }

}