﻿import { Component, Input, OnInit, EventEmitter, SimpleChange } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';
import { Control, CollectionView } from 'wijmo/wijmo';
import { FlexGrid, CellRangeEventArgs, FormatItemEventArgs, CellType } from 'wijmo/wijmo.grid';
//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//Modulos pixkua
import { ImpuestoDetailInfo, TipoAplicacion, TipoFactor } from '../../../accounting';
import { UnidadMedidaItem, UnidadMedidaTipoInfo, UnidadMedidaService, UnidadMedidaTipoService } from '../../../inventory';
//Ventas
import { VentaDetail } from '../../models/venta-detail';
import { VentaServicioDetail } from '../../models/venta-servicio-detail';
import { ServicioInfo } from '../../models/servicio';
import { ServicioService } from '../../services/servicio.service';
import { SucursalService } from '../../services/sucursal.service'

@Component({
    selector: 'venta-servicios-detail-editor',
    templateUrl: './venta-servicios-detail.editor.html'
})
export class VentaServiciosDetailEditor extends BasicEditor {
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new VentaServicioDetail();
    //
    trasladosCollection = new CollectionView();
    retencionesCollection = new CollectionView();
    unidadesMedida: UnidadMedidaItem[] = [];
    idZonaVenta: number = 0;

    //Objeto original que se editará
    @Input() detail: VentaServicioDetail = new VentaServicioDetail();
    @Input() idCliente: number = 0;


    constructor(protected context: Context, private builder: FormBuilder, private servicioService: ServicioService,
        private unidadService: UnidadMedidaService, private unidadTipoService: UnidadMedidaTipoService, private sucursalService: SucursalService) {
        super(context);

        this.form = builder.group({
            'idServicio': [0, AztecaValidators.selected],
            'cantidad':[0],
            'descripcion': [''],
            'precio': [0]
        });

        this.trasladosCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.TRASLADO;
        };

        this.retencionesCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.RETENCION;
        };

        this.form.controls.precio.valueChanges.subscribe(value => {
            this.onPrecioChanged(value);
        });
    }

    /////////////////// METODOS ///////////////////////////////

    loadUnidadMedida() {
        this.unidadTipoService.getDetailByCode(0, 'SVC')
            .then(tipo => {
                this.unidadService.loadList(tipo.identity)
                    .then(unidades => {
                        this.unidadesMedida = unidades;
                    })
                    .catch(error => this.handleError(error));
            })
            .catch(error => this.handleError(error));
    }

    /////////////////// HANDLERS //////////////////////////////
    //ngAfterViewInit() {
    ngOnInit() {
        setTimeout(_ => this.loadSucursal());
    }

    loadSucursal() {
        this.sucursalService.getDetailByID(this.context.app.defaultSite.identity)
            .then(suc => {
                this.idZonaVenta = suc.idZonaVenta;
            })
            .catch(error => this.handleError(error));
    }

    onPrecioChanged(precio: number) {
        if (precio > 0 && this.detailCopy && this.detailCopy.servicio.libre) {
            this.detailCopy.impuestos.forEach(impuesto => {
                impuesto.base = precio;
            });
            this.trasladosCollection.refresh();
            this.retencionesCollection.refresh();
        }
    }

    onCreatingItem() {
        //this.loadUnidadMedida();
        this.detailCopy = this.detail;
        this.form.reset({ idServicio: 0 });
        this.form.markAsPristine();
        Control.invalidateAll();
    }

    onLoadingItem(detail: VentaDetail) {
        //this.loadUnidadMedida();
        this.detailCopy = <VentaServicioDetail>this.copyItem(this.detail);
        Control.invalidateAll();
    }

    onUpdatingItem(detail: VentaDetail) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        //Si NO es un servicio libre, asignar descripcion del servicio
        if (!this.detailCopy.servicio.libre) {
            this.detailCopy.descripcion = this.detailCopy.servicio.nombre;
        }
        //Si es un servicio libre, recalcular importes de forma local
        else {
            let precio = this.servicioService.calcularPrecioLibre(this.detailCopy.precioUnitario, this.detailCopy.servicio.impuestoIncluido, this.detailCopy.impuestos);
            this.detailCopy.precioNeto = precio.precioNeto;
            this.detailCopy.impuestos = precio.impuestos;
            this.detailCopy.descuentos = precio.descuentos;
        }

        this.detailCopy.importe = this.detailCopy.precioNeto * this.detailCopy.cantidad;        
        this.savingItem.emit(this.detailCopy);
    }

    onServicioChanged(servicio: ServicioInfo) {
        if (servicio) {
            this.detailCopy.codigo = servicio.code;
            this.detailCopy.servicio = servicio;
            this.detailCopy.idEstructuraComercial = servicio.idEstCom;
            this.detailCopy.idUnidadMedida = servicio.idUnidadMedida;
            this.detailCopy.descripcion = servicio.nombre;
            //Hacer requerido el campo de descripción cuando el servicio sea de concepto libre
            if (this.detailCopy.servicio.libre) {
                this.form.controls.descripcion.setValidators(Validators.required);
            }
            else {
                this.form.controls.descripcion.setValidators(null);
            }
            this.form.controls.descripcion.updateValueAndValidity();
            //Cargar información de precios del servicio
            this.servicioService.calcularPrecioCliente(this.detailCopy.idServicio, this.idCliente, this.context.app.defaultCompany.identity, this.idZonaVenta, 0)
                .then(precio => {                    
                    this.detailCopy.precioUnitario = precio.precioBase;
                    this.detailCopy.precioNeto = precio.precioNeto;
                    this.detailCopy.impuestos = precio.impuestos;
                    this.detailCopy.descuentos = precio.descuentos;                    
                    //
                    this.trasladosCollection.sourceCollection = precio.impuestos;
                    this.retencionesCollection.sourceCollection = precio.impuestos;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.detailCopy.servicio = new ServicioInfo();
        }
    }

    onEditCell(grid: FlexGrid, args: CellRangeEventArgs) {
        if ( !grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cancel = true;
        }
    }

    onFormatCell(grid: FlexGrid, args: FormatItemEventArgs) {
        if (args.panel.cellType == CellType.Cell && !grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cell.style.backgroundColor = '#C0C0C0';
        }
    }
}
