﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';

// Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem, FileLink } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, ConditionValueType } from 'framework3/search';
import { MenuItem, DownloadPopupComponent, ContentViewerComponent } from 'framework3/controls';
//Pixkua
import { PolizaInfo, ComprobanteFiscalService, PolizaPerfilListComponent, PolizaPerfilItem, PolizaPerfilService, PolizaService } from '../../accounting';
import { CobranzaComponent, CobranzaFormaPagoDetail } from '../../credit';

//Locales
import { VentaInfo, VentaItem } from '../models/venta';
import { VentaDetail } from '../models/venta-detail';
import { VentaService } from '../services/venta.service';
import { EmpresaService } from '../services/empresa.service';

@Component({
    selector: 'sales-ventas-manager',
    templateUrl: 'ventas.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => VentasManager)
        },
        {
            provide: DocumentService, useExisting: VentaService
        }
    ]
})
export class VentasManager extends DocumentManager {
    ventas: VentaItem[];
    metodoCobranzaEfectivo: string = "";
    argsCurrentDocument: WorkflowEventArgs;
    idOrigen: number = 0;
    titleContentView: string = "";
    fileNameContentView: string = "";
    currentPoliza: PolizaInfo = new PolizaInfo();

    @ViewChild('viewDialog') dlgPolizaViewer: ModalDirective;
    @ViewChild('dlgCobranza') dlgCobranza: CobranzaComponent;
    @ViewChild(DownloadPopupComponent) downloadPopup: DownloadPopupComponent;
    @ViewChild(PolizaPerfilListComponent) dlgPolizaPerfil: PolizaPerfilListComponent;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private ventaService: VentaService, private empresaService: EmpresaService, private polizaSvc: PolizaService) {
        super(context, router, route);

        this.route.params.forEach((params: Params) => {
            this.idOrigen = params['idOrigen'] ? +params['idOrigen'] : 0;
        });
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();

        this.empresaService.getDetailFromCache(this.context.app.defaultCompany.identity)
            .then(empresa => {
                this.metodoCobranzaEfectivo = empresa.efectivoMetodoCobranza;
            })
            .catch(error => this.handleError(error));
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        //Si se especificó un origen, agregarlo como condición
        if (this.idOrigen > 0) {
            let conditionOrigen: SearchCondition = { field: 'IdOrigen', operator: '=', valueType: ConditionValueType.INT, value: this.idOrigen };
            request.conditions.push(conditionOrigen);
        }
        //
        this.ventaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.ventas = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['new', {idOrigen: this.idOrigen}], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.ventaService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {        
        this.argsCurrentDocument = args;
        let venta = this.ventas.find(doc => doc.id == args.document.id)
        if (args.step.tag == 'APPLY' && venta.condicionesPago == "E" && this.metodoCobranzaEfectivo == "A") {            
            this.dlgCobranza.show(venta.total, venta.idDeudor);
        }
        else {
            this.context.app.showSpinner();
            this.ventaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
                .then(_ => {
                    this.context.app.hideSpinner();
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ventaService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'PDF') {
            this.context.app.showSpinner();
            this.ventaService.generarPdfFactura(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    window.open('/file-server/' + file.uri, '_blank');
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "DOWNLOAD") {
            this.context.app.showSpinner();
            //Generar PDF
            this.ventaService.generarPdfFactura(this.manager.currentDocument.id)
                .then(pdf => {
                    //Descargar xml
                    this.ventaService.getXmlFactura(this.manager.currentDocument.id)
                        .then(xml => {
                            this.context.app.hideSpinner();
                            let links: FileLink[] = [];
                            links.push(pdf);
                            links.push(xml);
                            this.downloadPopup.show(links);
                        })
                        .catch(error => this.handleError(error));

                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "CONTABILIZAR") {
            this.dlgPolizaPerfil.show();
        }
        else if (option.name == "SUSTITUIR") {
            this.context.app.showSpinner();
            this.ventaService.regenerarFactura(this.manager.currentDocument.id)
                .then(report => {
                    this.context.app.hideSpinner();
                    this.showAlert('El comprobante fiscal ha sido sustituido con éxito.', 'success');
                }
                )
                .catch(error => this.handleError(error));
        }
        else if (option.name == "IMPRIMIR") {
            this.context.app.showSpinner();
            this.ventaService.imprimirFormatoVenta(this.manager.currentDocument.id)
                .then(report => {
                    this.context.app.hideSpinner();
                    this.titleContentView = "Ped_" + this.manager.currentDocument.id + ".pdf";
                    this.fileNameContentView = "Ped_" + this.manager.currentDocument.id + ".pdf";
                    this.reportViewer.show("data:application/pdf; base64, " + report);
                }
                )
                .catch(error => this.handleError(error));
        }
    }

    cobroAceptado(formasPagoCobro: CobranzaFormaPagoDetail[]) {
        let cobro = { formasPago: formasPagoCobro }
        this.context.app.showSpinner();
        this.ventaService.nextStep(this.argsCurrentDocument.document.id, this.argsCurrentDocument.step.idWorkflowStep, cobro)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
                this.dlgCobranza.clear();
            })
            .catch(error => this.handleError(error));
    }


    onSelectedPolizaPerfil(item: PolizaPerfilItem) {
        this.context.app.showSpinner();
        this.polizaSvc.loadPolizaByDocumento(this.manager.currentDocument.id, item.id, this.context.app.defaultSite.identity)
            .then(result => {
                this.currentPoliza = result;
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.config.backdrop = false;
                this.dlgPolizaViewer.show();
            }).catch(error => this.handleError(error));
    }

    onClickGuardar() {
        this.context.app.showSpinner();
        if (this.currentPoliza.idPolizaOrigen == 0) this.currentPoliza.idPolizaOrigen = 1 //Manual
        this.polizaSvc.update(this.currentPoliza)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.hide();
                this.manager.showAlert('La poliza ha sido registrada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }
}

