﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
import { TipoAplicacion, TipoFactor, ImpuestoDetailInfo } from '../../accounting';
//Locales
import { ServicioInfo } from '../models/servicio'
import { ServicioListaPreciosItem } from '../models/servicio-lista-precios';
import { ServicioGrupoDescuentoItem } from '../models/servicio-grupo-descuento';
import { PrecioInfo } from '../models/precio';
import { DescuentosInfo } from '../models/descuentos';


@Injectable()
export class ServicioService extends CatalogService<ServicioInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/ventas/servicios');
    }

    create(): Promise<ServicioInfo> {
        return Promise.resolve(new ServicioInfo());
    }

    //Lista de Precios
    loadServicioListaPrecios(idEmpresa: number, idServicio: number): Promise<ServicioListaPreciosItem[]> {
        let params = new HttpParams()
        params.append("idEmpresa", idEmpresa.toString());
        params.append("idServicio", idServicio.toString());
        return this.apiService.get<ServicioListaPreciosItem[]>("load-servicio-lista-precios", params)
            .toPromise();
    }

    updateServicioListaPrecios(data: ServicioListaPreciosItem[], idServicio: number): Promise<any> {
        let params = new HttpParams()
        params.append("idServicio", idServicio.toString());
        return this.apiService.post<any>('update-servicio-lista-precios', params, data)
            .toPromise();
    }

    //Grupo de Descuentos

    loadServicioGrupoDescuento(idEmpresa: number, idServicio: number): Promise<ServicioGrupoDescuentoItem[]> {
        let params = new HttpParams()
        params.append("idEmpresa", idEmpresa.toString());
        params.append("idServicio", idServicio.toString());
        return this.apiService.get<ServicioGrupoDescuentoItem[]>("load-servicio-grupo-descuento", params)
            .toPromise();
    }

    updateServicioGrupoDescuento(data: ServicioGrupoDescuentoItem[], idServicio: number): Promise<any> {
        let params = new HttpParams()
        params.append("idServicio", idServicio.toString());
        return this.apiService.post<any>('update-servicio-grupo-descuento', params, data)
            .toPromise();
    }

    calcularPrecio(idServicio: number, idListaPrecios: number, idGrupoDescuentos: number, idEmpresa: number, idZonaVenta: number, precioBruto: number): Promise<PrecioInfo> {
        let params = new HttpParams()
            .set("idServicio", idServicio.toString())
            .set("idListaPrecios", idListaPrecios.toString())
            .set("idGrupoDescuentos", idGrupoDescuentos.toString())
            .set("idEmpresa", idEmpresa.toString())
            .set("idZonaVenta", idZonaVenta.toString())
            .set("precioBruto", precioBruto.toString());            
        return this.apiService.get<PrecioInfo>('precio', params)
            .toPromise();
    }

    calcularPrecioCliente(idServicio: number, idCliente: number, idEmpresa: number, idZonaVenta: number, precioBruto: number): Promise<PrecioInfo> {
        let params = new HttpParams()
            .set("idServicio", idServicio.toString())
            .set("idCliente", idCliente.toString())
            .set("idEmpresa", idEmpresa.toString())
            .set("idZonaVenta", idZonaVenta.toString())
            .set("precioBruto", precioBruto.toString());
        return this.apiService.get<PrecioInfo>('precio-cliente', params)
            .toPromise();
    }

    calcularPrecioLibre(precioBase: number, impuestosIncluidos: boolean, impuestos: ImpuestoDetailInfo[]): PrecioInfo {
        let precio = new PrecioInfo();
        if (impuestosIncluidos) {
            precio.precioBase = this.calcularPrecioBase(precioBase, impuestos);
        }
        else {
            precio.precioBase = precioBase;
        }        
        //Recalcular importes de impuestos
        impuestos.forEach(impuesto => {            
            impuesto.importe = impuesto.base * impuesto.tasaOCuota;
        });        
        precio.impuestos = impuestos;
        //
        let totalTraslados = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.TRASLADO).forEach(i => totalTraslados += i.importe);
        //
        let totalRetenciones = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.RETENCION).forEach(i => totalRetenciones += i.importe);
        //
        precio.precioNeto = precio.precioBase + totalTraslados - totalRetenciones;
        return precio;
    }

    calcularPrecioBase(precioNeto: number, impuestos: ImpuestoDetailInfo[]): number {
        //Sumar las tasas de traslado
        let tasasTraslados: number = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.TRASLADO && i.tipoFactor == TipoFactor.TASA).forEach(i => tasasTraslados += i.tasaOCuota);
        //Sumar las cuotas de traslado
        let cuotasTraslados: number = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.TRASLADO && i.tipoFactor == TipoFactor.CUOTA).forEach(i => cuotasTraslados += i.tasaOCuota * i.base);

        //Sumar las tasas de las retenciones
        let tasasRetenciones: number = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.RETENCION && i.tipoFactor == TipoFactor.TASA).forEach(i => tasasRetenciones += i.tasaOCuota);

        //Sumar las cuotas de las retenciones
        let cuotasRetenciones: number = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.RETENCION && i.tipoFactor == TipoFactor.CUOTA).forEach(i => cuotasRetenciones += i.tasaOCuota * i.base);

        let precioBase = (precioNeto - cuotasTraslados + cuotasRetenciones) / (1 + (tasasTraslados - tasasRetenciones));

        return Math.round(precioBase * 100) / 100;
    }
}

