﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, EditableListComponent, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ConceptoService} from '../services/concepto.service';
import { ConceptoInfo, ConceptoItem} from '../models/concepto';
//Contabilidad
import { ImpuestoAsignacionInfo, ImpuestosAsignacionEditor } from '../../accounting';


@Component({
    selector: 'tesoreria-concepto-editor',
    templateUrl: './conceptos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ConceptosEditor)
        },
        {
            provide: CatalogService, useExisting: ConceptoService
        }
    ]
})
export class ConceptosEditor extends CatalogEditor {
    concepto: ConceptoInfo = new ConceptoInfo();        
    //
    impuestosCollection = new TrackableCollection();
    @ViewChild('impuestosList') impuestosList: EditableListComponent;
    @ViewChild(ImpuestosAsignacionEditor) impuestoEditor: ImpuestosAsignacionEditor;


    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private conceptoService: ConceptoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        this.impuestosCollection.sourceCollection = this.concepto.impuestos;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.conceptoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.concepto = info;    
                this.impuestosCollection.sourceCollection = this.concepto.impuestos;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.conceptoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.concepto = info;
                this.impuestosCollection.sourceCollection = this.concepto.impuestos;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.conceptoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.concepto = info;          
                this.impuestosCollection.sourceCollection = this.concepto.impuestos;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.conceptoService.update(this.concepto)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.concepto.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.concepto = info;
            })
            .catch(error => this.handleError(error));        
    }

    ///////////// Metodos generales //////////////////

    //onMenuItemClick(item: MenuItem) {
    //    if (item.name == 'LP') {
    //        this.router.navigate(['sales/articulo-listas-precio/' + this.process.id + '/' + this.concepto.identity.toString()]);
    //    } else if (item.name == 'GD') {
    //        this.router.navigate(['sales/articulo-grupo-descuento/' + this.process.id + '/' + this.concepto.identity.toString()]);
    //    }
    //}

    ///////////// IMPUESTOS /////////////////////////

    onImpuestosToolbar(button: ToolbarButton) {        
            this.impuestoEditor.show();        
    }

    onRemoveItem(impuesto: ImpuestoAsignacionInfo) {
        this.impuestosCollection.remove(impuesto);
    }

    onImpuestoUpdated(impuesto: ImpuestoAsignacionInfo) {
        let newImpuestos = this.concepto.impuestos.slice();        
        newImpuestos.push(impuesto);
        this.concepto.impuestos = newImpuestos;
        this.impuestosCollection.sourceCollection = newImpuestos;
    }

}