﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo, PolicyInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { ContrareciboInfo, ContrareciboItem } from '../models/contrarecibo';
import { ContrareciboDetail } from '../models/contrarecibo-detail';
import { ContrareciboService } from '../services/contrarecibo.service';
import { ContrareciboDetailEditor } from './parts/contrarecibos-tit-detail.editor';
import { CuentasPorPagarListComponent } from '../dialogs/cuenta-por-pagar-list.component';

import { CuentaPorPagarItem } from '../models/cuenta-por-pagar'
import { exec } from 'child_process';
import { BeneficiarioItem } from '../models';
import { DivisaItem, DivisaInfo } from '../../accounting';

@Component({
    selector: 'tesoreria-contrarecibos-tit-editor',
    templateUrl: './contrarecibos-tit.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ContrarecibosTitEditor)
        },
        {
            provide: DocumentService, useExisting: ContrareciboService
        }
    ]
})
export class ContrarecibosTitEditor extends DocumentEditor {
    contrarecibo = new ContrareciboInfo();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: ContrareciboDetailEditor;
    @ViewChild('dlgCuentasPorPagar') dlgCuentasPorPagar: CuentasPorPagarListComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private contrareciboService: ContrareciboService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idCaja': [0],
            'idDivisa': [null, AztecaValidators.selected],
            'idBeneficiario': [0, AztecaValidators.selected],
            'vencimiento': ['']
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.contrareciboService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.contrarecibo = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.contrareciboService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.contrarecibo = doc;
                this.detailsEditor.clientIndexStart = this.contrarecibo.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.contrareciboService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.contrarecibo = doc;
                this.detailsEditor.clientIndexStart = this.contrarecibo.details.length;
                if (this.contrarecibo.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        if (this.contrarecibo.details.length > 0) {
            this.context.app.showSpinner();
            this.contrareciboService.update(this.contrarecibo)
                .then(updatedDoc => {
                    this.context.app.hideSpinner();
                    //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                    this.reset();
                    if (!args.closeEditor && this.contrarecibo.identity == '') {
                        this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                    }
                    else {
                        if (args.closeEditor)
                            this.router.navigate(['../'], { relativeTo: this.route });
                        else
                            this.showAlert('La información ha sido actualizada con éxito', 'success');
                    }
                    this.contrarecibo = updatedDoc;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.showAlert('No hay cuentas por pagar relacionadas', 'danger');
        }
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.contrareciboService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.contrareciboService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.contrarecibo.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    onDivisaChanged(args: DivisaInfo) {
        if (args != undefined) {
            this.contrarecibo.idDivisa = args.identity;
            this.documentForm.updateValueAndValidity();
        }
    }

    ///////////// DETALLE DEL DOCUMENTO /////////////////////////
    onBeneficiarioSelected(beneficiario: BeneficiarioItem) {
        if (this.contrarecibo.identity.length == 0)
            this.contrarecibo.details = [];
        else {
            this.contrarecibo.details.forEach(detail => detail.state = 'DELETED');
        }
    }

    onAddingDetail() {
        this.dlgCuentasPorPagar.show();
    }

    onEditingDetail(detail: ContrareciboDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: ContrareciboDetail) {
        this.detailsList.deleteItem(detail);

    }

    onListItemsUpdated(updatedDetails: ContrareciboDetail[]) {
        this.contrarecibo.details = updatedDetails;
    }

    onSavingDetail(detail: ContrareciboDetail) {
        this.detailsList.updateItem(detail);
    }

    /////////// Details

    onCuentaPorPagarSelected(args: CuentaPorPagarItem) {
        if (args) {
            var exist = false;
            this.contrarecibo.details.forEach(item => {
                if (item.idCuentaPorPagarTit == args.id && item.state != 'DELETED') {
                    exist = true;
                }
            });
            if (!exist) {
                let d = new Date();
                let detail = new ContrareciboDetail();
                detail._clientID = parseInt(d.getHours() + "" + d.getMinutes() + "" + d.getSeconds());
                detail.idCuentaPorPagarTit = args.id;
                detail.fechaCuentaPorPagar = args.fechaSys;
                detail.subTotal = args.subTotal;
                detail.impuestosTrasladados = args.impuestosTrasladados;
                detail.impuestosRetenidos = args.impuestosRetenidos;
                detail.total = args.total;
                this.onSavingDetail(detail);
            }
        }
    }

    selectedItemsChanged(args: CuentaPorPagarItem[]) {
        if (args) {
            args.forEach(item => {
                this.onCuentaPorPagarSelected(item);
            });
        }
    }

    applyPolicy(policy: PolicyInfo) {
        console.log('Aplicar politica');
    }

}

