﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { CuentaPorPagarInfo, CuentaPorPagarItem } from '../models/cuenta-por-pagar';
import { CuentaPorPagarDetail } from '../models/cuenta-por-pagar-detail';
import { CuentaPorPagarService } from '../services/cuenta-por-pagar.service';
import { CuentaPorPagarDetailEditor } from './parts/cuentas-por-pagar-detail.editor';
import { ImportesInfo } from '../../accounting/models/importes';
import { DivisaService } from '../../accounting/services/divisa.service';
import * as moment from 'moment';

import { BeneficiarioInfo } from '../models';
import { OperacionTipoItem } from '../models/operacion-tipo';
import { OperacionTipoService } from '../services/operacion-tipo.service';

@Component({
    selector: 'tesoreria-cuentas-por-pagar-editor',
    templateUrl: './cuentas-por-pagar.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CuentasPorPagarEditor)
        },
        {
            provide: DocumentService, useExisting: CuentaPorPagarService
        }
    ]
})
export class CuentasPorPagarEditor extends DocumentEditor {
    cuentaPorPagar = new CuentaPorPagarInfo();
    operacionesTipos: OperacionTipoItem[] = [];
    
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: CuentaPorPagarDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private cuentaPorPagarService: CuentaPorPagarService,
        private operacionTipoService: OperacionTipoService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idBeneficiario': [0, Validators.required],
            'idOperacionTipo': [0, Validators.required],
            'idDivisa': [0, Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.loadOperacionesTipos();
    }

    onCreatingDocument() {
        this.cuentaPorPagar.fechaDocumentoRef = new Date(1901, 1, 1);
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.cuentaPorPagarService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cuentaPorPagar = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.cuentaPorPagarService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cuentaPorPagar = doc;
                this.detailsEditor.clientIndexStart = this.cuentaPorPagar.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.cuentaPorPagarService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cuentaPorPagar = doc;
                this.detailsEditor.clientIndexStart = this.cuentaPorPagar.details.length;
                if (this.cuentaPorPagar.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.cuentaPorPagarService.update(this.cuentaPorPagar)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.cuentaPorPagar.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.cuentaPorPagar = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.cuentaPorPagarService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.cuentaPorPagarService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.cuentaPorPagar.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new CuentaPorPagarDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: CuentaPorPagarDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: CuentaPorPagarDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: CuentaPorPagarDetail[]) {
        this.cuentaPorPagar.details = updatedDetails;
    }

    onSavingDetail(detail: CuentaPorPagarDetail) {
        this.detailsList.updateItem(detail);
    }

    ///////////// METODOS GENERALES /////////////////////////
    get importes() {
        return this.importesDesglosados();
    }

    importesDesglosados(): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        let elementos = this.cuentaPorPagar.details.filter(item => item.state != 'DELETED');

        elementos.forEach(item => {
            desglose.subtotal += item.costoUnitario * item.cantidad;
            //desglose.descuentos += (item.descuentos.descuento1 + item.descuentos.descuento2 + item.descuentos.descuento3 + item.descuentos.descuento4 + item.descuentos.descuento5) * item.cantidad;
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "T").length > 0)
                desglose.impuestosTrasladados += item.impuestos.filter(tras => tras.tipoAplicacion == "T").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "R").length > 0)
                desglose.impuestosRetenidos += item.impuestos.filter(tras => tras.tipoAplicacion == "R").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
        });

        desglose.total = desglose.subtotal - (desglose.descuentos ? desglose.descuentos:0) + desglose.impuestosTrasladados - desglose.impuestosRetenidos;

        return desglose;
    }    

    onBeneficiarioChanged(beneficiario: BeneficiarioInfo) {
        if (this.cuentaPorPagar.identity == "") {
            this.cuentaPorPagar.details = [];
            this.detailsList.refresh();
            this.cuentaPorPagar.vencimiento = moment(new Date).add(beneficiario.diasCredito, "days").toDate();
        }
    }

    onDivisaChanged(divisa: any) {

    }

    loadOperacionesTipos() {
        this.operacionTipoService.loadListByUser(this.context.app.defaultSite.identity)
            .then(operaciones => {
                this.operacionesTipos = operaciones.filter(item=> item.afectador=='C' && item.interno==false);
            })
            .catch(error => this.showAlert(error, 'danger'));
    }

}

