﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control, CollectionView } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo, FileLink } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
import { LoteImpresionInfo, LoteImpresionService, PrinterService } from 'framework3/printing';
//Locales
import { PagoInfo, PagoItem } from '../models/pago';
import { PagoDetail } from '../models/pago-detail';
import { PagoService } from '../services/pago.service';
import { PagoDetailEditor } from './parts/pagos-detail.editor';
import { CajaService } from '../services/caja.service';
import { CajaItem } from '../models/caja';
import { PagoTipoService } from '../services/pago-tipo.service'
import { PagoTipoItem } from '../models/pago-tipo'
import { DivisaService } from '../../accounting/services/divisa.service';
import { DivisaInfo } from '../../accounting/models/divisa';
import { DivisasSelectorComponent } from '../../accounting/controls/divisas-selector.component';
import { EmpresaService } from '../../accounting/services/empresa.service';
import { EmpresaInfo } from '../../accounting/models/empresa';
import { ModalDirective } from 'ngx-bootstrap';

@Component({
    selector: 'tesoreria-pagos-editor',
    templateUrl: './pagos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PagosEditor)
        },
        {
            provide: DocumentService, useExisting: PagoService
        }
    ]
})
export class PagosEditor extends DocumentEditor {
    pago = new PagoInfo();
    cajas: CajaItem[] = [];
    pagosTipo: PagoTipoItem[] = [];
    acumulado: number = 0;
    idDivisaDefault: number = 0;
    tipoCambio: string;
    cuentasXPagar: CollectionView = new CollectionView();
    intervalId: any;
    establecerMonto: boolean = false;


    formato: string = "C2";
    decimales: number = 2;

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: PagoDetailEditor;
    @ViewChild('divisasSelector') divisasSelector: DivisasSelectorComponent;
    @ViewChild('dialogDecimales') dialogDecimales: ModalDirective;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private cajaService: CajaService,
        private pagoService: PagoService,
        private divisaService: DivisaService,
        private pagoTipoService: PagoTipoService,
        private empresaService: EmpresaService,
        private loteImpresionService: LoteImpresionService,
        private printerService: PrinterService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'caja': ['', Validators.required],
            'idBeneficiario': [0, Validators.required],
            'idPagoTipo': [0, Validators.required],
            'idChequera': [0, Validators.required],
            'idDivisa': [0, Validators.required],
            'observaciones': ['', Validators.required],
            'monto': ['0', _ => this.validateMontos()],
            'acumulado': ['0', _ => this.validateMontos2()],
            'establecerMonto':[false]
        });
    }



    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.loadCajas();
        this.loadPagosTipo();
        this.getDivisaDefault(false);
    }

    onCreatingDocument() {
        this.getDivisaDefault(true);
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.pagoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.pago = doc;
                this.cuentasXPagar.sourceCollection = this.pago.details;
                this.calcularAcumulado();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.pagoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.pago = doc;
                this.detailsEditor.clientIndexStart = this.pago.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.pagoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.pago = doc;
                this.cuentasXPagar.sourceCollection = this.pago.details;
                this.calcularAcumulado();
                this.detailsEditor.clientIndexStart = this.pago.details.length;
                if (this.pago.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();

        this.pagoService.update(this.pago)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.pago.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.pago = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.pagoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.pagoService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.pago.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        //this.detailsEditor.detail = new PagoDetail();
        //this.detailsEditor.show();
    }

    onEditingDetail(detail: PagoDetail) {
        //this.detailsEditor.detail = detail;
        //this.detailsEditor.show();
    }

    onDeletingDetail(detail: PagoDetail) {
        //this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: PagoDetail[]) {
        this.pago.details = updatedDetails;
    }

    onSavingDetail(detail: PagoDetail) {
        this.detailsList.updateItem(detail);
    }

    //////////////////////PERSONALIZACION//////////////////////

    loadCajas() {
        this.cajaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cajas => this.cajas = cajas)
            .catch(error => this.handleError(error));
    }

    loadPagosTipo() {
        this.pagoTipoService.loadList(this.context.app.defaultSite.identity)
            .then(pagosT => this.pagosTipo = pagosT)
            .catch(error => this.handleError(error));
    }


    validateMontos() {
        if (Math.abs(this.pago.total - this.acumulado) <= 0.001 && this.acumulado > 0)
            return null;
        else
            return 'No coincide el monto deseado con el monto acumulado.';
    }

    validateMontos2() {
        if (Math.abs(this.pago.total - this.acumulado) <= 0.001 && this.acumulado > 0)
            return null;
        else
            return 'No coincide el monto deseado con el monto acumulado.';
    }

    onDetailsLoaded(details: PagoDetail[]) {
        this.acumulado = 0;

        let detailsOrder: PagoDetail[] = details;
        this.detailsList.listItems = detailsOrder;
        this.pago.details = detailsOrder;
        this.cuentasXPagar.sourceCollection = detailsOrder;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.detailsEditor.detail = new PagoDetail();
            this.detailsEditor.show();
        }
        else if (button.name == 'DECIMALES') {
            this.dialogDecimales.config.backdrop = false;
            this.dialogDecimales.show();
        }
        else if (button.name == 'IMPRIMIR') {

            var pagosToCheque: string[] = [];

            this.context.app.showSpinner();

            pagosToCheque[0] = this.pago.identity;



            this.pagoService.getLoteCheques(pagosToCheque)
                .then(lote => {
                    this.intervalId = setInterval(_ => this.checaStatus(lote.id), 5000);
                })
                .catch(error => this.handleError(error));

        }

    }

    getStatusColor(vencimiento: Date): string {

        let venc = new Date(vencimiento);
        let today = new Date();
        if (today < venc)
            return '#298A08';
        else
            return 'orangered';
    }

    cellEditEndingHandler(sender: any, e: any) {
        let cxp: any = sender.rows[e.row].dataItem;

        var flex = sender,
            oldVal = flex.getCellData(e.row, e.col),
            newVal = flex.activeEditor.value;

        setTimeout(_ => this.calcularAcumulado());

        if (newVal > cxp.saldo) {
            this.showAlert("No puede abonar un importe mayor al saldo del documento", "danger");
            e.cancel = true;
            cxp.importe = cxp.saldo;
        }


        if (!isNaN(newVal)) // returns true if NaN, otherwise false            
            cxp.saldoFin = cxp.saldo - newVal;
        else  //proviene del keydown ya que activaron el beginEdit
            e.cancel = true;

        cxp.state = "MODIFIED";

    }

    calcularAcumulado() {
        this.acumulado = this.pago.details
            .map(c => c.importe)
            .reduce((sum, current) => sum + current);

        this.documentForm.get('monto').updateValueAndValidity();

        if (!this.establecerMonto) this.pago.total = this.acumulado;

    }

    onCellEditEnded(flex: any, e: any) {
        this.cuentasXPagar.refresh();
        let Total: number = this.pago.total;
        this.pago.total = Total;
        setTimeout(_ => this.calcularAcumulado());
    }

    onChequeraSelected() {

    }

    onBeneficiarioSelected(args: any) {

        if (this.pago.status == DocumentStatus.DRAFT) {
            this.pago.details = [];
            this.cuentasXPagar.sourceCollection = [];
        }
    }

    keydown(grid, args, cell) {
        if (args.keyCode == 13) {

            let cxp: any = grid.rows[grid.selection.row].dataItem;

            if (this.acumulado == undefined)
                this.acumulado == 0;

            let disponible: number = this.pago.total - this.acumulado
            if (disponible < 0 || disponible == undefined || isNaN(disponible))
                disponible = 0;

            if (disponible > cxp.saldo || !this.establecerMonto)
                cxp.importe = cxp.saldo;
            else
                cxp.importe = disponible;


            cxp.saldoFin = cxp.saldo - cxp.importe;
            cxp.state = "MODIFIED";

            //let total: number = this.cobranza.total+1;
            //this.cobranza.total = total-1;

            //this.documentForm.get('monto').markAsPristine();
            //this.documentForm.get('monto').markAsPending();
            this.detailsList.refresh();

            this.documentForm.get('monto').updateValueAndValidity();

            setTimeout(_ => this.calcularAcumulado());
        }
    } 


    getDivisaDefault(creating: boolean) {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(r => {
                this.idDivisaDefault = r.identity;
                if (creating) { this.pago.idDivisa = r.identity;}
                
            })
            .catch(error => this.handleError(error));

    }

    onDivisaSelected(args: any) {
        if (this.pago.status == DocumentStatus.DRAFT) {
            this.pago.details = [];
            this.cuentasXPagar.sourceCollection = [];
        }
        if (this.pago.idDivisa == this.idDivisaDefault) {
            this.pago.tipoCambio = 1;
            this.tipoCambio = this.pago.tipoCambio.toString();
        }
        else {
            this.divisaService.loadTiposCambio(this.context.app.defaultCompany.identity)
                .then(tipos => {
                    let tCambio = tipos.find(search => search.idDivisaOrigen == this.pago.idDivisa  && search.idDivisaDestino == this.idDivisaDefault);
                    if (tCambio == undefined) {
                        this.showAlert("No está definido el tipo de cambio para la divisa seleccionada", "danger");
                        this.tipoCambio = null;
                        this.pago.idDivisa = null;
                        this.pago.tipoCambio = 0;
                    }
                    else {
                        this.empresaService.getDetailFromCache(this.context.app.defaultCompany.identity)
                            .then(emp => {
                                if (emp.tipoCambio == 'O') {
                                    this.pago.tipoCambio = tCambio.precioOtro;
                                    this.tipoCambio = tCambio.precioOtro.toString();
                                }
                                else if (emp.tipoCambio == 'V') {
                                    this.pago.tipoCambio = tCambio.precioVenta;
                                    this.tipoCambio = tCambio.precioVenta.toString();
                                }
                                else if (emp.tipoCambio == 'C') {
                                    this.pago.tipoCambio = tCambio.precioCompra;
                                    this.tipoCambio = tCambio.precioCompra.toString();
                                }
                            })
                    }
                });
        }
    }

    assignFormat() {
        this.formato = 'C' + this.decimales.toString();
        this.dialogDecimales.hide();
        this.calcularAcumulado();

    }

    checaStatus(idLote) {
        //this.context.app.hideSpinner();
        this.loteImpresionService.getStatus(idLote)
            .then(result => {

                if (result == 'C') {
                    this.loteImpresionService.getFiles(idLote)
                        .then(files => {
                            this.context.app.hideSpinner();
                            let links: FileLink[] = [];
                            let Cheques: FileLink = new FileLink;
                            Cheques.uri = files[0];
                            Cheques.fileName = files[0];
                            links.push(Cheques);
                            //this.printerService.printPdfFile('/file-server/' + files[0], 'EPSON FX-2190 Ver 2.0');
                            this.printerService.printTextFile('/file-server/' + files[0]);
                            //this.downloadPopup.show(links);
                            clearInterval(this.intervalId);
                        })

                }
                /*
                if (result == 'R') {
                    this.showAlert('Generando bloque de impresión, el proceso puede tardar varios minutos', 'success');
                }
                */
                if (result == 'E') {
                    clearInterval(this.intervalId);
                    this.context.app.hideSpinner();
                    this.showAlert('Error al obtener el bloque de impresión de cheques', 'success');
                }


            })
            .catch(error => this.handleError(error))
    }

    onChangeEstablecerMonto(args: any) {
        Control.invalidateAll();
    }


}

